"""
    PyWiew --- Image viewer/editor using wxPython/PIL
    Version: 1.2

    Load/Edit images using wxPython API and the Python Imaging Library(PIL). 
    Requirements: Python 2.1+, wxPython and PIL 1.1.3(optional, needed for most of the filters)
    Author: Anand B Pillai <abpillai@lycos.com>
    Copyright (C) 2003-2004 Anand B Pillai

    PyWiew is released under GNU LGPL. For licensing information,
    visit http://www.gnu.org/copyleft/lgpl.html.

    The latest version should be available at the following location:

    <URL: http://members.fortunecity.com/anandpillai/python/pyWiew.pyw.html>

    The latest sources should be available at the following location:

    <URL: http://members.fortunecity.com/anandpillai/PyWiewsrc.zip>

    PyWiew homepage: http://members.fortunecity.com/anandpillai
    Python homepage: http://www.python.org 
    wxPython homepage: http://www.wxpython.org 
    PIL homepage: http://www.pythonware.com/products/pil

    Code:

    Modification History:

        Fri Feb 07 19:52:33 2003: Fixed many bugs. Smarter code
        for filter in file dialog box.
        Fri Feb 14 18:43:00 2003: Added 2 new filters which I wrote
        .One filter does a 3d-shading and other does a sketching.
        Mon Feb 17 19:27:30 2003: Fixed a bug in saving current image.
        . Added code in globals to recoginze wxImage file formats.
        Wed Feb 19 10:04:49 2003: Fixed an event handler problem.
            Moved file drag & drop handling to
            wxWindow class.
        Thu Feb 20 14:30:16 2003: Fixed a problem with saving of
            thumbnails. Destroy instance of
            confirmation dialog.

        Sat Feb 22 18:19:43 2003: Added a menu option for opening
            images directly from the internet.
            Modified the options class to set
            internet options on the tab-page.
            User can set proxies/proxy authori
            sation etc in the tab page.

       Wed Feb 26 12:41:34 2003: Added sizing code to resize the image
            when user resizes the frame. The earlier idea was to keep the
            image size and use scrollbars, but managing own scrolling seems
            a bit tough on the coding side :-).

       Thu Feb 27 16:49:26 2003: Fixed a problem with the timer/display
            thread.

       Mon Mar 03 11:18:49 2003: Set default zoom selection to 100%.
       Mon Mar 03 20:34:08 2003: Manage fitting of toolbar/statusbar in the
                                             program code since we cannot call Fit()
                                             anymore. Modified function Update() to do
                                             this.

      Tue Mar 04 18:37:53 2003: Modified config writing to write settings
                                            under separate registry keys. Obfuscate
                                            proxies also. Use lowercase strings for
                                            config entries.

      Wed Mar 05 19:51:57 2003: Fixed stupid code problems in SetImage.
                                Wrote separate functions for finding prev/next
                                images instead of traversing thru a filelist.
                                (Image loading is slow for big directories, since
                                time is taken to make file list!).

      Fri Mar 14 12:30:16 2003: Finally fixed scrolling problem :-O !. Derived
                                a class from wxScrolledWindow to provide automatic
                                scrolling. We paint images to this class's window instead
                                of a wxPanel as before. Added code for intelligent
                                scrolling. That is, scrollbars become visible only if
                                the window size in any direction becomes less than the
                                image size in that direction. Otherwise scrollbars are
                                invisible. Removed previous sizing code.

      Tue Mar 18 18:28:38 2003: Reverted back to sizing from scrolling. Because,
                                window capture requires proper device context which
                                is screwed up by using scrollbars.

                                Caption is set for new image/ clipboard.
                                Saving images does not change caption.
                                Cropping uses inbuilt function of wxImage instead
                                of device context.

    Mon Mar 24 21:45:13 2003: Experimental support for animation. Animation
                              code draws directly to device context to avoid
                              flicker. (No optimization set for device context.)

    Tue Mar 25 20:56:37 2003: Removed wxStaticBitmap as a holder for images.
                              We paint images directly on a wxPanel. This makes
                              app less heavy and animations are smoother.

    Wed Mar 26 17:58:45 2003: More bug fixes. Use an internal flag so that sizing
                              done by program code does not call the sizing window
                              callback function. (Use MyFit() instead of Fit() for
                              main window). Fixed a problem with copying image from
                              clipboard. Fixed problem with setting background color
                              of panel. Fixed a problem with Multi-image operations.

    Fri Mar 28 14:42:24 2003: Modified managing of clipboard. Open clipboard everytime
                              we need to use it, then close it after use.

    Fri Mar 28 15:21:43 2003: Code cleanup and modularization.
    Sat Mar 29 18:51:09 2003: Added printing support. Waah...!
    Tue Apr 01 17:43:38 2003: Fixed a problem with updating url list.
                              I was using the wrong variable.

    Fri Apr 11 15:11:15 2003: Fixed problem with saving of thumbnails.
    Fri Apr 11 16:05:28 2003: Fixed drag n drop proble. self.DragAcceptFiles line
                              keeps getting deleted ! :-(

    Tue Apr 15 19:13:20 2003: Rewrote code to handle rot/flip. I realized that rot/flip
                              are both interrelated so wrote one function to handle both.
                              Also rot/flip works on displayed image. Only zoom works
                              on original image. doCloseImage() does not reset image status.
                              Modified displaying of image. Always display _dispimg. Filters
                              make changes to display image and not original image. Fixed
                              other bugs in cropping. Added an Reopen Image file menu command.
                              
    Tue Apr 15 20:10:59 2003: Code cleanup. Replaced many wrong boolean expressions with
                              proper python boolean checks. Lots of small bug fixes. Fixed
							  problem with fitting image/window to screen.
    
"""

import sys, os
import tempfile
import random
import math
import time
import thread, threading
import shutil
import httplib
import urllib2
import cStringIO
import _winreg
import base64

# Separate import needed for module to work with py2exe
# You need to have PIL (Python Imaging Library) in your PYTHONPATH
import Image

# All plugins
import JpegImagePlugin    
import TgaImagePlugin
import PngImagePlugin
import GifImagePlugin
import PcxImagePlugin
import PpmImagePlugin
import BmpImagePlugin
import FliImagePlugin
import EpsImagePlugin
import DcxImagePlugin
import FpxImagePlugin
import ArgImagePlugin
import CurImagePlugin
import GbrImagePlugin
import IcoImagePlugin
import ImImagePlugin
import ImtImagePlugin
import IptcImagePlugin
import McIdasImagePlugin
import MicImagePlugin
import MspImagePlugin
import PcdImagePlugin
import PdfImagePlugin
import PixarImagePlugin
import PsdImagePlugin
import SgiImagePlugin
import SunImagePlugin
import TgaImagePlugin
import TiffImagePlugin
import WmfImagePlugin
import XVThumbImagePlugin
import XbmImagePlugin
import XpmImagePlugin

import pyWiewGlobals 
import pyWiewEvtHandler 
import pyWiewWidgets 
import pyWiewMiscWidgets
import pyWiewConfig
import pyWiewOptions
import pyWiewAnim

from wxPython.wx import *

class pyWiewPreviewFrame(wxPreviewFrame):
    """ Widget to perform a print out preview """

    def __init__(self, parent, preview, title, pos=wxDefaultPosition, size=wxDefaultSize):
        wxPreviewFrame.__init__(self, preview, parent, title, pos, size)
        # hide parent
        parent.Show(FALSE)
        EVT_CLOSE(self, self.OnCloseWindow)

    def OnCloseWindow(self, event):
        parent = self.GetParent()
        # show parent
        parent.Show(TRUE)
        parent.SetFocus()
        parent.Enable(TRUE)
        self.Show(FALSE)
        self.Destroy()
    
class pyWiewPrintOut(wxPrintout):
    """ Class to perform a printout of pyWiew bitmap """

    def __init__(self, panel, title="PyWiew printout"):
        wxPrintout.__init__(self, title)
        self._panel = panel
        pass

    def OnPrintPage(self, pgno):
        dc = self.GetDC()
        dc.SetDeviceOrigin(0, 0)
        dc.SetUserScale(1.0, 1.0)
        # get bitmap size
        bmap = self._panel.GetBitmap()
        bx, by = bmap.GetWidth(), bmap.GetHeight()
        # have 50 device units margin
        marginx, marginy = 50, 50
        # Add margin to graphic size
        bx += 2*marginx
        by += 2*marginy

        # Get size of dc in pixels
        dw, dh = dc.GetSizeTuple()
        # Calculate a suitable scaling factor
        scalex = float(dw/bx)
        scaley = float(dh/by)

        print 'Scale = ', scalex, scaley
        actualScale = pyWiewGlobals.Min(scalex, scaley)
        # Calculate position on dc for centering graphic
        posX = float((dw - (bx*actualScale))/2.0)
        posY = float((dh - (by*actualScale))/2.0)

        print 'Actual Scale = ', actualScale
        # Set scale and origin
        dc.SetUserScale(actualScale, actualScale)
        print 'Positions = ', posX, posY
        dc.SetDeviceOrigin(long(posX), long(posY))
        ppiScreenX, ppiScreenY = self.GetPPIScreen()
        ppiPrinterX, ppiPrinterY = self.GetPPIPrinter()
        scale = float(ppiPrinterX/ppiScreenX)
        dw, dh = dc.GetSizeTuple()
        pageW, pageH = self.GetPageSizePixels()
        overallScale = scale * float(dw/float(pageW))
        dc.SetUserScale(overallScale, overallScale)
        print 'Overallscale = ', overallScale
        dc.DrawBitmap(bmap, 0, 0, TRUE)
        return TRUE

    def HasPage(self, pgno):

        # Since we print only one image at a time
        return (pgno == 1)

    def OnBeginDocument(self, stpg, endpg):
        if not self.base_OnBeginDocument(stpg, endpg):
            return FALSE
        
        return TRUE

    def GetPageInfo(self):

        # this method in wxPython is different from
        # its wxWindows (C++) counterpart. It takes
        # no arguments and returns a 4 tuple of integers
        # The C++ function takes 4 arguments and returns
        # nothing. Just the inverse, eh ? ;-)
        
        minpg=1
        maxpg=1
        selpgto=1
        selpgfrm=1
        return (minpg, maxpg, selpgfrm, selpgto)


class pyWiewScroller(wxScrolledWindow):
    """ Canvas class of PyWiew, this class draws the images,
    and manages scrolling the image """

    def __init__(self, parent, id, pos=wxDefaultPosition, size=wxDefaultSize):
        wxScrolledWindow.__init__(self, parent, id, pos, size, wxHSCROLL | wxVSCROLL |  wxNO_FULL_REPAINT_ON_RESIZE)
        self.CenterOnParent()
        self.Fit()

    def SetPanel(self, panel):
        self._panel = panel

    def SetBestSize(self):
        self._panel.SetBestSize()
        self.Fit()
        
    def ManageSizing(self, event):
        """ Manage image sizing """

        ws, hs=self.GetSizeTuple()
        wb, hb=self._panel.GetSizeTuple()

        scunit=5
        scunitsx=int(float(wb)/5.0)
        scunitsy=int(float(hb)/5.0)
        if ws>wb and hs >hb:
            self.EnableScrolling(FALSE,FALSE)
            self.SetScrollbars(0, 0, 0, 0)
        if ws<wb and hs > hb:
            self.EnableScrolling(TRUE,FALSE)
            pos=self.GetScrollPos(wxHORIZONTAL)
            self.SetScrollbars(scunit, 0, scunitsx, 0, pos)
        if ws> wb and hs<hb:
            self.EnableScrolling(FALSE, TRUE)
            pos=self.GetScrollPos(wxVERTICAL)
            self.SetScrollbars(0, scunit, 0, scunitsy, 0, pos)
        if ws<wb and hs<hb:
            self.EnableScrolling(TRUE, TRUE)
            posx=self.GetScrollPos(wxHORIZONTAL)
            posy=self.GetScrollPos(wxVERTICAL)
            self.SetScrollbars(scunit, scunit, scunitsx, scunitsy, posx, posy)
    

class pyWiewPanel(wxPanel):

    def __init__(self, parent, ID, bgcolor):
        wxPanel.__init__(self, parent,ID, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxSIMPLE_BORDER)
        self.SetBackgroundColour(bgcolor)
        self._bmap=wxNullBitmap
        self._clipbmap=wxNullBitmap
        self._handler = None
        self._posn=(0,0)
        wxStaticLine(self, -5, wxPoint(0,5))
        self.CenterOnParent()

    def RegisterEventHandlers(self, handler):
        """ Register event handlers for this window """

        self._handler = handler
        EVT_MIDDLE_DOWN(self, handler.OnMouseEvent)
        EVT_LEFT_DOWN(self, handler.OnMouseEvent)
        EVT_LEFT_UP(self, handler.OnMouseEvent)
        EVT_MOTION(self, handler.OnMouseEvent)
        EVT_PAINT(self, self.doPaint)

    def PaintImage(self):
        """ Paint current bitmap to the gdi """

        dc=wxClientDC(self)
        color=self.GetBackgroundColour()
        dc.SetBrush( wxBrush(color, wxTRANSPARENT))
        if self._bmap.Ok():
            dc.DrawBitmap(self._bmap, self._posn[0], self._posn[1], true)
        else:
            dc.Clear()
        
    def doPaint(self, event):
        """ Manages paint events """
        
        if not self._bmap.Ok():
            return

        wxPaintDC(self)
        upd=wxRegionIterator(self.GetUpdateRegion())
        dc=wxClientDC(self)

        while upd:

            dc.BeginDrawing()
            dc.DrawBitmap(self._bmap, self._posn[0], self._posn[1], true)
            upd = upd.Next()
            dc.EndDrawing()

    def Clear(self):
        """ Clear the panel (dc) """

        self.SetBitmap(wxNullBitmap)
        dc=wxClientDC(self)
        dc.Clear()
        
    def Refresh(self):
        """ Refresh the panel """

        # call PaintImage to repaint the panel
        self.PaintImage()

    def SetBackground(self, color):
        """ Set background color for the panel """
        
        dc=wxClientDC(self)
        brush=wxBrush(color, wxSOLID)
        dc.SetBackground(brush)
        dc.Clear()

        # also set background for window
        self.SetBackgroundColour(color)

    def SetPosition(self, posn):
        """ Set position of bitmap in panel """
        self._posn=posn
        
    def SetBitmap(self, bmap):
        """ Set current bitmap for the panel """
        self._bmap = bmap

    def SetBestSize(self):
        """ Set the best size of the panel
        This is defned as the size of the current bitmap """
        
        w, h = self._bmap.GetWidth(), self._bmap.GetHeight()
        self.SetSize(wxSize(w, h))

    def GetBestSize(self):
        """ Get best size of panel. Best size is defined
        as size of the current bitmap """
        
        w, h = self._bmap.GetWidth(), self._bmap.GetHeight()        
        return wxSize(w, h)
    
    def SetClip(self, bmap):
        """ Set the clipboard bitmap """
        self._clipbmap = bmap

    def GetClip(self):
        """ Return the current clipboard bitmap """
        return self._clipbmap
    
    def GetBitmap(self):
        """ Return the current bitmap """
        return self._bmap

    def CopyBitmap(self, box):
        """ Copy portion of current bitmap using the selected box """

        w,h =   box[2] - box[0], box[3] - box[1]
        # Client DC makes sure that we dont overdraw on the main window's widgets
        dc=wxClientDC(self)
        dc.SetClippingRegion(box[0], box[1], w, h)
        memory = wxMemoryDC() 
        bmap = wxEmptyBitmap( w-2, h-2, -1 ) 
        memory.SelectObject( bmap )
        memory.Blit( 0,0, w-2, h-2, dc, box[0]+1, box[1]+1) 
        memory.SelectObject(wxNullBitmap)
        self._clipbmap=bmap
        # clear selection
        self.PaintImage()

    def CutBitmap(self, box):
        """ Cut portion of current bitmap using selected box """

        # get the image
        w,h =   box[2] - box[0], box[3] - box[1]
        # Client DC makes sure that we dont overdraw on the main window's widgets
        dc = wxClientDC(self)
        dc.SetClippingRegion(box[0]-1, box[1]-1, w+1, h+1)
        memory = wxMemoryDC() 
        bmap = wxEmptyBitmap( w-2, h-2, -1 ) 
        memory.SelectObject( bmap )
        memory.Blit( 0,0, w-2, h-2, dc, box[0]+1, box[1]+1) 
        memory.SelectObject(wxNullBitmap)
        # global clipboard
        self._clipbmap=bmap
        if wxTheClipboard.Open():
            wxTheClipboard.SetData(wxBitmapDataObject(bmap))
            wxTheClipboard.Close()
            
        # now blacken the selection
        dc.SetBackground(wxBrush(wxColour(0,0,0), wxSOLID))
        dc.SetPen(wxPen(wxColour(0,0,0), 1, wxSOLID))
        # Clear cut portion (shows as a black hole)
        dc.Clear()
        # Update current bitmap
        x, y = self.GetClientSizeTuple()
        bmap=wxEmptyBitmap(x,y,-1)
        memory.SelectObject( bmap )
        memory.Blit( 0, 0, x, y, dc, 0,0)
        memory.SelectObject(wxNullBitmap)
        self.SetBitmap(bmap)

    def CutBitmapInverse(self, box):
        """ Cut rest of selected bitmap """

        w,h =   box[2] - box[0], box[3] - box[1]
        # Client DC makes sure that we dont overdraw on the main window's widgets
        dc=wxClientDC(self)
        memory = wxMemoryDC() 
        bmap = wxEmptyBitmap(w-2, h-2,-1) 
        memory.SelectObject(bmap)
        memory.Blit(0, 0, w-2, h-2, dc, box[0]+1, box[1]+1) 
        
        # Now clear the window
        dc.SetBackground(wxBrush(wxColour(0,0,0), wxSOLID))
        dc.SetPen(wxPen(wxColour(0,0,0), 1, wxSOLID))
        dc.Clear()
        
        # Copy back the selected bitmap
        dc.Blit(box[0], box[1], w-2, h-2, memory, 0, 0)
        # now copy this dc to memory
        memory = wxMemoryDC()
        imgw, imgh = self._bmap.GetWidth(), self._bmap.GetHeight()
        bmap = wxEmptyBitmap(imgw, imgh, -1) 
        memory.SelectObject(bmap)
        memory.Blit(0,0,imgw,imgh, dc,0,0) 
        memory.SelectObject(wxNullBitmap)
        self._clipbmap=bmap
        # global clipboard
        if wxTheClipboard.Open():
            wxTheClipboard.SetData(wxBitmapDataObject(bmap))
            wxTheClipboard.Close()
            
        self.SetBitmap(bmap)

    def PasteBitmap(self, box):
        """ Paste clipboard bitmap onto selected box """

        w = box[2] - box[0]
        h = box[3] - box[1]
        st,end=box[0], box[1]
        
        print 'Inside pastebitmap code...!'
        memory = wxMemoryDC()
        memory.SelectObject(self._clipbmap)
        dc=wxClientDC(self)
        dc.SetClippingRegion(st, end, w, h)
        dc.Blit(st, end, w-2, h-2, memory, 0,0)
        memory.SelectObject( wxNullBitmap)
        bmap=self.GetBitmap()
        memory.SelectObject(bmap)
        reg=self.GetUpdateRegion()
        rect=reg.GetBox()
        # Added this code to update only region
        # that need to be painted
        modw, modh=0,0
        
        modw,modh=rect.width, rect.height       
        dc.SetLogicalFunction(wxCOPY)
        memory.Blit( 0,0, modw, modh, dc, 0,0) 
        memory.SelectObject( wxNullBitmap)
        self.SetBitmap(bmap)

         
# Main image frame class
class pyWiewMainFrame(wxFrame):
    """Main class of PyWiew, this class creates and manages the main window 
    frames, its subwindows and the image objects """

    def __init__(self, parent, ID, title, pos=wxDefaultPosition,size=wxDefaultSize):
        print 'Starting PyWiew...'

        # Initialize variables
        self.InitVariables()
        # config class instance
        self._config = pyWiewConfig.pyWiewConfig(self, "PyWiew")
        # options class instance
        self._options = pyWiewOptions.pyWiewImageOptions()
        # read the config file
        self._config.ReadConfigFile()

        if pyWiewGlobals.TestPilStatus():
            print 'PIL is ok'
            self._options.SetPilStatus(true)
        else:
            print 'PIL is screwed'
            self._options.SetPilStatus(false)

        style = 0
        if not self._options.ResizeAllowed():
            style = wxSYSTEM_MENU | wxCAPTION 
        else:
            style = wxDEFAULT_FRAME_STYLE 

        if self._options.FrameOnTop():
            print 'Sexual frustration: Staying on top of my girl... '
            style = style | wxSTAY_ON_TOP 

        print 'Initializing Frame...'
        wxFrame.__init__(self, parent, ID, title, pos, size, style)
        # Add statusbar
        self.AddStatusBar()
        self.AddToolBar()
        # Add Canvas
        self.AddCanvas()
        # Make menus
        self.MakeMenus()
        # Register event handlers
        self.RegisterEventHandlers()
        # Initialize network 
        self.InitNetwork()
        print 'Setting View Menu...'
        self.ToggleViewMenu( self._options.GetFit())
        self._W, self._H = self.GetSizeTuple()
        print 'Displaying Frame...'
        # Set menu item visiblity
        self.SetMenuItems()
        # Display window
        self.Display()
        self.Center()

        # time keeper/clipboard monitor
        self._thread=helperThread("Helper", self)
        self._thread.start()
        
        print 'PyWiew is up and running.'

    def InitVariables(self):
        """ Initialize all variables for the class  """

        print 'Initializing variables...'
        self.InitStringVars()
        self.InitListVars()
        self.InitStateVars()
        self.InitImageVars()
        self.InitImageParamVars()
        self.InitObjectVars()
        self.InitPrintVars()
        
    def InitStringVars(self):
        """ Initialize string type variables """

        # filename of image
        self._fileName = ""
        # current directory
        self._dir = ""
        # directory where file is saved
        self._savedir = ""
        # caption
        self._caption=""
        # error message
        self._errmsg=""

    def InitListVars(self):
        """ Initialize list type variables """

        # most recently used directories...
        self._mrud = []
        # undo list
        self._undoList = []
        # filelist
        self._files=[]
        # url list
        self._mruu=[]

    def InitImageVars(self):
        """ Init image type variables """

        # first wxImage instance for class (original image)
        self._img = wxEmptyImage()
        # second wxImage instance for class (displayed image)
        self._dispimg = wxEmptyImage()
        # clipboard image
        self._clipimg = wxEmptyImage()
        # image stream (imags downloaded from internet)
        self._imgstream=None
        # pil image instance (for animations)
        self._pilimg=None

    def InitImageParamVars(self):
        """ Image parameter variables """

        # window height/width upon open
        self.orig_ht = pyWiewGlobals.DH
        self.orig_wd = pyWiewGlobals.DW
        # current image size ratio
        self._zoom = 1.0
        # image flip-rot status
        self._fliprot1 = 0
        self._fliprot2 = 0
        # mode of image
        self._depth=24
        # fit needed for small/large images?
        # 0- no fit
        # 1- fitting a small image to window
        # 2- fitting an image to screen
        self._fitflag=0
        # load time for image
        self._loadtime=0.0

    def InitStateVars(self):
        """ State keeper variables (flags) """
        
        # fit status for curr image
        self._fitdone=false
        # fullscreen status of frame
        self._fullscreen=false
        # current file index
        self._currindex = 0
        # animation flag
        self._animation=false
        # prog sizing flag
        self._progsize=false
        # shuffle directory files
        self._shuffle=0
        # has opened an image yet?
        self._set = false

    def InitObjectVars(self):
        """ Object type variables """
        
        # event handler instance
        self._handler = pyWiewEvtHandler.pyWiewEvtHandler(self)
        # animation class instance
        self._pyanim=None

    def InitPrintVars(self):
        """ Initialize variables associated with printing """

        self._printData = wxPrintData()
        self._printpgSetupData = wxPageSetupDialogData()
        
    def RegisterEventHandlers(self):
        "Register all event handlers for the main window and its children"

        print 'Registering event handlers...'
        # Manage window closing
        EVT_CLOSE(self, self.doClose)
        
        # For filtering images
        # 10007 stands for wxMenuCommand event
        EVT_COMMAND_RANGE(self, 21,  29, 10007, self._handler.OnFilterEvent)
        EVT_COMMAND_RANGE(self, 31,  60, 10007, self._handler.OnFilterEvent)
        EVT_COMMAND_RANGE(self, 701, 708, 10007, self.CustomZoom)
        EVT_COMMAND_RANGE(self, 800, 805, 10007, self._handler.MultiImageFilter)
        EVT_COMMAND_RANGE(self, 901, 903, 10007, self.DesktopWallpaper)

        # index numbers 300-304 reserved for mrud
        EVT_COMMAND_RANGE(self, 300, 304, 10007, self.OnMrud)
        EVT_COMMAND_RANGE(self, 310, 314, 10007, self.OnMruu)
        EVT_MENU(self, wxID_EXIT,   self.doExit)
        EVT_MENU(self, wxID_CLOSE,  self.doClose)

        # For keyboard events
        EVT_CHAR(self, self._handler.OnKeyEvent)
        EVT_CHAR_HOOK(self, self._handler.OnKeyEvent)
        EVT_KEY_DOWN(self, self._handler.OnKeyEvent)
        EVT_SIZE(self, self.ManageSizing)
        
        # Process events posted by animation thread
        EVT_MENU(self, pyWiewGlobals.ANIMNOTIFEVENT, self.ProcessAnimationEvents)
        # Allow drag n drop of files
        self.DragAcceptFiles(true)
        EVT_DROP_FILES(self, self._handler.OnDropFiles)
        
        
    def AddCanvas(self):

        bgcolor = self._options.GetPanelBgColor()
        self._mainpanel = pyWiewPanel(self, -10, bgcolor)
        self._mainpanel.RegisterEventHandlers(self._handler)
        
    def AddToolBar(self):          

        # Begin toolbar config
        print 'Adding toolbar...'
        self._tb = self.CreateToolBar(wxNO_BORDER|wxTB_3DBUTTONS|wxTB_HORIZONTAL|wxTB_DOCKABLE)

        self._tb.AddSimpleTool(1000, pyWiewGlobals.getOpenBitmap(), "Open", "Open an Image file")
        EVT_TOOL(self, 1000, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1000, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1001, pyWiewGlobals.getNewBitmap(), "New", "Create a new Image")
        EVT_TOOL(self, 1001, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1001, self._handler.OnToolRClick)
        self._tb.AddSimpleTool(1002, pyWiewGlobals.getSaveBitmap(), "Save", "Save current image")
        EVT_TOOL(self, 1002, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1002, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1003, pyWiewGlobals.getCloseBitmap(), "Close", "Close current image")
        EVT_TOOL(self, 1003, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1003, self._handler.OnToolRClick)

        self._tb.AddSeparator()

        self._tb.AddSimpleTool(1004, pyWiewGlobals.getCopyBitmap(), "Copy", "Copy Image to clipboard ")
        EVT_TOOL(self, 1004, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1004, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1005, pyWiewGlobals.getCutBitmap(), "Cut", "Cut a portion of the image")
        EVT_TOOL(self, 1005, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1005, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1006, pyWiewGlobals.getPasteBitmap(), "Paste", "Paste Image")
        EVT_TOOL(self, 1006, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1006, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1007, pyWiewGlobals.getUndoBitmap(), "Undo ", "Undo Last Command")
        EVT_TOOL(self, 1007, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1007, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1008, pyWiewGlobals.getRedoBitmap(), "Redo ", "Redo Last Command")
        EVT_TOOL(self, 1008, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1008, self._handler.OnToolRClick)

        self._tb.AddSeparator()

        self._tb.AddSimpleTool(1009, pyWiewGlobals.getPrevBitmap(), "Prev Image", "Goto Previous Image")
        EVT_TOOL(self, 1009, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1009, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1010, pyWiewGlobals.getNextBitmap(), "Next Image", "Goto Next Image")
        EVT_TOOL(self, 1010, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1010, self._handler.OnToolRClick)

        self._tb.AddSeparator()
        self._tb.AddSeparator()

        self._tb.AddSimpleTool(1011, pyWiewGlobals.getZoominBitmap(), "Zoom in", "Zoom into image")
        EVT_TOOL(self, 1011, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1011, self._handler.OnToolRClick)

        self._tb.AddSimpleTool(1012, pyWiewGlobals.getZoomoutBitmap(), "Zoom out", "Zoom out of image")
        EVT_TOOL(self, 1012, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1012, self._handler.OnToolRClick)

        self._tb.AddSeparator()
        self._tb.AddSimpleTool(1013, pyWiewGlobals.getBulbLightedBitmap(), "Screenshot", "Take a Screenshot")
        EVT_TOOL(self, 1013, self._handler.OnToolClick)
        EVT_TOOL_RCLICKED(self, 1013, self._handler.OnToolRClick)
        EVT_TOOL_RCLICKED(self, -1, self._handler.OnToolRClick)  # Match all

        self._tb.Realize()

        if self._options.ShowtoolBar():
            self._tb.Show(true)
        else:
            self._tb.Show(false)
        
    def AddStatusBar(self):

        self.CreateStatusBar()
        stbar=self.GetStatusBar()
        # set fields for the status bar
        # field 0 - for messages
        # field 1 - image size x image depth
        # field 2 - percentage of current size to original image size
        # field 3 - image file size in KB
        # field 4 - position of image in the directory (index/number)
        # field 5 - Current time
        # field 6 - Loading time
        # field 7 - Current Time
        # Size of fields
        # field 1/field 5/field 6 - large
        # field 2/field 3/field 4 - small
        # field 7/field 8 - medium
        print 'Adding Statusbar...'
        stbar.SetFieldsCount(8)
        widthfields = [130, 90, 40, 60, 50, 70, 130, 120]
        stbar.SetStatusWidths(widthfields)
        if self._options.ShowStatusBar():
            stbar.Show(true)
        else:
            stbar.Show(false)

    def ManageSizing(self,event):
        """ This function does sizing """

        # sizing done by program code,
        # dont do anything
        if self._progsize:
            return
        
        wp, hp = self.GetClientSizeTuple()
        self._mainpanel.SetSize(wxSize(wp, hp))

        # resize image to best fit window
        if self._set:
            newimg=self.FitImageToSize(self._img, self._mainpanel.GetSize())
            self._dispimg=newimg
            self.UpdateBitmap(newimg.ConvertToBitmap())
            self.UpdateStatusBar()

    def MakeMenus(self):
        print 'Adding Menus...'

        self.MainMenu = wxMenuBar()
        self.AddMenus(self.MainMenu)
        self.SetMenuBar(self.MainMenu)

    def AddMenus(self, menu):
        """ Adds all the menus for the window """

        self.AddFileMenu(menu) # File menu
        self.AddEditMenu(menu) # Edit menu
        self.AddImageMenu(menu) # Image menu
        self.AddEffectsMenu(menu) # Filters menu
        self.AddViewMenu(menu)    # View menu
        self.AddToolsMenu(menu) # Tools menu
        self.AddHelpMenu(menu)   # Help menu

    def AddEditMenu(self, menu):
        """ Adds the Edit Menu """

        editMenu = wxMenu()
        self.AddMenuItem(editMenu, 'Undo \tCtrl-Z', 'Undo the last command', self.doUndo)
        self.AddMenuItem(editMenu, 'Redo \tCtrl-D', 'Redo the last command', self.doRedo)
        self.AddMenuItem(editMenu, 'C&lear \tCtrl-L ', 'Clear Display', self.doClearDisplay)
        self.AddMenuItem(editMenu, '&Copy \tCtrl-C' , 'Copy image to Clipboard', self.doCopyImage)
        editMenu.AppendSeparator()
        self.AddMenuItem(editMenu, 'C&ut Selection \tCtrl-X', 'Cut selected part of the image', self.doCutImageSel)
        self.AddMenuItem(editMenu, 'C&ut, Leave Selection \tCtrl-U', 'Cut unselected part of the image', self.doCutImageLeaveSel)
        editMenu.AppendSeparator()
        self.AddMenuItem(editMenu, 'Cro&p \tCtrl-P', 'Crop Image ', self.doCropImage)
        self.AddMenuItem(editMenu, '&Paste \tCtrl-V', 'Paste Image from Clipboard ', self.doPasteImage)
        self.AddMenuItem(editMenu, '&Add Text \tCtrl-A', 'Annotate current image', self.Annotate)
        editMenu.AppendSeparator()
        self.AddMenuItem(editMenu, 'Erase &Program Clipboard \tCtrl-I', 'Clear local image clipboard', self.ClearClipboard) 
        self.AddMenuItem(editMenu, 'Erase &System Clipboard \tCtrl-S', 'Clear global clipboard', self.ClearGlobalClipboard) 
        self.AddMenuItem(editMenu, 'Erase &All Clipboards ', 'Clears all the clipboard data', self.ClearAllClipboard)
        menu.Append(editMenu, 'E&dit')

    def AddViewMenu(self, menu):
        """ Adds the View Menu """

        viewMenu = wxMenu()
        fitMenu = wxMenu()
        self.AddMenuItem(fitMenu,'Fit window to &image\t n/N' , 'Fits Image to Window', self.doFitWindow,true)
        self.AddMenuItem(fitMenu, 'Fit image to &window\t w/W' , 'Fits Window to Image', self.doFitImage,true)
        self.AddMenuItem(fitMenu, 'Toggle fit image to &screen\t f/F' , 'Fits the Screen', self.ToggleFitScreen,true)
        self.AddMenuItem(fitMenu, 'Toggle fit window to &screen\t i/I', 'Fits only the window to screen', self.ToggleFitWindowScreen,true)         
        self.AddMenuItem(fitMenu, 'Random Fit \t r/R', 'Choose a random fit', self.RandomFit,true)

        # fit submenu id is -600
        viewMenu.AppendMenu(-600, 'Image Fit', fitMenu, 'Current image fit')
        fitsmall = self._options.GetSmallImgFit()
        fitlrg = self._options.GetLargeImgFit()

        self.AddMenuItem(viewMenu, 'Fit small images to window\t t/T', 'Fits small images to window', self.doFitSmallImage,true,fitsmall)
        self.AddMenuItem(viewMenu, 'Fit large images to screen\t l/L', 'Fits large images to screen', self.doFitLargeImage,true,fitlrg)
        menu.Append(viewMenu, '&View')

    def AddFileMenu(self, menu):
        """ Adds the File Menu """

        fileMenu = wxMenu()
        self.AddMenuItem(fileMenu, '&Open File \tCtrl+O', 'View Image', self.doOpenFile)
        self.AddMenuItem(fileMenu, '&Open Web Image \tCtrl+N', 'Open an image from the internet', self.doOpenUrl)
        self.AddMenuItem(fileMenu, '&Reopen File \tCtrl+R', 'Reopen Image From Disk', self.doReOpenFile)
        self.AddMenuItem(fileMenu, '&Reopen Image \tShift+Ctrl+R', 'Reopen Current Image', self.doReOpenImage)
        self.AddMenuItem(fileMenu, 'C&lose Image \tAlt+C', 'Close Image', self.doCloseImage)
        self.AddMenuItem(fileMenu, '&Save Image \tAlt+S', 'Save Image', self.doSaveImage)
        self.AddMenuItem(fileMenu, 'Save Image &As \tShift+Alt+S', 'Save Image in another format', self.doSaveImageAs)
        fileMenu.AppendSeparator()
        self.AddMenuItem(fileMenu, '&Print Image \tAlt+P', 'Print Image', self.OnPrintImage)
        self.AddMenuItem(fileMenu, 'Print &Setup \tAlt+U', 'Printer Setup', self.OnPrintSetup)
        self.AddMenuItem(fileMenu, 'Pa&ge Setup \tAlt+G', 'Print Page Setup', self.OnPageSetup)     
        self.AddMenuItem(fileMenu, 'Print Pre&view \tAlt+V', 'Preview Print', self.OnPrintPreview)
        fileMenu.AppendSeparator()      
        self.AddMenuItem(fileMenu, 'Save Thu&mbnail \tAlt+T', 'Save Image As Thumbnail', self.OnSaveAsThumb)
        fileMenu.AppendSeparator()
        self.AddMenuItem(fileMenu, '&Next Image \tPg Dn', 'View Next Image', self.DirWalkDown)
        self.AddMenuItem(fileMenu, '&Previous Image \tPg Up', 'View Previous Image', self.DirWalkUp)
        fileMenu.AppendSeparator()
        self.AddMenuItem(fileMenu, '&First Image \tHome', 'View first image in the directory', self.ShowFirstFile)
        self.AddMenuItem(fileMenu, '&Last Image \tEnd', 'View last image in the directory', self.ShowLastFile)
        fileMenu.AppendSeparator()
        self.CreateMrud(fileMenu)
        self.CreateMruu(fileMenu)
        fileMenu.AppendSeparator()
        fileMenu.Append(wxID_CLOSE, 'Close \tCtrl+W', 'Close the window')
        fileMenu.Append(wxID_EXIT, 'E&xit \tEsc', 'Exit the program')
        menu.Append(fileMenu, '&File')

    def AddImageMenu(self, menu):
        """ Adds the Image handling menu """

        imageMenu = wxMenu()
        self.AddMenuItem(imageMenu, 'Rotate &Right \t->' , 'Rotate right', self.OnRotateRt)
        self.AddMenuItem(imageMenu, 'Rotate &Left \t<-', 'Rotate left', self.OnRotateLt)
        self.AddMenuItem(imageMenu, '&Horizontal Mirror \tH ', 'Mirror image horizontally', self.OnHorFlip)
        self.AddMenuItem(imageMenu, '&Vertical Mirror \tV', 'Mirror image vertically', self.OnVerFlip)
        imageMenu.AppendSeparator()
        submenu=wxMenu()
        submenu.AppendRadioItem(701, 'Zoom 25%', 'Zoom to 25 percent of image')
        submenu.AppendRadioItem(702, 'Zoom 50%', 'Zoom to 50 percent of image')
        submenu.AppendRadioItem(703, 'Zoom 75%', 'Zoom to 75 percent of image')
        submenu.AppendRadioItem(704, 'Zoom 100%', 'Zoom to 100 percent of image')
        submenu.AppendRadioItem(705, 'Zoom 125%', 'Zoom to 125 percent of image')
        submenu.AppendRadioItem(706, 'Zoom 150%', 'Zoom to 150 percent of image')
        submenu.AppendRadioItem(707, 'Zoom 175%', 'Zoom to 175 percent of image')
        submenu.AppendRadioItem(708, 'Zoom 200%', 'Zoom to 200 percent of image')
        item=submenu.FindItemById(704)
        item.Check(true)
        imageMenu.AppendMenu(700, 'Zoom ', submenu, 'Select Custom Zoom')
        self.AddMenuItem(imageMenu, 'Zoom &In \t(+)', 'Zoom In', self.OnZoomIn)
        self.AddMenuItem(imageMenu, 'Zoom &Out \t(-)', 'Zoom Out', self.OnZoomOut)
        self.AddMenuItem(imageMenu, 'Zoom Into Selection \t(/)', 'Zoom into selection', self.OnZoomSelect)
        imageMenu.AppendSeparator()
        imageMenu.Append( 29, '&Negative ', 'Negative Of Image')
        imageMenu.Append( 32, '&Grayscale ', 'Convert image to grayscale')
        imageMenu.AppendSeparator()
        submenu=wxMenu()
        submenu.Append(38, 'Red Channel', "Shows the red channel")
        submenu.Append(39, 'Green Channel', "Shows the green channel")
        submenu.Append(41, 'Blue Channel', "Shows the blue channel")
        imageMenu.AppendMenu(wxNewId(), "Color Channel",  submenu, "Shows r,g,b channels")
        submenu=wxMenu()
        submenu.Append(42, 'RGB->GBR', 'RGB->GBR')
        submenu.Append(43, 'RGB->BRG', 'RGB->BRG')
        submenu.Append(44, 'RGB->BGR', 'RGB->BGR')
        submenu.Append(45, 'RGB->RBG', 'RGB->RBG')
        submenu.Append(46, 'RGB->GRB', 'RGB->GRB')           
        imageMenu.AppendMenu(wxNewId(), "Swap Colors", submenu, "Swap r,g,b components")
        imageMenu.AppendSeparator()
        self.AddMenuItem(imageMenu, '&Create New Image', 'Creates a new Image', self.MakeNewImage)
        menu.Append(imageMenu, '&Image')                     

    def AddEffectsMenu(self, menu):
        """ Adds the filter menu """

        effectsMenu = wxMenu()
        effectsMenu.Append( 21, '&Blur ', 'Blur Image')
        effectsMenu.Append( 22, '&Sharpen ', 'Sharpen Image')
        effectsMenu.Append( 23, 'S&mooth ', 'Smooth Image')
        effectsMenu.Append( 28, '&Detail ', 'Image detail')
        effectsMenu.AppendSeparator()
        effectsMenu.Append( 25, 'Edge De&tect ', 'Detect image edges')
        effectsMenu.Append( 26, 'Edge En&hance ', 'Enhance image edges')
        effectsMenu.Append( 27, '&Emboss ', 'Image emboss')
        effectsMenu.Append( 24, '&Contour Detect ', 'Detect image contours')
        effectsMenu.Append( 49, '&Granite Etch', 'Give image an etched look')
        effectsMenu.Append( 50, '&3D Shading', 'Detect 3D Features')     
        effectsMenu.Append( 47, '&Autocontrast ', 'Perform image autocontrast')
        effectsMenu.Append( 48, '&Solarize ', 'Solarize image')
        effectsMenu.AppendSeparator()
        effectsMenu.Append( 31, '&Modify Color', 'Enhance colors by a factor')
        effectsMenu.Append( 33, '&Vary Sharpness', 'Vary sharpness by a factor')
        effectsMenu.Append( 34, '&Adjust Contrast ', 'Vary contrast by a factor')
        effectsMenu.Append( 35, '&Adjust Brightness ', 'Vary brightness by a factor')
        effectsMenu.AppendSeparator()
        effectsMenu.Append( 36, 'Horizontal wrap', 'Wrap image around horizontally')
        effectsMenu.Append( 37, 'Vertical wrap', 'Wrap image around vertically')
        menu.Append(effectsMenu, '&Effects')  

    def AddToolsMenu(self, menu):
        """ Adds the tools menu """

        toolsMenu = wxMenu()
        self.AddMenuItem(toolsMenu, '&Image Browser ', 'Browse Images', self.OnBrowseDir)
        self.AddMenuItem(toolsMenu, '&Snapshot ', 'Takes a snapshot of the Desktop', self.ScreenShot)
        wallpapermenu=wxMenu()
        wallpapermenu.AppendRadioItem(901, '&Centered', 'Centered Wallpaper')
        wallpapermenu.AppendRadioItem(902, '&Tiled', 'Tiled Wallpaper')
        wallpapermenu.AppendRadioItem(903, '&Streched', 'Stretched Wallpaper')
        toolsMenu.AppendMenu(900, 'Set Image As Wallpaper', wallpapermenu, 'Set Wallpaper')
        self.AddMenuItem(toolsMenu, '&Automatic Viewing ', 'Start automatic viewing of images', self._handler.doSlideShow, true)
        self.AddMenuItem(toolsMenu, '&Options ', 'Show Options Pane', self.OptionsPane)
        submenu=wxMenu()
        submenu.Append(800, 'Blend two images', 'Blends two images of same size')
        submenu.Append(801, 'Subtract two images', 'Subtracts two images')
        submenu.Append(802, 'Add two images', 'Adds up two images')
        submenu.Append(803, 'Superimpose two images', 'Superimpose one image on another')
        submenu.Append(804, 'Create lighter image', 'Make image out of lighter pixels')
        submenu.Append(805, 'Create darker image', 'Make image out of darker pixels')
        toolsMenu.AppendMenu(wxNewId(), 'Multi-Image Functions', submenu, 'Perform filter operations on multiple images')
        menu.Append(toolsMenu, '&Tools')

    def AddHelpMenu(self, menu):
        """ Adds the help menu """

        helpMenu = wxMenu()
        self.AddMenuItem(helpMenu, 'About', 'About the program', self.doHelpAbout)
        menu.Append(helpMenu, '&Help')

    def AddAnimationMenu(self, menu):
        """ Add an animation menu """

        animMenu = wxMenu()
        self.AddMenuItem(animMenu, '&Restart Animation ' , 'Restart the animation', self.RestartAnimation)
        self.AddMenuItem(animMenu, '&Suspend Animation ' , 'Suspend the animation', self.SuspendAnimation)
        self.AddMenuItem(animMenu, '&End Animation ', 'End the animation thread', self.EndAnimation)
        self.AddMenuItem(animMenu, '&Increase animation speed', 'Increase the speed of animation', self.IncAnimationSpeed)
        self.AddMenuItem(animMenu, '&Decrease animation speed', 'Decrease speed of animation',self.DecAnimationSpeed)
        self.AddMenuItem(animMenu, '&Rewind Animation', 'Rewind the animation', self.RewindAnimation)
        self.AddMenuItem(animMenu, '&Seek Last Frame', 'Goto last frame', self.SeekAnimationFrameLast)
        self.AddMenuItem(animMenu, '&Dump all frames', 'Dump all frames in animation', self.DumpAnimationFrames)
        menu.Insert(3, animMenu, '&Animation')

    def AddMenuItem(self, menu, itemText, itemDescription, itemHandler,check=false,chkstatus=false):
        "Add a normal menu item (with no submenus)"

        menuId = wxNewId()
        item=wxMenuItem(menu, menuId, itemText,itemDescription,check)
        menu.AppendItem(item)
        if chkstatus is true:
            item.Check(true)
        EVT_MENU(self, menuId, itemHandler)
        return menuId

    def MakeNewImage(self, event):
        """ Creates a fresh image """

        dlg= pyWiewWidgets.pyWiewCreateImageDialog(self)
        result = dlg.ShowModal()

        if result == wxID_OK:
            w,h= int(dlg.GetImgWidth()), int(dlg.GetImgHeight())
            xdpi, ydpi = int(dlg.GetXdpi()), int(dlg.GetYdpi())
            depth = dlg.GetBitdepth()
            gry = dlg.GetGrayScale()
            col = dlg.GetBackgroundColor()

            dlg.Destroy()
            # first close current image
            self.doCloseImage(event)
            # create an image of size (w,h) of color col
            if gry is true:
                r,g,b=col.Red(),col.Green(),col.Blue()
                gray=int(0.56*g + 0.33*r + 0.11*b)
                col=wxColour(gray,gray,gray)

            img=pyWiewGlobals.ImgOfSize((w,h), col, depth)   
            self._img=img
            bmap=img.ConvertToBitmap()
            bmap.SetDepth(depth)
            self._depth = depth
            self.UpdateBitmap(bmap)
            # reset image handlers and params
            self.ResetImageParams()
            self._set=true
            self._caption="(New Image)"
            self._fileName='(New Image)'
            self._dir=""
            self.DrawImage()
            self.GetStatusBar().SetStatusText('New Image Created', 0)

    def SetRandom(self, val):
        if val:
            self._shuffle=1
        else:
            self._shuffle=0

        if self._shuffle:
            # randomize list
            random.shuffle(self._files)
        else:
            # sort list
            self._files.sort()      
            
    def RandomFit(self, event):
        self.ToggleViewMenu(4)

    def ToggleViewMenu(self,index):
        "Toggle the check status of the sub-menu items in view menu"

        id = self.MainMenu.FindMenu('View')
        viewMenu=self.MainMenu.GetMenu(id)
        if index < 5:
            # get image fit item
            fitMenu = viewMenu.FindItemById(-600)
            # get submenu
            submenu = fitMenu.GetSubMenu()
            itemList = submenu.GetMenuItems()
            item1 = itemList[index]
            item1.Check(true)

            itemList.pop(index)
            for x in range(0,len(itemList)):
                item=itemList[x]
                item.Check(false)

    def ToggleViewMenu2(self, index,val):
        """Toggle the check status of the menu items in view menu """

        id = self.MainMenu.FindMenu('View')
        viewMenu=self.MainMenu.GetMenu(id)
        itemList = viewMenu.GetMenuItems()
        item = itemList[index]
        item.Check(val)

    def CreateMruu(self, menu):
        """ Create a submenu with most recently used url list """

        submenu=wxMenu()

        index = 310
        print 'MRUU length = ', len(self._mruu)

        for x in range(0, len(self._mruu)):
            url = self._mruu[x]
            submenu.Append(index, url, url)
            index += 1

        # mruu id is -600
        id = -600
        menu.AppendMenu(id, 'Recent Urls ', submenu, 'Most recently visited URLs')

        
    def CreateMrud(self,menu):
        """Create a submenu with the most recently used directory list"""
        submenu=wxMenu()

        index = 300
        print 'MRUD Length = ', len(self._mrud)
        for x in range(0,len(self._mrud)):
            dir = self._mrud[x]
            if os.path.exists(dir) and os.path.isdir(dir):
                submenu.Append(index, dir, dir)
                index += 1

        # mrud id is -500
        id=-500
        menu.AppendMenu(id, 'Recent directories ', submenu, "Most recently used directories")

        
    def doUndo(self,event):
        """ Perform an undo of the previous command """

        if len(self._undoList)==0:
            self.SetStatusText('Cannot undo further!')
            self.SetEditMenuItem('Undo ', false)
            return

        undoInfo=self._undoList.pop()
        if undoInfo is None:
            self.SetStatusText('Error in undo data!')
            return

        dispimg=undoInfo['display']
        if dispimg and dispimg.Ok():
            self._dispimg=dispimg
        else:
            self.SetStatusText('Error in image data!')
            return

        self._fileName=undoInfo['filename']
        self._zoom=undoInfo['zoom']
        flip=undoInfo['fliprot']

        self._fliprot1 = flip/10
        self._fliprot2 = flip%10

        # perform any flip/zoom/rotation on the image
        newimg=self.ApplyCurrentRotflip(self.doZoom())
        if newimg and newimg.Ok():
            self._set=true
            self.DrawImage(newimg)
            self.SetMenuItems()

        self.SetEditMenuItem('Redo ', true)

    def doRedo(self,event):
        """ Perform a redo command """

        pass

    def SaveUndoInfo(self):
        """ Saves undo information for the last command """

        sz=len(self._undoList)
        if sz==10:
            self.SetStatusText('Undo ring is full')
            return

        zoom, fliprot = 1.0, 0

        if sz>=1:
            prevUndoInfo=self._undoList[sz-1]
            zoomchng,rotflipchng=false,false
            prevzoom=prevUndoInfo['zoom']
            prevfliprot=prevUndoInfo['fliprot']
            
            if self._zoom != prevzoom:
                zoomchng=true

            currentfliprot = self._fliprot1*10 + self._fliprot2
            if currentfliprot != prevfliprot:
                rotflipchng=true
            
            if zoomchng or rotflipchng:
                zoom=self._zoom
                fliprot=currentfliprot
        else:
            zoom=self._zoom
            fliprot=self._fliprot1*10 + self._fliprot2

        undoInfo={ 'display' : self._dispimg,
                   'filename' : self._fileName,
                   'zoom' : zoom,
                   'fliprot' : fliprot }

        self._undoList.append(undoInfo)
        self.SetEditMenuItem('Undo ', true)

    def SetAnimationStartMenuItem(self, val):
        """ Setting animation start menu item disable/enable """

        print 'Setting animation menu items ...'
        # set the menuitem disabled
        id = self.MainMenu.FindMenu('Animation')
        animMenu=self.MainMenu.GetMenu(id)

        itemList = animMenu.GetMenuItems()
        for x in range(0, len(itemList)):
            item=itemList[x]
            if item.GetLabel() == 'Restart Animation ':
                item.Enable(val)
            elif item.GetLabel() == 'Suspend Animation ':
                item.Enable(not val)
            elif item.GetLabel() == 'End Animation ':
                item.Enable(not val)
                
    def RestartAnimation(self, event):
        """ Restart the animation """

        print 'In restart animation menu item...'
        if self._pyanim:
            if self._pyanim.isAlive():
                print 'Thread alive... restarting!'
                self._pyanim.RestartAnimation()
                self.SetAnimationStartMenuItem(false)               
                return
            
        print 'Thread ended ... starting fresh!'
        # Simulate a fresh start by re-opening image
        self.doReOpenImage(event)
        # set the menuitem disabled
        self.SetAnimationStartMenuItem(false)

    def SuspendAnimation(self, event):

        if self._pyanim and self._pyanim.isAlive():
            self._pyanim.SuspendAnimation()

            # set the menuitem disabled
            id = self.MainMenu.FindMenu('Animation')
            animMenu=self.MainMenu.GetMenu(id)
            itemList = animMenu.GetMenuItems()

        self.SetAnimationStartMenuItem(true)

    def EndAnimation(self, event):
        """ Stop the animation """

        if self._pyanim:
            self._pyanim.end()
            self._pyanim=None
            
        self.SetAnimationStartMenuItem(true)
        
    def IncAnimationSpeed(self, event):
        """ Increase animation speed """

        if self._pyanim:
            self._pyanim.SpeedUpAnimation()

    def DecAnimationSpeed(self, event):

        if self._pyanim:
            self._pyanim.SlowDownAnimation()

    def RewindAnimation(self, event):

        if self._pyanim:
            self._pyanim.RewindAnimation()

    def SeekAnimationFrameLast(self, event):

        if self._pyanim:
            self._pyanim.GotoLastFrame()

    def DumpAnimationFrames(self, event):
        pass

    def ProcessAnimationEvents(self, event):
        """ Process events posted by the animation thread """

        print 'Processing animation events ...'
        self.SetAnimationStartMenuItem(true)
        self._pyanim=None
        
    def SetEditMenuItem(self, label, value):
        """ Set the value of the edit menu item """

        id = self.MainMenu.FindMenu('Edit')
        editMenu=self.MainMenu.GetMenu(id)
        itemList = editMenu.GetMenuItems()
        for x in range(0, len(itemList)):
            item=itemList[x]
            if item.GetLabel() == label:
                item.Enable(value)
                break

    def SetMenuItems(self):
        """ Sets the menu items """

        if not self._set:
            # FileMenu, disable all items except Open Image, MRUDS and Close/Exit
            id = self.MainMenu.FindMenu('File')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()

            for x in itemList:
                label=x.GetLabel()
                if label == 'Open File ' or label == 'Open Web Image ' or label == 'Recent directories ' or label == 'Close ' or label == 'Recent Urls ' or label == 'Exit ': 
                    continue
                else:
                    x.Enable(false)

            # Edit menu, disable all items except last two items
            id = self.MainMenu.FindMenu('Edit')
            editMenu=self.MainMenu.GetMenu(id)
            itemList = editMenu.GetMenuItems()
            for x in range(0, len(itemList) - 2):
                item=itemList[x]
                # Undo menu item, disable it only if there are no more items in undo list
                if item.GetLabel() == 'Undo ':
                    if len(self._undoList)==0:
                        item.Enable(false)
                    else:
                        item.Enable(true)

                # Paste menu item, disable it only if there is no clipboard data
                if item.GetLabel() == 'Paste ':
                    if self.IsClipboardEmpty():
                        item.Enable(false)
                    else:
                        item.Enable(true)
                else:
                    item.Enable(false)

            # Image menu, disable all items
            id = self.MainMenu.FindMenu('Image')
            imageMenu=self.MainMenu.GetMenu(id)
            itemList = imageMenu.GetMenuItems()
            for x in range(0, len(itemList)):
                item=itemList[x]
                if item.GetLabel() != 'Create New Image':
                    item.Enable(false)

            # View menu, enable all items
            id = self.MainMenu.FindMenu('View')
            editMenu=self.MainMenu.GetMenu(id)
            itemList = editMenu.GetMenuItems()
            for x in range(0, len(itemList)):
                item=itemList[x]
                item.Enable(true)

            # Effects menu, disable all items
            id = self.MainMenu.FindMenu('Effects')
            editMenu=self.MainMenu.GetMenu(id)
            itemList = editMenu.GetMenuItems()
            for x in range(0, len(itemList)):
                item=itemList[x]
                item.Enable(false)

            # Tools menu, enable just two items
            id = self.MainMenu.FindMenu('Tools')
            editMenu=self.MainMenu.GetMenu(id)
            itemList = editMenu.GetMenuItems()
            for x in range(0, len(itemList)):
                item=itemList[x]
                label=item.GetLabel()
                if label == 'Options ' or label == 'Snapshot ' or label == 'Image Browser ' \
                       or label == 'Multi Image Filter':
                    item.Enable(true)
                else:
                    item.Enable(false)

        else:
            # File menu, enable all items, for directory items, check the directory validity
            id = self.MainMenu.FindMenu('File')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                label=x.GetLabel()
                if label == 'First Image ' or label == 'Last Image ' or label == 'Previous Image ' or label == 'Next Image ':
                    if self._dir != "" and os.path.isdir(self._dir):
                        x.Enable(true)
                    else:
                        x.Enable(false)
                else:
                    x.Enable(true)

            # Edit menu, enable all items
            id = self.MainMenu.FindMenu('Edit')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                x.Enable(true)


            # Disable redo menu item
            self.SetEditMenuItem('Redo ', false)

            # Image menu, enable all items
            id = self.MainMenu.FindMenu('Image')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                # if image is not an animation, dont enable animation
                # items
                label=x.GetLabel()
                # look for keyword animation
                if self._animation:
                    if label == 'Start Animation ':
                        x.Enable(false)
                        continue
                elif string.find(label, 'Animation') != -1:
                    x.Enable(false)
                    continue

                x.Enable(true)


            # Effects menu, enable all items
            id = self.MainMenu.FindMenu('Effects')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                x.Enable(true)


            # View menu, enable all items
            id = self.MainMenu.FindMenu('View')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                x.Enable(true)


            # Tools menu, enable all items, for slideshow item check directory
            id = self.MainMenu.FindMenu('Tools')
            fileMenu=self.MainMenu.GetMenu(id)
            itemList = fileMenu.GetMenuItems()
            for x in itemList:
                label=x.GetLabel()
                if label == 'Automatic Viewing ':
                    if self._dir != "" and os.path.isdir(self._dir):
                        x.Enable(true)
                    else:
                        x.Enable(false)
                else:
                    x.Enable(true)

    def UpdateMruu(self, url):
        """ Update the mruu list with the directory path 'dir' """

        # get filemenu
        fileMenu = self.MainMenu.GetMenu(0)
        # get mruu item
        mruuitem = fileMenu.FindItemById(-600)
        # get submenu
        submenu = mruuitem.GetSubMenu()
        id = 310
        # get item count
        count = submenu.GetMenuItemCount()
        urlpath = str(url)
        
        if count < 5:
            submenu.Append(id + count, urlpath, urlpath)
        else:
            itemlist = submenu.GetMenuItems()
            # create the menu fresh
            for x in range(0,5):
                submenu.Delete(id+x)
            for x in range(0,5):
                urlpath = self._mruu[x]
                submenu.Append(id, urlpath, urlpath)
                id += 1
        
        
    def UpdateMrud(self, dir):
        """ Update the mrud list with the directory path 'dir' """

        # get filemenu
        fileMenu = self.MainMenu.GetMenu(0)
        # get mrud item
        mruditem = fileMenu.FindItemById(-500)
        # get submenu
        submenu = mruditem.GetSubMenu()
        id = 300
        count = submenu.GetMenuItemCount()
        # last path added 
        path = dir

        if count < 5:
            submenu.Append(id + count, path, path)
        else:
            itemlist = submenu.GetMenuItems()
            # create the menu fresh
            for x in range(0,5):
                submenu.Delete(id+x)
            for x in range(0,5):
                path = self._mrud[x]
                submenu.Append(id, path, path)
                id += 1


    def OnMruu(self, event):
        """ Event handler for most recently visited urls menu """

        id = event.GetId()
        # id ranges from 310 to 314
        index = id - 310
        url = self._mruu[index]
        self.OpenUrl(url)
        
    def OnMrud(self,event):
        """ Event handler for Most recently used directories menu """

        id = event.GetId()
        # id ranges from 300 to 304
        # map id to directories in mrud list
        index = id - 300
        self._dir = self._mrud[index]
        self.doOpenFile(event)

    def CustomZoom(self, event):
        """ Select a custom zoom """

        id = event.GetId()
        # zoom id ranges from 700 to 708
        index = id - 700
        self._zoom=float(index)/4.0
        newimg=self.doZoom()
        self.DrawImage(newimg)

    def DesktopWallpaper(self, event):
        """ Set the desktop wallpaper """

        # right now this just sets the wallpaper settings
        # in the regsitry, but does not do any actual setting
        # because the changes are not activated.
        id = event.GetId()
        # wallpaper style id ranges from 901 - 903
        index = id - 900 - 1
        # index == 0 -> Centered
        # index == 1 -> Tiled
        # index == 2 -> Stretched

        if not os.name == "nt":
            self.MessageDialog("Cannot set wallpaper on non-windows platform")
            return

        # Copy the current image to %SystemRoot% directory as bitmap image
        systemdir=os.environ.get('SystemRoot')
        # if not set, look for 'windir' environment variable
        if systemdir=='':
            systemdir=os.environ.get('windir')
        if systemdir=='':
            return

        # get bitmap filename from current imagename
        bmpfullfilename=os.path.join(systemdir, "PyWiew Wallpaper.bmp")

        ret=self.SaveImageAsFormat(self._img, "BMP", bmpfullfilename)
        if not ret:
            return

        # Now modify windows registry key to set the wallpaper
        # The current wallpaper setting is located in the registry key
        # My Computer\HKEY_CURRENT_USER\Control Panel\Desktop\Wallpaper
        # The current wallpaper style is located in the registry key
        # My Computer\HKEY_CURRENT_USER\Control Panel\Desktop\WallpaperStyle
        x=_winreg.ConnectRegistry(None, _winreg.HKEY_CURRENT_USER)
        y=_winreg.OpenKey(x, r"Control Panel\Desktop", 0, _winreg.KEY_ALL_ACCESS)
        _winreg.SetValueEx(y, "Wallpaper", 0, _winreg.REG_SZ, bmpfullfilename)
        _winreg.SetValueEx(y, "WallpaperStyle",0,  _winreg.REG_SZ, str(index))
        _winreg.CloseKey(y)
        _winreg.CloseKey(x)
        # os.system("rundll32.exe shell32.dll,Control_RunDLL Desk.cpl")
        
    def OptionsPane(self,event):
        "Pop up the user options pane"

        optdlg=pyWiewOptions.pyWiewOptionsDlg(self, wxNewId())
        result=optdlg.ShowModal()

        if result == wxID_OK:
            print 'Setting options...'
            # get the options from optdlg
            viewPageTuple = optdlg.GetViewOptions()
            # appearance dictionary
            apprdict = viewPageTuple[0]
            # image fit dictionary
            imagefitdict = viewPageTuple[1]

            # appearance options
            self._options.SetAppearanceOptions(apprdict)

            # image fit options
            self._options.SetImageFitOptions(imagefitdict)

            # internet options
            netpageoptions=optdlg.GetInternetOptions()
            self._options.SetInternetOptions(netpageoptions)
            self.Update()
        else:
            print 'Dialog box killed!'

    def GetOptionsObject(self):
        return self._options

    def GetCanvas(self):
        return self._mainpanel
    
    def ToggleSlideMenu(self):
        "Toggles the slideshow menuitem"

        id = self.MainMenu.FindMenu('Tools')
        toolsMenu=self.MainMenu.GetMenu(id)
        itemList = toolsMenu.GetMenuItems()
        slideItem = itemList[3]
        if slideItem.IsChecked():
            slideItem.Check(false)

    def Annotate(self,event):
        """ Annotate images """

        if not self._set or not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        self.SaveUndoInfo()
        self._handler.AddText(event)

    def PrintInfo(self, event):
        "Show information on image "

        # This function is not implemented
        dict = {}
        imgsrc = wxImgToPILImg(self._img)
        imgsrc.info(dict)

    def ClearClipboard(self,event):

        # clear local clipboard
        if self._clipimg and self._clipimg.Ok():
            # destroy image data
            self._clipimg.Destroy()
            self._clipimg=None
            
        if self.IsClipboardEmpty():
            self.SetEditMenuItem('Paste ', false)

        return

    def ClearGlobalClipboard(self, event):
        """ Clear global clipboard """

        if wxTheClipboard.Open():
            wxTheClipboard.Clear()
            wxTheClipboard.Close()

        if self.IsClipboardEmpty():
            self.SetEditMenuItem('Paste ', false)

    def ClearAllClipboard(self, event):
        """ Clears all the clipboards """

        self.ClearGlobalClipboard(event)
        self.ClearClipboard(event)
        self.SetEditMenuItem('Paste ', false)

    def WindowShot(self, event):
        """ Snapshot of a window under mouse focus """

        self.SaveUndoInfo()
        time.sleep(10)
        wdw = wxGetActiveWindow()
        dc = wxClientDC(wdw)

        if dc:
            sz = dc.GetSize()
            x,y = sz.GetWidth(), sz.GetHeight()

            memory = wxMemoryDC() 
            bmap = wxEmptyBitmap( x, y, -1 ) 
            memory.SelectObject( bmap )
            memory.Blit( 0,0, x,y, dc, 0,0) 
            memory.SelectObject( wxNullBitmap )
            try:
                self._img=wxImageFromBitmap(bmap)
            except:
                self.SetStatusText('Screen Capture Error.')
                return

            self.UpdateBitmap(bmap)
            # update current image
            self._fileName = '(Screenshot)'
            self._set = true
            self._dir=""
            # reset image handlers and params
            self.ResetImageParams()
            self.DrawImage()

    def ScreenShot(self, event):
        """ Take a snapshot of desktop, and set it as current image """

        self.SaveUndoInfo()
        dc = wxScreenDC()

        if dc:
            sz = dc.GetSize()
            x,y = sz.GetWidth(), sz.GetHeight()

            memory = wxMemoryDC() 
            bmap = wxEmptyBitmap( x, y, -1 ) 
            memory.SelectObject( bmap )
            memory.Blit( 0,0, x,y, dc, 0,0) 
            memory.SelectObject( wxNullBitmap )
            try:
                self._img=wxImageFromBitmap(bmap)
            except:
                self.SetStatusText('Screen Capture Error.')
                return

            self.UpdateBitmap(bmap)
            # update current image
            self._fileName = '(Screenshot)'
            self._caption = '(Screenshot)'
            self._set = true
            self._dir=""
            # reset image handlers and params
            self.ResetImageParams()
            self.DrawImage()

    def OnPrintSetup(self, event):
        """ Setup printer and preview """

        pdlgdata = wxPrintDialogData()
        pdlg = wxPrintDialog(self, pdlgdata)

        pdlg.GetPrintDialogData().SetSetupDialog(TRUE)
        res=pdlg.ShowModal()
        if res==wxID_OK:
            self._printData = pdlg.GetPrintDialogData().GetPrintData()
        
    def OnPageSetup(self, event):
        """ Print page setup """

        pgsetupdlg = wxPageSetupDialog(self, self._printpgSetupData)
        res = pgsetupdlg.ShowModal()
        if res==wxID_OK:
            self._printData = pgsetupdlg.GetPageSetupData().GetPrintData()
            self._printpgSetupData = pgsetupdlg.GetPageSetupData()
        
    def OnPrintPreview(self, event):
        """ Print Preview """

        self.ShowPreview()

    def ShowPreview(self):

        preview = wxPrintPreview(pyWiewPrintOut(self._mainpanel, self._caption),
                                 pyWiewPrintOut(self._mainpanel, self._caption),
                                 self._printData)
        if not preview.Ok():
            wxLogError("Error in preview")
            return

        previewframe = pyWiewPreviewFrame(self, preview, "PyWiew Preview", wxPoint(100, 100),
                                          wxSize(600, 650))

        previewframe.Centre(wxBOTH)
        previewframe.Initialize()
        previewframe.Show(TRUE)

    def OnPrintImage(self, menu):
        "Print image on printer (not implemented)"

        pdlgdata = wxPrintDialogData()
        printer = wxPrinter(pdlgdata)
        printout = pyWiewPrintOut(self._mainpanel, self._caption)
        printer.Print(self, printout)

    def OnMakeThumbs(self, menu):
        "Make thumbnails of all images in a directory"
        # make thumbnails for all image files in a directory
        # This function is not implemented
        pass

    def OnSaveAsThumb(self, menu):
        """ Save current image as thumbnail """

        # create file name
        # Append a "_t" to the base filename to show thumbnail
        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        # get file extension
        if not pyWiewGlobals.FileValidForRead(self._fileName):
            self.MessageDialog("Error: '%s' is not a valid filename" % self._fileName)
            self.SetStatusText("No valid filename")
            return

        list = os.path.splitext(self._fileName)
        base = list[0]   # test.jpeg -> test, .jpg
        newfile = list[0] + '_t' + list[1]   # newfile = test_t.jpg
        f = newfile

        size = self._options.GetThumbnailSizeTuple(wxSize(self._img.GetWidth(), self._img.GetHeight()))
        saved=0
        if self._options.GetPilStatus():
            imgsrc = pyWiewGlobals.wxImgToPILImg(self._img)
            imgsrc.thumbnail(size)
            format = pyWiewGlobals.PILGetFormat(f, 1)           
            try:
                imgsrc.save(f, format)
                saved=1
                self.SetStatusText('Saved thumbnail of image.')             
            except:
                self.SetStatusText('Error in saving thumbnail!')

        if saved==0:
            newimg=self._img.Scale(size[0],size[1])
            format = pyWiewGlobals.wxGetFormat(f, 1)
            try:
                newimg.SaveFile(f, format)
            except:
                self.SetStatusText('Error in saving thumbnail')
                return

            self.SetStatusText('Saved thumbnail for image.')

    def NextFile(self, filename):
        """ Return the next file after <filename> in the
        directory listing of current directory """

        files = self._files
        index = -1

        
        for x in range(0, len(files)):
            filepathname = os.path.abspath(os.path.join(self._dir, files[x]))
            if os.name == 'nt':
                filepathname = string.lower(filepathname)
            if filename == filepathname:
                index = x
                break


        # end of list
        if index == len(files) - 1:
            searchidx = -1
        else:
            searchidx = index
            
        # find next valid image file
        x=searchidx+1
        nextfile=os.path.abspath(os.path.join(self._dir, files[x]))

        while not pyWiewGlobals.FileValidForRead(nextfile) and x != searchidx:
            nextfile = os.path.abspath(os.path.join(self._dir, files[x]))
            x += 1

            if x == len(files) -1:
                x = 0

        return (x, nextfile)

    def GetFileIndex(self):
        """ Get current file index in directory """
        return self._currindex

    def PreviousFile(self, filename):
        """ Return the previous file before <filename> in the
        directory listing of current directory """

        files = self._files
        
        index = -1
        
        for x in range(0, len(files)):
            filepathname = os.path.abspath(os.path.join(self._dir, files[x]))
            if os.name == 'nt':
                filepathname = string.lower(filepathname)
            if filename == filepathname:
                index = x
                break

        # beginning of list
        if index == 0:
            searchidx = len(files)
        else:
            searchidx = index
            
        # find previous valid image file
        x=searchidx - 1
        prevfile=os.path.abspath(os.path.join(self._dir, files[x]))

        while not pyWiewGlobals.FileValidForRead(prevfile) and x != searchidx:
            prevfile = os.path.abspath(os.path.join(self._dir, files[x]))
            x -= 1

            if x == 0:
                x = len(files) - 1

        return (x, prevfile)
            
    def DirWalkUp(self,event):
        """ Walk up a directory in reverse alphabetical order and display images """

        dir = self._dir
        if os.name == 'nt':
            currfile = string.lower(os.path.abspath(self._fileName))
        else:
            currfile = os.path.abspath(self._fileName)

        index, filename = self.PreviousFile(currfile)
        self._currindex=index
        
        self.OpenImage(filename)

        return index

    def DirWalkDown(self,event):
        """ Walk down a directory in alphabetical order and display images """

        dir = self._dir
        if os.name == 'nt':
            currfile = string.lower(os.path.abspath(self._fileName))
        else:
            currfile = os.path.abspath(self._fileName)

        index, filename = self.NextFile(currfile)
        self._currindex=index       
        self.OpenImage(filename)

        return index

    def ShowLastFile(self, event):
        """ Show last file in directory """
        if self._dir == "":
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            self.MessageDialog("Error: not in a valid directory")
            return

        pass

    def ShowFirstFile(self, event):
        """ Show the first file in the directory """

        if self._dir == "":
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            self.MessageDialog("Error: not in a valid directory")
            return

    def ImageSet(self):
        return self._set

    def SetFileName(self, filename):
        self._fileName=filename

    def SetCurrentImage(self, img):
        if img and img.Ok():
            self._img = img
            self.ResetImageParams()
        else:
            return

    def SetDisplayImage(self, img):
        if img and img.Ok():
            self._dispimg = img
        else:
            return

    def DispImgOk(self):
        return self._dispimg.Ok()

    def doReOpenImage(self, event):
        """ Reopen current image """

        self.doCloseImage(event)
        if self._set:
            self.DrawImage(self._img)

    def doReOpenFile(self, event):

        self.doCloseImage(event)
        if os.path.isfile(self._fileName):
            self.OpenImage(self._fileName)
            
    def OnBrowseDir(self, event):
        """ Browse directories and view image thumbnails """

        # directory dialog box
        # get the current browse frame
        posn = self.GetPosition()
        sz = self.GetSize()
        w, h = sz.GetWidth(), sz.GetHeight()
        bwnd=pyWiewWidgets.pyWiewBrowseWindow(self, self._dir,"Browse Images", wxDefaultPosition, wxSize(500,600))
        self.Show(false)
        bwnd.SetFocus()

    def doClearDisplay(self, event):
        """ Clears current image """

        self.SaveUndoInfo()
        self._mainpanel.Clear()
        self._set=false

    def doCopyImage(self, event):
        """ Copy a portion of current image 
        update local and global clipboards """

        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        self.SaveUndoInfo()
        # copy the selected part
        box = self._handler.GetCurrentSelection()
        self._mainpanel.CopyBitmap(box)
        # copy img to local clipboard
        bmap = self._mainpanel.GetClip()
        self._clipimg=wxImageFromBitmap(bmap)
        # copy data to global clipboard
        if wxTheClipboard.Open():
            wxTheClipboard.SetData(wxBitmapDataObject(bmap))
            wxTheClipboard.Close()

    def doCutImageSel(self, event):
        """ Cut a portion of image 
        update local and global clipboards """
        
        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        # save the undo information
        self.SaveUndoInfo()
        # get current selection
        box = self._handler.GetCurrentSelection()
        self._mainpanel.CutBitmap(box)
        # set this bitmap as current image
        bmap = self._mainpanel.GetBitmap()
        clipbmap=self._mainpanel.GetClip()
        self._clipimg=wxImageFromBitmap(clipbmap)
        self._dispimg=wxImageFromBitmap(bmap)
        self._mainpanel.PaintImage()

    def doCutImageLeaveSel(self, event):
        """ Cut the image, leave selected area intact """

        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        # save the undo information
        self.SaveUndoInfo()
        # get current selection
        box = self._handler.GetCurrentSelection()
        # get the image
        self._mainpanel.CutBitmapInverse(box)
        bmap=self._mainpanel.GetBitmap()
        clipbmap=self._mainpanel.GetClip()
        self._clipimg=wxImageFromBitmap(clipbmap)       
        self._dispimg=wxImageFromBitmap(bmap)       
        self._mainpanel.PaintImage()
        
    def doPasteImage(self, event):
        """ Paste from clipboard to selection 
        local clipboard is given preference to global clipboard """

        # paste clipbrd to current selection
        st,end,w,h=0,0,0,0
        if self._set:
            box = self._handler.GetCurrentSelection()
            
            # resize clipboard image
            w,h = 0,0
            w = box[2] - box[0]
            h = box[3] - box[1]
            st,end=box[0], box[1]

        clipscldimg=wxEmptyImage()
        clipbmap=self.GetBitmapFromClipboard()
        if clipbmap and clipbmap.Ok():
            print 'Image from clipboard...!'
            img=wxImageFromBitmap(clipbmap)
            print 'After making image...!'
            if self._set:           
                try:
                    clipscldimg=img.Scale(w,h)
                except:
                    print 'Exception in image scaling...!'
                    clipscldimg=img.Copy()
                    
                clipbmap=clipscldimg.ConvertToBitmap()

        elif self._clipimg and self._clipimg.Ok():
            print 'Image from local clipboard...!'
            if self._set:
                try:
                    clipscldimg=self._clipimg.Scale(w,h)
                except:
                    clipscldimg=self._clipimg.Copy()
            else:
                clipscldimg=self._clipimg.Copy()
            clipbmap=clipscldimg.ConvertToBitmap()
        else:
            # clear global clipboard
            self.ClearGlobalClipboard(event)
            self.SetStatusText('Error in clipboard data!')
            return
        
        if clipbmap == wxNullBitmap:
            self.SetStatusText('Clipboard is empty')
            return

        self.SaveUndoInfo()
         
        if not self._set:
            w,h=clipbmap.GetWidth(),clipbmap.GetHeight()
            print w, h
            try:
                self._img=wxImageFromBitmap(clipbmap)
            except:
                return
              
        # Client DC makes sure that we dont overdraw on the main window's widgets!
        if self._set:
            self._mainpanel.SetClip(clipbmap)
            self._mainpanel.PaintImage()
            self._mainpanel.PasteBitmap(box)
            self._mainpanel.PaintImage()
        else:
            self.ResetImageParams()
            self._fileName='(Clipboard)'
            self._caption='(Clipboard)'
            self._set=true
            self.SetMenuItems()
            self.DrawImage()
            
        return

    def GetBitmapFromClipboard(self, thread=false):
        """ Returns the current clipboard data as an image """

        bmap=wxEmptyBitmap(pyWiewGlobals.DW,pyWiewGlobals.DH,-1)
        bmapobj=wxBitmapDataObject()
        try:
            if wxTheClipboard.Open():
                ret=wxTheClipboard.GetData(bmapobj)
                wxTheClipboard.Close()
        except:
            return None
         
        if ret is false:
            if not thread:
                self.SetStatusText('Empty Clipboard')
            return wxNullBitmap
         
        bmap=bmapobj.GetBitmap()
        return bmap


    def IsClipboardEmpty(self,thread=false):
        """ Checks if the clipboard is empty """

        bmap=self.GetBitmapFromClipboard(thread)
        if bmap == wxNullBitmap or self._clipimg is None:
            return true
        else:
            return false
         
    def UpdateBitmap(self, bmap, posn=(0, 0)):
        """ Update current bitmap and set its event handlers """

        self._mainpanel.SetPosition(posn)
        self._mainpanel.SetBitmap(bmap)
        self._mainpanel.PaintImage()
         
    def doCropImage(self, event):
        """ Crop the current image """

        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        self.SaveUndoInfo()
        # cropping based on left top and right bottom
        box = self._handler.GetCurrentSelection()
        img=self.CropImage(box)
        # set image into clipboard
        self._clipimg = img
        # we also need to set crop image as current!
        self._dispimg = img
        bmap = img.ConvertToBitmap()
        
        # set data for global clipboard
        if wxTheClipboard.Open():
            wxTheClipboard.SetData(wxBitmapDataObject(bmap))
            wxTheClipboard.Close()
            
        # Clear the selection
        self._handler.ClearSelection()
        fit=self._options.GetFit()
        
        # set fit to 0 (fit window to image)
        self._options.SetFit(0)
        self.DrawImage()
        # set fit to old value
        self._options.SetFit(fit)
        
    def CropImage(self, box):
        """ Crop the image by the area defined by box and return it """

        if box is None:
            return self._dispimg
         
        w,h =   box[2] - box[0], box[3] - box[1]
        # Get the sub image of this box and return
        rect = wxRect(box[0], box[1], w, h)
        # Check if the rectangle is within the image
        cw, ch = self._mainpanel.GetSizeTuple()
        if box[0]>0 and box[1]>0 and w<cw and h<ch:
            subimg = self._dispimg.GetSubImage(rect)
        else:
            self.SetStatusText('Invalid Region')
            return self._dispimg
            
        return subimg
    
    def OnZoomSelect(self, event):
        """ Zoom into the current selection """

        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        self.SaveUndoInfo()
        fit=self._options.GetFit()
        box = self._handler.GetCurrentSelection()
        img= self.CropImage(box)
        # we also need to set crop image as current!
        self._dispimg = img
        self._handler.ClearSelection()
        # set fit to 1 (let image occupy the window)
        self._options.SetFit(1)
        self.DrawImage()
        # reset fit value
        self._options.SetFit(fit)

    def FitScreen(self,img=None):
        "Fit image to the screen"

        # assume all images are not to be fit
        self._fitflag=0
        if img is None and self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return None

        newimg=wxEmptyImage()
        if img:
            newimg=img
        else:
            newimg=self._img
              
        fit = self._options.GetFit()

        # Fit only large images to screen, leave all others as normal
        if fit is 2:
            self._fitflag = 2
        elif self._options.GetLargeImgFit():
            # check image size and compare with screen size
            w,h = self._img.GetWidth(),self._img.GetHeight()
            # get screen device context
            dc=wxScreenDC()
            sw,sh = dc.GetSizeTuple()
            # if either width or height is greater,
            # we need to fit
            if w > sw or h > sh:
                self._fitflag=2
              
        # This will make the frame to fit the whole screen
        if self._fitflag:
            self.ShowFullScreen(wxFULLSCREEN_ALL)
            # set panel size to screen size
            self.Centre()
            return self.FitImageToSize(newimg, self.GetSize())
        else:
            # resets fullscreen mode if any
            if fit is not 3:
                self.ShowFullScreen(false)
            return newimg
     
    def ToggleFitWindowScreen(self, event):
        """ Fits window to screen ,but not the image """

        self.ToggleViewMenu(3)
        
        # Toggle between the fullscreen mode and
        # normal window mode
        fit= self._options.GetFit()
        
        if fit is not 3:
            # not fullscreen, set fullscreen
            self._options.SetFit(3)
            self.DrawImage()
        else:
            # already fullscreen, go to normal (windowsize = imagesize) mode
            self.doFitWindow(event)
            
    def FitWindowScreen(self):
        """ This function fits window to screen """
        
        self.ShowFullScreen(wxFULLSCREEN_ALL)
         
    def ToggleFitScreen(self,event):
         "Event handler to fit the image to screen"
         
         self.ToggleViewMenu(2)
         
         # Toggle between the fullscreen mode and
         # normal window mode
         fit= self._options.GetFit()
         if fit is not 2:
             self._options.SetFit(2)
             self.DrawImage()
         else:
             # already fullscreen, go to normal (windowsize = imagesize) mode
             self.doFitWindow(event)
         
    def doFitImage(self,event):
        "Fit current image to window"
        
        self.ToggleViewMenu(1)
        self._options.SetFit(1)
        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        # do a fit on the panel first
        # this makes sure that the bitmap
        # does not overdraw on other parts
        # of the frame
        # Do this only if the window was resized
        w,h=self.GetSizeTuple()
        if w == self._W and h == self._H:
            return

        self.Fit()           
        self.DrawImage()
        
    def doFitLargeImage(self, event):
        """ Fit only large images to screen """
        
        # Fit images larger than screen to screen
        # Display other images as normal

        val = self._options.GetLargeImgFit()
        newval = not val
        self._options.SetLargeImgFit(newval)
        self.ToggleViewMenu2(2,newval)
        # earlier we needed to fit large img, now dont
        if val:
            return
         
        self.DrawImage()
                             
    def doFitSmallImage(self,event):
        """ Fit only small images """

        val = self._options.GetSmallImgFit()
        newval = not val
        self._options.SetSmallImgFit(newval)
        self.ToggleViewMenu2(1,newval)
        
        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        # earlier we needed to fit small img, now dont
        if val:
            return

        # do a fit on the panel first
        # this makes sure that the bitmap
        # does not overdraw on other parts
        # of the frame

        # Do this only if the window was resized
        w,h=self.GetSizeTuple()
        if w == self._W and h == self._H:
            return

        self.DrawImage()
         
    def doFitWindow(self, event):
        "Fit the window to current image"

        self.ToggleViewMenu(0)
        self._options.SetFit(0)
        # since we are keeping image size, reset zoom value to 1.0
        self._zoom = 1.0
        
        # resets fullscreen mode if any
        self.ShowFullScreen(false)
        # reset display image to original image
        self._dispimg = self._img
        self.DrawImage()

    def FitImage(self,img=None,fitscreen=false):
        "Function that fits image to current window size"
        
        imgw, imgh = 0,0
        
        if img is None and self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return None
        
        newimg = wxEmptyImage()
        if img:
            newimg=img.Copy()
        else:
            newimg=self._img.Copy()

        if self._fitdone:    # Fit done already
            return newimg

        imgw, imgh = self._img.GetWidth(), self._img.GetHeight()
        fit = self._options.GetFit()
        # if we need to fitscreen or we need to fit to window always(fit=1)
        # we need not check for small images fit, otherwise we need to check
        if not fitscreen:
            if fit is 1:
                self._fitflag=1 
            else:
                fitsmall = self._options.GetSmallImgFit()
                # fit only small images
                if fitsmall:
                    # if either width or height is greater than
                    # minimum width or height, we dont do a fit(large image)
                    if imgw > pyWiewGlobals.DW or imgh > pyWiewGlobals.DH:
                        self._fitflag = 0
                    else:
                        self._fitflag = 1
                    if self._fitflag == 0:
                        return newimg

        return self.FitImageToSize(newimg, self._mainpanel.GetSize())

    def FitImageToSize(self, img, size):

        if img is None:
            img = self._img
            
        imgw = img.GetWidth()
        imgh = img.GetHeight()

        h,w = size.GetHeight(), size.GetWidth()
        # fit to a height less than 10 pixels
        # of the panel(to prevent overdraws)
        h -= 10
        if imgh > imgw:  # fit according to height
            newh = h
            neww = imgw*newh/imgh
            # calculate new width
            while neww > w:
                newh = newh-1
                neww = imgw*newh/imgh          
                
        elif imgw >= imgh:  # fit according to width
            neww = w
            newh = imgh*neww/imgw
            # calculate new height
            while newh > h:
                neww = neww-1
                newh = imgh*neww/imgw

        newimg=img.Scale(neww,newh)

        # update current zoom
        self._zoom = float(neww)/float(imgw)
        self._fitdone=true
        
        return newimg
    
    def ResetImageParams(self):
        """ Resets the current image parameters (zoom, rotation, flip) """
        
        self._zoom = 1.0
        self._fliprot1 = 0
        self._fliprot2 = 0
        
        EVT_CHAR(self, self._handler.OnKeyEvent)
        EVT_CHAR_HOOK(self, self._handler.OnKeyEvent)
         
    def OnRotateRt(self,event):
        """ Right rotation """

        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        self.SaveUndoInfo()
        # rotation changes orientation of image
        self._fliprot1 = not self._fliprot1
        # right rotation switches image corner (0,0)
        # in the order (0, 1, 2, 3, 0)
        self._fliprot2 += 1
        if self._fliprot2 == 4:
            self._fliprot2 = 0
        
        smimgfit=self._options.GetSmallImgFit()
        self._options.SetSmallImgFit(false)
        
        rotang=-float(math.pi/2.0)
        rotimg=self._dispimg.Rotate(rotang,wxPoint(0,0),false)
        self.DrawImage(rotimg)
        self._options.SetSmallImgFit(smimgfit)

    def OnRotateLt(self,event):
        """ Left rotation """

        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        self.SaveUndoInfo()
        # rotation changes orientation of image
        self._fliprot1 = not self._fliprot1
        # right rotation switches image corner (0,0)
        # in the order (3, 2, 1, 0, 3)
        self._fliprot2 -= 1
        if self._fliprot2 == -1:
            self._fliprot2 = 3
                
        smimgfit=self._options.GetSmallImgFit()
        fit=self._options.GetFit()
        self._options.SetSmallImgFit(false)
        self._options.SetFit(0)
        
        rotang=float(math.pi/2.0)
        rotimg=self._dispimg.Rotate(rotang,wxPoint(0,0),false)
        
        self.DrawImage(rotimg)      
        self._options.SetSmallImgFit(smimgfit)
        self._options.SetFit(fit)
        
    def OnVerFlip(self, event):
        """ Ver flip """
        
        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        print 'Mirror Vertical'     
        # ver flip switches top left corner of image to bot left corner
        # and top right corner to bot right corner
        # i.e 0 <==> 3 and 1 <==> 2
        if self._fliprot2 == 0:
            self._fliprot2 = 3
        elif self._fliprot2 == 3:
            self._fliprot2 = 0

        if self._fliprot2 == 1:
            self._fliprot2 = 2
        elif self._fliprot2 == 2:
            self._fliprot2 = 1
            
        self.SaveUndoInfo()
        smimgfit=self._options.GetSmallImgFit()
        self._options.SetSmallImgFit(false)
        newimg = self._dispimg.Mirror(false)
        self.DrawImage(newimg)
        self._options.SetSmallImgFit(smimgfit)
         
    def OnHorFlip(self, event):
        " Hor flip "

        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        print 'Mirror horizontal'
        self.SaveUndoInfo()
        # ver flip switches top left corner of image to top right corner
        # and bot left corner to bot right corner
        # i.e 0 <==> 1 and 3 <==> 2
        if self._fliprot2 == 0:
            self._fliprot2 = 1
        elif self._fliprot2 == 1:
            self._fliprot2 = 0

        if self._fliprot2 == 3:
            self._fliprot2 = 2
        elif self._fliprot2 == 2:
            self._fliprot2 = 3      

        smimgfit=self._options.GetSmallImgFit()
        self._options.SetSmallImgFit(false)
        newimg = self._dispimg.Mirror()
        self.DrawImage(newimg)
        self._options.SetSmallImgFit(smimgfit)

    def doZoom(self, img=None):

        if img is None:
            img=self._img

        wold = img.GetWidth()
        hold = img.GetHeight()
        
        w = wold*self._zoom
        h = hold*self._zoom

        return img.Scale(w,h)       

    def OnZoomIn(self, event):
        """ Zoom In """

        scal = 1.2
        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        self.SaveUndoInfo()
        self._zoom = self._zoom*scal
        smimgfit = self._options.GetSmallImgFit()
        
        newimg = self.ApplyCurrentRotflip(self.doZoom())
        self._options.SetSmallImgFit(false)
        self.DrawImage(newimg)
        self._options.SetSmallImgFit(smimgfit)
        
    def OnZoomOut(self, event):
        """ Zoom Out """

        scal = 1.2
        if not self._img.Ok():
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        self.SaveUndoInfo()
        self._zoom = self._zoom/scal
        
        newimg = self.ApplyCurrentRotflip(self.doZoom())
        fit = self._options.GetFit()
        smimgfit = self._options.GetSmallImgFit()
        self._options.SetFit(0)
        self._options.SetSmallImgFit(false)
        self.DrawImage(newimg)
        self._options.SetFit(fit)
        self._options.SetSmallImgFit(smimgfit)

    def ApplyCurrentRotflip(self, img):

        # Apply current rot-flip to image
        rotflip = self._fliprot1*10 + self._fliprot2

        rotflippedimg=wxEmptyImage()

        if rotflip==0:
            # no change
            rotflippedimg=img
        elif rotflip==1:
            # corner 0 moved to corner 1 => hor flip
            rotflippedimg=img.Mirror()
        elif rotflip==2:
            # corner 0 moved to corner 2 => hor + ver flip
            # or right rotation by 180 degrees
            newimg=img.Mirror()
            rotflippedimg=newimg.Mirror(false)
        elif rotflip==3:
            # corner 0 moved to corner 3 => ver flip
            rotflippedimg=img.Mirror(false)
        elif rotflip==10:
            # corner 0 remains same but we have a change
            # in orientation. Rotation to right + hor flip
            rotang=-float(math.pi/2.0)
            newimg=img.Rotate(rotang,wxPoint(0,0),false)
            rotflippedimg=newimg.Mirror()
        elif rotflip==11:
            # image orientation change + corner 0=>1
            # right rotation by 90 degree
            rotang=-float(math.pi/2.0)
            rotflippedimg=img.Rotate(rotang,wxPoint(0,0),false)         
        elif rotflip==12:
            # image orientation change + corner 0=>2
            # rotation to right + ver flip
            rotang=-float(math.pi/2.0)
            newimg=img.Rotate(rotang,wxPoint(0,0),false)
            rotflippedimg=newimg.Mirror(false)
        elif rotflip==13:
            # image orientation change + corner 0=>3
            # rotation to left
            rotang=float(math.pi/2.0)
            rotflippedimg=img.Rotate(rotang,wxPoint(0,0),false)

        return rotflippedimg
        
        
    def doOpenFile(self, event):
        """ Open a file """
        
        hint = (os.path.split(self._fileName))[1]
        dir = self._dir
        
        fileName = self.SelectFileDialog(dir, 0, hint)
        
        if fileName is None:
            self.OpenFileError(fileName)
        else:
            fileName = os.path.abspath(fileName)
            self.OpenImage(fileName)

    def InitNetwork(self, httpauthinfo=None):

        # get internet status
        netstatus = self._options.GetInternetStatus()

        if netstatus==0:
            # no internet connection
            return
        elif netstatus==1:
            # direct connection
            authinfo=urllib2.HTTPBasicAuthHandler()
            if httpauthinfo:
                realm=httpauthinfo[0]
                url=httpauthinfo[1]
                user=httpauthinfo[2]
                passwd=httpauthinfo[3]
                passwrdmngr=urllib2.HTTPPasswordMgr()
                passwrdmngr.add_password(realm, url, user, passwd)
                authinfo=urllib2.HTTPBasicAuthHandler(passwrdmngr)
                
            opener=urllib2.build_opener(authinfo, urllib2.HTTPHandler)
            urllib2.install_opener(opener)
        elif netstatus==2:
            # proxy based connection
            proxylist=self._options.GetProxyOptions()

            if len(proxylist):
                httpproxy=proxylist["http"]
                secproxy=proxylist["https"]
                ftpproxy=proxylist["ftp"]
                
                l=string.split(httpproxy,"http://")
                if len(l)==2:
                    httpproxy=l[1]
                l=string.split(secproxy,"http://")
                if len(l)==2:
                    secproxy=l[1]
                l=string.split(ftpproxy,"http://")
                if len(l)==2:
                    ftpproxy=l[1]

                if self._options.ProxyNeedsAuth():
                    auth=self._options.GetProxyAuth()
                    user=auth[0]
                    passwd=auth[1]

                    httpproxystring="http://" + user + ':' + passwd + "@" + httpproxy
                    httpsproxystring="https://" + user + ':' + passwd + "@" + secproxy
                    ftpproxystring="http://" + user + ':' + passwd + "@" + ftpproxy

                else:
                    httpproxystring="http://" + httpproxy
                    httpsproxystring="https://" + secproxy
                    ftpproxystring="http://" + ftpproxy
                    
            
                proxy_support=urllib2.ProxyHandler({"http":httpproxystring,
                                                    "https":httpsproxystring,
                                                    "ftp":ftpproxystring})

                authinfo=urllib2.HTTPBasicAuthHandler()
                opener=urllib2.build_opener(proxy_support, authinfo, urllib2.HTTPHandler)
                urllib2.install_opener(opener)
            

    def SetErrorMsg(self, msg):
        self._errmsg=msg

    def SetImageStream(self, stream):
        self._imgstream=stream

    def OpenStreamedImage(self, site, error=false):
        """ Open and display the current image stream """

        print 'opening a streamed image...'
        if self._imgstream is None or error:
            self.MessageDialog(self._errmsg)
            return

        stream=cStringIO.StringIO(self._imgstream)
        try:
            img=wxImageFromStream(stream)
        except:
            pass

        # We can directly open the file, but wxWindows
        # always shows an error for the image opened
        # from the web, though there is no problem with
        # the image. So we save the image to a temp directory,
        # open the saved image. This prevents problems in
        # manipulating the image and also avoids the ugly
        # popup from wx runtime about the invalid image.
        # We save the image in BMP format, since this seems
        # to work for all images regardless of errors in
        # data stream due to network problems.
            
        # create a temp filename
        tmpdir=os.environ.get('TMPDIR')
        if tmpdir=="":
            tmpdir=os.environ.get('TEMP')
        if tmpdir=="":
            tmpdir=os.environ.get('TMP')
                
        f=os.tempnam(tmpdir, "streamedimg")
        pimg=pyWiewGlobals.wxImgToPILImg(img)
        # save the file in  format
        if img:
            ret=self.SaveImageAsFormat(img, "BMP", f)
            if ret is false:
                return
        else:
            self.MessageDialog("Error: Invalid Image!")
            return
            
        self.SaveUndoInfo()
        # open it again
        self.SetImage(f)

        # get filename from url
        properurl = string.split(site, 'http://')[1]
        urlstrings = string.split(properurl, '/')

        l=len(urlstrings)
        filenamefromurl = urlstrings[l-1]

        self._fileName=filenamefromurl
        self._caption=site
        self.SetStatusText('Displaying image..')
        # remove the file
        try:
            os.remove(f)
        except:
            pass
        
        self.DrawImage()

    def doOpenUrl(self, event):
        """ Open a file from its internet location """

        if self._options.GetInternetStatus()==0:
            # no internet connection
            msg="Error: No internet connection is configured!"
            self.MessageDialog(msg)
            return
        
        dlg=pyWiewMiscWidgets.pyWiewUrlOpener(self, wxNewId(), 'Open Image From the Internet')
        res=dlg.ShowModal()

        if res == wxID_OK:
            url=dlg.GetUrl()
            # check if the url needs authorization
            urlneedsauth=dlg.UrlNeedsAuth()

            if urlneedsauth:
                authuser=dlg.GetUserName()
                authpwd=dlg.GetPassword()

                print 'URL needs auth!'
                # build the request
                user_pass=authuser + '@' + authpwd
                user_pass=base64.encodestring(urllib2.unquote(user_pass)).strip()
                req=urllib2.Request(url)
                req.add_header('Authorization', 'Basic ' + user_pass)
                site=req
                # url = string.split(url, 'http://')[1]
                # site = 'http://' + authuser + ':' + authpwd  + '@' + url
            else:
                site=url
                
            dlg.Destroy()
            # update mruu list
            self.SetMruu(url)
            self.SetStatusText('Downloading image...')
            
            self.OpenUrl(site)

    def OpenUrl(self, url):
        """ Open the passed url and display the URL image """

        wait=wxBusyCursor()
        flag=0
        try:
            self._imgstream=urllib2.urlopen(url).read()
        except urllib2.HTTPError, e:
            msg=str(e)
            msg += ' '
            msg+=url
            msg += '\n'
            if string.find(msg, "server") != -1 or \
                   string.find(msg, "400") != -1 or \
                   string.find(msg, "404") != -1 or \
                   string.find(msg, "500") != -1:
                msg+="Check the URL"
            else:
                msg+="Check your internet settings"
                self.SetErrorMsg(msg)
                flag=-1
        except urllib2.URLError, e:
            msg=str(e) 
            msg+=' '
            msg+=url
            msg+='\n'
            msg+="Check the URL"
            self.SetErrorMsg(msg)
            flag=-1
        except ValueError, e:
            msg = str(e)
            msg +=' '
            msg += url
            msg += '\n'
            msg += 'Check the URL'
            flag=-1
            self.SetErrorMsg(msg)
        if flag==0:
            # ok!
            self.OpenStreamedImage(url)
        else:
            # error!
            self.OpenStreamedImage(url, true)

    def SetImage(self, fileName):
        """ Open a file, sets the current file 
        and return a wxImage instance """

        opened=0
        self._fileName = fileName
        
        if self._options.GetPilStatus():
            t1=time.clock()
            try:
                imgsrc = Image.open(fileName)
                print 'Opened using PIL...'
                opened=1
            except:
                pass

            if opened:
                t2=time.clock()

                # find out if image is animation
                try:
                    self._pyanim=pyWiewAnim.pyWiewAnimation(imgsrc)
                except IOError:
                    pass
                
                if self._pyanim and self._pyanim.IsAnimation():
                    self._animation=true
                    self._pilimg=imgsrc.copy()
                    # get duration between frames
                    try:
                        durn=imgsrc.info['duration']
                    except ValueError:
                        durn=0

                    self._pyanim.SetDuration(durn)
                else:
                    self._pyanim=None
                    self._animation=false
                    
                try:
                    img=pyWiewGlobals.PilImgTowxImg(imgsrc)
                    self._img=img.Copy()
                except IOError:
                    opened=0
                except AttributeError:
                    opened=0

        if not opened:
            wxformat = pyWiewGlobals.wxGetFormat(fileName, 0)
            img=wxEmptyImage()
            t1=time.clock()
            if wxformat:
                try:
                    img = wxImage(fileName, wxformat)
                    print 'Opened with wx...'
                except:
                    pass

                if img.Ok():
                    opened=1
                    self._img=img.Copy()
                else:
                    print 'Bad image!'
                    
                if opened:
                    t2=time.clock()
                    self._loadtime = float(math.ceil((t2-t1)*100)/100)
                    
        if opened:
            self._dispimg = self._img
            self._set = true
            self._caption = self._fileName
            self.SetCurrentDir()
            return true
        else:
            self.SetStatusText("Image Error")
            return false

    def OpenTextFileAsImage(self, fileName):
        """ Open a text file and create an image """

        font = self._options.Font()
        fontcolor = self._options.FontColor()
        pass
    
    def DeleteFile(self, event):
        """ Remove the current image file """
        
        # wxRemoveFile(self._fileName)
        fname=os.path.split(self._fileName)[1]
        msg='Shred the file ' + fname + ' ?'
        cfmdlg=pyWiewMiscWidgets.pyWiewConfirmDialog(self, wxNewId(), 'Confirmation ')
        
        cfmdlg.SetMsg(msg)
        res=cfmdlg.ShowModal()
        if res == wxID_OK:
            if os.name == 'nt':
                # shred the current file
                # please note that on windows, this command
                # actually removes the file from the system
                # and does not move it to recycle bin. 
                os.remove(self._fileName)
              
    def OpenImage(self, fileName):
        "Opens an image file"
        
        if not os.path.isfile(fileName):
            self.MessageDialog("Error: '%s' is not a valid filename" % fileName)
            return
        
        if not pyWiewGlobals.FileValidForRead(fileName):
            self.MessageDialog("Error: invalid extension, %s" % fileName)
            return

        # if any animation thread was running
        # stop it
        if self._pyanim and self._pyanim.isAlive():
            self._pyanim.end()
        
        # reset undo list
        self._undoList=[]
        self.SaveUndoInfo()
        
        if not self.SetImage(fileName):
            self.MessageDialog("Error: Invalid Image!")
            return
            
        self.orig_ht = self._img.GetHeight()
        self.orig_wd = self._img.GetWidth()
        
        # Re-initialize image params
        self.ResetImageParams()
        bmap=self._img.ConvertToBitmap()
        self._depth=bmap.GetDepth()

        self._mainpanel.Clear()
        # Display animations
        id=self.MainMenu.FindMenu('Animation')
        if self._pyanim:
            # Add animation menu item
            if id == wxNOT_FOUND:
                self.AddAnimationMenu(self.MainMenu)
            self._pyanim.SetImageFrame(self)
            self._pyanim.start()
        else:
            # Remove animation menu
            if not id == wxNOT_FOUND:
                self.MainMenu.Remove(id)
            # normal images
            self.DrawImage()

    def MyFit(self):

        self._progsize=true
        self.Fit()
        self._progsize=false
        
    def AutoSize(self, from_menu=false):
        "Manages sizing of windows"

        finalfit = self.ActualFit()
        
        if finalfit is 0:
            self._mainpanel.SetBestSize()
            sizer=wxBoxSizer(wxBOTH)
            sizer.Add(self._mainpanel, 0, 0)
            self.SetSizer(sizer)
            self.MyFit()
            self.SetSizer(None)
            pass
        else:
            pass
              
        # set sizes (for check during fitting images)
        self._W, self._H = self.GetSizeTuple()
        # reset st and end points of mouse
        # if there is no selection, selection defaults
        # to the current window
        self._handler.ResetSelection()
        self.UpdateStatusBar()
        self.SetTitle(self._caption + " - " + pyWiewGlobals.CAPTION_FRAME)
        
    def ActualFit(self):
        """ Return the actual fit value """
        
        finalfit=0
        fit=self._options.GetFit()
        if fit is 3:
            finalfit=fit
            return finalfit

        if self._fitflag == 0:
            finalfit=0
        elif self._fitflag == 1:
            finalfit=1
        elif self._fitflag == 2:
            finalfit = 2

          
        return finalfit
              
    def doSaveImage(self, menu):
        "Saves current image"

        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return
        
        if not os.path.isfile(self._fileName):
            self.MessageDialog("Error: '%s' is not a valid filename" % self._fileName)
            self.SetStatusText("No valid filename")
            return
        
        msg='Save the file ' + self._fileName
        cfmdlg=pyWiewMiscWidgets.pyWiewConfirmDialog(self, wxNewId(), 'Confirmation ')
        
        cfmdlg.SetMsg(msg)
        res=cfmdlg.ShowModal()

        if res == wxID_CANCEL:
            cfmdlg.Destroy()
            return

        cfmdlg.Destroy()
        saved=0
        
        if self._fileName:
            if self._options.GetPilStatus():
                imgsrc = pyWiewGlobals.wxImgToPILImg(self._dispimg)
                format = pyWiewGlobals.PILGetFormat(self._fileName, 1)
                try:
                    imgsrc.save(self._fileName, format)
                    self.SetStatusText('Saved file successfully')
                    saved=1
                except:
                    pass

            if saved==0:
                try:
                    bitmap=self._mainpanel.GetBitmapWindow()
                    bmap=bitmap.GetBitmap()
                    format = pyWiewGlobals.wxGetFormat(self._fileName)
                    bmap.SaveFile(self._fileName, format)
                    saved=1
                except:
                    self.SetStatusText('Error in saving file')
                    return
                
                self.SetStatusText('Saved file successfully')
        else:
            self.SetStatusText('No filename!')

    def SaveImageAsFormat(self, img, format, fileName):
        """ Save the image in the given format.
        The format is a PIL format """

        saved=0
        if self._options.GetPilStatus():
            imgsrc = pyWiewGlobals.wxImgToPILImg(img)
            if imgsrc:
                imgout=imgsrc.copy()
            else:
                self.MessageDialog("Error: Invalid Image!")
                return false
            
            try:
                imgout.save(fileName, format)
                saved=1
            except IOError, e:
                msg = 'Error : ' + str(e)
                self.MessageDialog(msg)
                print msg
                self.SetStatusText('Error in saving file!')
                return false
            except KeyError, e:
                msg = 'Error : ' + str(e)
                self.MessageDialog(msg)
                print msg
                self.SetStatusText('Error in saving file!')
                return false
            except TypeError, e:
                print 'Type error saving image using PIL'

        # PIL could not save the file, try wx
        if saved==0:
            # get a mapping from PIL to wxImage format
            wxformat=pyWiewGlobals.PILToWxFormat(format)
            try:
                bmap=img.ConvertToBitmap()
                bmap.SaveFile(fileName, wxformat)
            except:
                self.SetStatusText('Error in saving file')
                return false

        return true
            

    def doSaveImageAs(self, menu):
        "Saves current image in many formats"
        
        if self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        dir=""
        if self._savedir == "" or not os.path.isdir(self._savedir):
            dir=self._dir
        else:
            dir=self._savedir

        # set hint as current filename
        fileName=(os.path.split(self._fileName))[1]
        fileNameWithoutExt=(os.path.splitext(fileName))[0]
        fileName = self.SelectFileDialog(dir, 1, fileNameWithoutExt)

        if fileName is None:
            self.OpenFileError(fileName)
        else:
            if not pyWiewGlobals.FileValidForWrite(fileName):
                self.MessageDialog("Error: invalid extension,    %s " % fileName)
                return
                
            self.SetMrud(fileName)
            ext = os.path.splitext(fileName)
            fext = ext[1]
            # set saved directory
            self._savedir = (os.path.split(fileName))[0]

            format=pyWiewGlobals.PILGetFormat(fileName, 1)
            print format
            done=self.SaveImageAsFormat(self._dispimg, format, fileName)

            if done:
                self.SetStatusText('Saved file successfully')

    def Draw(self, dc):
        bmap = self._mainpanel.GetBitmap()
        dc.DrawBitmap(bmap, 0, 0, TRUE)
        
    def DrawImage(self, img=None):
        """ Draw image bitmap, manage fitting and resizing """
        
        if img is None and self._set is false:
            self.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return None
        
        self._fitflag=0
        if img:
            newimg=img
        else:
            newimg=self._dispimg

        if not newimg.Ok():
            self.SetStatusText("Image Error")
            return

        posn=(0,0)
        fit=self._options.GetFit()
        # if random fit, get a fit value
        if fit == 4:
            # random fit, generate a number between
            # 0 and 3
            fit=random.randrange(0,4)
              
        fitsmall = self._options.GetSmallImgFit()
        fitlarge = self._options.GetLargeImgFit()
        
        self._fitdone=false

        # fitting large images to screen, takes precedence
        # over everything else
        fitimg=newimg
        if fitlarge or fit is 2:
            if newimg:
                fitimg=self.FitScreen(newimg)
            else:
                fitimg=self.FitScreen()
            # calculate top-left position of image
            # for a centered image
            sw, sh = self.GetSizeTuple()
            imgw, imgh = fitimg.GetWidth(), fitimg.GetHeight()
            posn = float(sw - imgw)/2.0, float(sh - imgh)/2.0
                
        if not self._fitdone:
            if fit is 1 or fitsmall:
                if newimg:
                    fitimg=self.FitImage(newimg)
                else:
                    fitimg=self.FitImage()

            if fit is 3:
                self._fitflag=2
                self.FitWindowScreen()
                # calculate top-left position of image
                # for a centered image
                sw, sh = self.GetSizeTuple()
                imgw, imgw = fitimg.GetWidth(), fitimg.GetHeight()
                posn = float(sw - imgw)/2.0, float(sh - imgh)/2.0               
                
        # Set the current displayed image
        self._dispimg=fitimg
        if not self._animation:
            self._mainpanel.Clear()
            
        self.UpdateBitmap(self._dispimg.ConvertToBitmap(), posn)
            
        self.AutoSize()
        # Set menu item visibility
        self.SetMenuItems()
        
    def doCloseImage(self, event):
        """ Closes current image """

        self._mainpanel.SetBackground(self._options.GetPanelBgColor())
        self.SetTitle('No Image - ' + pyWiewGlobals.CAPTION_FRAME)
        self.SetSize(wxSize(pyWiewGlobals.DW,pyWiewGlobals.DH))

        # if we are running an animation, kill it
        if self._animation:
            if self._pyanim:
                self._pyanim.end()
            self._pyanim=None

        # Remove menu item Animation
        id = self.MainMenu.FindMenu('Animation')
        if id != wxNOT_FOUND:
            self.MainMenu.Remove(id)

        self.ClearStatusBar()
        # Set menu item visibility
        self.SetMenuItems()
        self._mainpanel.Clear()

    def OpenFileError(self, fileName):
        """ Show error in opening a file """
        
        if fileName:
            self.MessageDialog("Error opening file '%s'!" % fileName)

    def MessageDialog(self, text, title="PyWiew Error"):
        "General message dialog"
        
        messageDialog = wxMessageDialog(self, text, title, wxOK | wxICON_INFORMATION)
        messageDialog.ShowModal()
        messageDialog.Destroy()

    def GetImage(self):
        return self._img
    
    def GetMrud(self):
        return self._mrud

    def GetMruu(self):
        return self._mruu
    
    def SetMruu(self, url):
        """ Update the mruu list with the passed url """

        newurl = url
            
        # update mruu list
        new=true
        sz=len(self._mruu)
        
        for x in range(0,sz):
            currurl=self._mruu[x]
                   
            if currurl == newurl:
                new=false
                break

        if new is true:
            if sz <  5:
                self._mruu.append(url)
            else:
                # push back list items
                # 1,2,3,4 <== 2,3,4,5
                self._mruu.pop(0)
                self._mruu.append(url)

            print 'New Url ! - updating mruu...!'
            self.UpdateMruu(url)
            
    def SetMrud(self, fileName):
        """ Update the mrud list based on the fileName (directory of the file)
        Does not update current directory """
        
        filename=""
        if os.name == 'nt':
            filename = string.lower(fileName)
        else:
            filename = fileName

        bdir=os.path.split(fileName)[0]
        sdir=""
        if filename:
            sdir=os.path.split(filename)[0]
        else:
            sdir="."
              
        # update mrud list
        new=true
        sz=len(self._mrud)
        
        for x in range(0,sz):
            checkdir=self._mrud[x]
            if os.name == 'nt':
                checkdir=string.lower(checkdir)
                   
            if checkdir == sdir:
                new=false
                break

        if new is true:
            if sz <  5:
                self._mrud.append(bdir)
            else:
                # push back list items
                # 1,2,3,4 <== 2,3,4,5
                self._mrud.pop(0)
                self._mrud.append(bdir)

            print 'New Directory ! - updating mrud...!'
            self.UpdateMrud(bdir)
              
    
    def SetCurrentDir(self):
        """ Set/get current directory 
        This sets the current mrud also """

        print 'Setting current dir ...'
        dir = (os.path.split(self._fileName))[0]

        if os.name == 'nt':
            dir = string.lower(dir)
            currdir= string.lower(self._dir)
        else:
            currdir = self._dir

        self._files = os.listdir(dir)
        
        if dir != currdir:
            print 'Setting MRUD...'
            self._dir = dir
            self.SetMrud(self._fileName)

        return self._dir
    
    def doClose(self, event):
        """ Respond to the Close menu command"""
        
        # keep data on the clipboard
        print 'Closing...'
        if wxTheClipboard.Open():
            wxTheClipboard.Flush()
            wxTheClipboard.Close()
            
        if self._thread:
            self._thread.end()
        if self._pyanim:
            self._pyanim.end()

        self._img.Destroy()
        self._dispimg.Destroy()
        self._clipimg.Destroy()
        self._mainpanel.Destroy()
        self.Destroy()

        print 'Exiting PyWiew...'
        print 'Done.'

    def doExit(self, event, normalexit=true):
        """ Exit the application """

        print 'Exiting...'
        # keep data on the clipboard
        if wxTheClipboard.Open():
            wxTheClipboard.Flush()
            wxTheClipboard.Close()

        # Destroy bitmaps, images etc
        if normalexit:
            self._config.WriteConfigFile()

        # clean up
        if self._thread:
            self._thread.end()
        if self._pyanim:
            self._pyanim.end()          

        self._img.Destroy()
        self._dispimg.Destroy()
        self._clipimg.Destroy()
        self._mainpanel.Destroy()

        self.Destroy()
        print 'Exiting PyWiew...'
        print 'Done.'
         
    def SelectFileDialog(self, defaultDir=None, mode = 0, defaultFile=None, wildCard=None):
        """ Select file dialog for browsing images 
        This function is called for both opening and saving files """
        
        if defaultDir == None:
            defaultDir = "."
        if defaultFile == None:
            defaultFile = ""
        if wildCard == None:
            if mode is 0:
                wildCard=pyWiewGlobals.GetFileFilterRead()
            else:
                wildCard=pyWiewGlobals.GetFileFilterWrite()

        fileName = None
        if mode is 0:
            title = "Open File..."
            fileDialog = wxFileDialog(self, title, defaultDir, defaultFile, wildCard, wxOPEN|wxMULTIPLE)

        else:
            title = "Save File..."
            fileDialog = wxFileDialog(self, title, defaultDir, defaultFile, wildCard, wxSAVE|wxOVERWRITE_PROMPT)

        result = fileDialog.ShowModal()
        if result == wxID_OK:
            fileName = fileDialog.GetPath()

        fileDialog.Destroy()
        return fileName

    def doHelpAbout(self, event):
        "Show help about"
        
        self.MessageDialog(pyWiewGlobals.ABOUT_TEXT, "About PyWiew")
        
    def Display(self):
        "Show current frame and set its focus"
        
        wxFrame.Show(self, TRUE)
        self.SetFocus()
         
    def Update(self):
        """ Called when user has made changes to options and we need an update """
         
        # change style
        if not self._options.ResizeAllowed():
            style = wxSYSTEM_MENU | wxCAPTION 
        else:
            style = wxDEFAULT_FRAME_STYLE
              
        if self._options.FrameOnTop():
            style = style | wxSTAY_ON_TOP

        self.SetWindowStyle(style)
        # toolbar/statusbar
        tbchng=false
        stbchng=false

        showtoolbar = self._options.ShowtoolBar()
        if self._tb is None:
            self.AddToolBar()
              
        if self._tb:
            tbshwn=self._tb.IsShown()
            self._tb.Show(showtoolbar)
            # change in show flag ?
            if showtoolbar != tbshwn:
                tbchng=true

        sbshwn = self.GetStatusBar().IsShown()
        showstbar = self._options.ShowStatusBar()

        if self.GetStatusBar():
            self.GetStatusBar().Show(showstbar)
            # change in show flag ?
            if showstbar != sbshwn:
                stbchng = true

        # panel bg color
        bgcolor = self._options.GetPanelBgColor()
        self._mainpanel.SetBackground(bgcolor)
            
        # update fit options
        fit=self._options.GetFit()
        self.ToggleViewMenu(fit)
        
        smallfit = self._options.GetSmallImgFit()
        self.ToggleViewMenu2(1,smallfit)
        largefit = self._options.GetLargeImgFit()
        self.ToggleViewMenu2(2,largefit)
        # display time ?
        tstatus = self._options.ShowTime()
        if not tstatus:
            if self._thread.isAlive():
                stbar=self.GetStatusBar()
                stbar.SetStatusText('',5)
                self._thread.stoptime()
            else:
                pass
        else:
            if self._thread and self._thread.isAlive():
                self._thread.starttime()
            else:
                self._thread=helperThread("Helper", self)
                self._thread.start()

        self._mainpanel.Refresh()

        # change in visibility of stbar/toolbar
        # so re-fit needed
       # #  if tbchng or stbchng:
# #             # self._mainpanel.Fit()
# #             self.Fit()
            
        if tbchng:
            # This is dirty coding!
            # tool bar became invisible, move panel up by height of toolbar
            wp, hp=self._mainpanel.GetSizeTuple()
            posn = self._tb.GetPosition()
            if not self._tb.IsShown():
                # new dimensions, origin same as toolbar position, height goes up
                # by toobar height
                wt, ht = self._tb.GetSizeTuple()                
                self._mainpanel.SetDimensions(posn.x, posn.y, wp, hp + ht)
            else:
                # new dimensions, origin goes down by toolbar height, height goes
                # down by toolbar height
                wt, ht = self._tb.GetSizeTuple()
                self._mainpanel.SetDimensions(posn.x, posn.y + ht, wp, hp - posn.y)

        if stbchng:
            stbar=self.GetStatusBar()
            wt, ht = stbar.GetSizeTuple()
            wp, hp=self._mainpanel.GetSizeTuple()
            if not stbar.IsShown():
                # new dimensions, origin remains same, height goes up by size
                # of statusbar
                self._mainpanel.SetSize(wxSize(wp, hp + ht))
            else:
                # new dimensions, origin remains same, height goes down by size
                # of statusbar
                self._mainpanel.SetSize(wxSize(wp, hp - ht))

            
        self.InitNetwork()
        if self._set:
            img=self.ApplyCurrentRotflip(self.doZoom())
            self.DrawImage(img)
              
    def UpdateStatusBar(self):
        """ Update the fields of the status bar """
        
        print 'Updating status bar...'
        showst = self._options.ShowStatusBar()
        stbar = self.GetStatusBar()
        
        if not showst:
            return
        elif not self._set:
            stbar.SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
            return

        stbar.Refresh()
         
        # zeroth field is for messages, display image fit information here
        finalfit = self.ActualFit()
        if finalfit is 0:
            statustxt = "No Fit"
        elif finalfit is 1:
            statustxt = "Fit Window"
        elif finalfit is 2 or finalfit is 3:
            statustxt="Fit Screen"

        stbar.SetStatusText(statustxt, 0)
        # first field image size/depth
        w,h,d=self._img.GetWidth(), self._img.GetHeight(), self._depth
        text='\t' + str(w) + 'x' + str(h) + 'x' + str(d) + '\t'
        stbar.SetStatusText(text,1)
        # second field ,percentage of current img size to orig size
        # This value is the ratio of dimension of displayed image to
        # original image
        wd,hd=self._dispimg.GetWidth(), self._dispimg.GetHeight()
        w,h = self._img.GetWidth(), self._img.GetHeight()
        
        if wd and hd and w and h:
            sizeperc = int(math.ceil(100.0*(float(float(wd)/float(w) + float(hd)/float(h)))/2.0))
            text='\t' + str(sizeperc) + '%' + '\t'
            stbar.SetStatusText(text,2)
            
        # third field, file size of the image
        if os.path.exists(self._fileName):
            fstat=os.stat(self._fileName)
            if fstat:
                sz1=math.ceil(float(fstat.st_size*10/1024.0))
                sz2=float(sz1/10)
                text='\t' + str(sz2) + 'KB' + '\t'
        else:
            text='\t(No File)\t'
              
        stbar.SetStatusText(text,3)
        # fourth field, position of file in directory
        # Note: size of directory also includes non-images
        if os.path.exists(self._fileName):
            try:
                if self._dir != '':
                    files = os.listdir(self._dir)
                    filename = (os.path.split(self._fileName))[1]
                    posn=files.index(filename) + 1
            except:
                posn=1

            total=len(os.listdir(self._dir))
            text = '\t' + str(posn) + '/' + str(total) + '\t'
        else:
            text='\t0/0\t'
              
        stbar.SetStatusText(text,4)
        # fifth field is the current local time whose
        # display is managed by a separate thread
        
        # sixth field is the load time for the image
        text='\tLoaded in ' + str(self._loadtime) + ' s\t'
        stbar.SetStatusText(text, 6)
        
    def ClearStatusBar(self):
        """ Clear the status bar fields """
        
        for x in range(0, 5):
            self.GetStatusBar().SetStatusText('', x)

        # clear sixth field also, but keep fifth field(time)
        self.GetStatusBar().SetStatusText('', 6)     
        self.GetStatusBar().SetStatusText(pyWiewGlobals.NO_IMAGE_ERROR)
        
    def HandleException(self):
        """ Handle exceptions that occur while we are in the middle
        of something """
        
        # Manage  busy cursors if any, otherwise the
        # window is left with a busy cursor which does not
        # look nice
        if wxIsBusy():
            wxBeginBusyCursor()
            wxEndBusyCursor()
            self.SetCursor(wxStockCursor(wxCURSOR_ARROW))
             
        self.DrawImage()                     

class helperThread(threading.Thread):

    def __init__(self, name, frame):

        print 'Starting timer thread...'
        threading.Thread.__init__(self,None,None,name)
        self._flag=1
        self._frame = frame
        tz = (time.tzname)[0]
        # take only the first character of each word in tz
        self._tz=''
        words=string.split(tz)
        for x in words:
            self._tz += x[0]

        self._hr='-1'
        self._min='-1'
        self._showtime=1

    def run(self):

        options=self._frame.GetOptionsObject()
        while self._flag:
            
            if options.ShowTime():

                timestr=str(time.strftime("%H:%M",time.localtime()))
                timelst=string.split(timestr, ':')
                hr=timelst[0]
                min=timelst[1]
                if min != self._min:
                    self._min=min
                    self._hr=hr
                    if self._showtime:
                        self.write()
                    
                # check for clipboard data and enable Edit->Paste menu item
                if self._frame.IsClipboardEmpty(true):
                    self._frame.SetEditMenuItem('Paste ', false)
                else:
                    self._frame.SetEditMenuItem('Paste ', true)

                time.sleep(1)


    def write(self):
        text='\t' + self._hr + ':' + self._min +  ' ' + self._tz + '\t'
        self._frame.SetStatusText(text, 5)

    def stoptime(self):
        self._showtime=0

    def starttime(self):
        self._showtime=1
        
    def end(self):
        self._flag=0
         
# ----------------------------------------------------------------------------
class pyWiewApp(wxApp):
    """ The main PyWiew application object.
    """

    def OnInit(self):
        """Initialise the application.
        """

        wxInitAllImageHandlers()
        self.SetUseBestVisual(TRUE)

        options = pyWiewOptions.pyWiewImageOptions()
        if options.SingleInstance():
            print 'Checking for another instance'
            # single instance checker
            name='PyWiew-' + wxGetUserId()
            singchecker=wxSingleInstanceChecker(name)
            if singchecker.IsAnotherRunning():
                print 'Error: Another instance is already running!'
                wxLogMessage('Error: Another PyWiew instance is already Running!')
                return FALSE
        
        title= 'No Image - ' + pyWiewGlobals.CAPTION_FRAME
        global _frame
        _frame = pyWiewMainFrame(None, -1, title, size=(pyWiewGlobals.DW,pyWiewGlobals.DH))
        if _frame is None:
            return FALSE

        self.SetTopWindow(_frame)
        self.SetAppName("PyWiew")
        print 'Argument length = ', len(sys.argv)
        if len(sys.argv) > 1:
            # Load the file(s) specified on command line
            # create filename argument
            filename=''
            arglist=sys.argv[1:]
            arglen=len(arglist)
            for x in range(0, arglen-1):
                filename += arglist[x]
                filename += ' '
                filename += arglist[arglen-1]

            if filename:
                _frame.OpenImage(filename)
              
        return TRUE
              
def main():
    """ Start up the pyWiew application.
    """
    global _app
    
    # Redirect python exceptions to a log file.
    sys.stderr = pyWiewGlobals.ExceptionHandler('pywiew.err')

    # Redirect debug messages to a log file.
    #sys.stdout = pyWiewGlobals.DebugMessageHandler('pywiew.log')

    # Create and start pyWiew application.
    _app = pyWiewApp(0)
    _app.MainLoop()

if __name__ == "__main__":
    main()
