""" pyWiewConfig.py --- class for reading/writing config based setting for pyWiew #
    Copyright 2002-2003 Anand B Pillai <abpillai@Lycos.com>#

 Modified Sat Feb 22 18:27:53 2003 Added config options for read/write internet
                                   config stuff. Username/password are saved in
                                   obfuscated format.
 Fri Apr 11 15:05:57 2003 : Replaced int(string) with Read() for config.

                                   """

import os
import pyWiewCrypt
from pyWiewGlobals import *
from pyWiewOptions import *
from wxPython.wx import *

class pyWiewConfig:

    def __init__(self, viewFrame, label):
        self._frame = viewFrame
        self._config = wxConfig(label)

    def ReadConfigFile(self):
        "Open the config file and update local config variables"

        print "Read Config file..."
        #fill mrud list
        if self._config is None:
            return

        if self._config.Exists("paths/mruds"):
            pathlist = self._config.Read("paths/mruds")
            #Most recently used directories
            #parse pathlist
            paths = string.split(pathlist, ';')
            for x in range(0, len(paths)):
                if os.path.exists(paths[x]) and os.path.isdir(paths[x]):
                    self._frame._mrud.append(paths[x])

        if self._config.Exists("paths/mruus"):
            urllist = self._config.Read("paths/mruus")
            #Most recently used URLs
            #parse urllist
            urls = string.split(urllist, ';')
            for x in range(0, len(urls)):
                if urls[x] != '':
                    self._frame._mruu.append(urls[x])

        #other options
        _options = self._frame.GetOptionsObject()

        #Appearance options
        if self._config.Exists("Version"):
            _options.SetVersion(self._config.Read("version"))
        if self._config.Exists("graphic/windowontop"):
            _options.SetFrameOnTop(int(self._config.Read("graphic/windowontop")))
        if self._config.Exists("graphic/startupbitmap"):
            _options.SetstartupBitmapStatus(int(self._config.Read("graphic/startupbitmap")))
        if self._config.Exists("graphic/showtoolbar"):
            _options.SetShowtoolBar(int(self._config.Read("graphic/showtoolbar")))
        if self._config.Exists("graphic/showstatusbar"):
            _options.SetShowstatusBar(int(self._config.Read("graphic/showstatusbar")))
        if self._config.Exists("graphic/panelbgcolor"):
            colstr = self._config.Read("graphic/panelbgcolor")
            rgb=string.split(colstr,',')
            r=int(rgb[0])
            g=int(rgb[1])
            b=int(rgb[2])
            _options.SetPanelBgColor(wxColour(r,g,b))
        if self._config.Exists("graphic/showTime"):
            _options.SetShowTime(int(self._config.Read("graphic/showTime")))
        if self._config.Exists("graphic/resize"):
            _options.SetresizeAllowed(int(self._config.Read("graphic/resize")))
        if self._config.Exists("image/imagefit"):
            _options.SetFit(int(self._config.Read("image/imagefit")))
        if self._config.Exists("image/smallimgfit"):
            _options.SetSmallImgFit(int(self._config.Read('image/smallimgfit')))
        if self._config.Exists("image/largeimgfit"):
            _options.SetLargeImgFit(int(self._config.Read("image/largeimgfit")))
        if self._config.Exists("image/lastpilstatus"):
            _options.SetPilStatus(int(self._config.Read("image/lastpilstatus")))
        if self._config.Exists("image/thumbwidth"):
            _options.SetThumbnailWidth(int(self._config.Read("image/thumbwidth")))
        if self._config.Exists("slideshow/ssgap"):
            _options.SetShowDuration(int(self._config.Read("slideshow/ssgap")))
        if self._config.Exists("slideshow/ssloop"):
            _options.SetShowLoop(int(self._config.Read("slideshow/ssloop")))
        if self._config.Exists("slideshow/ssrandom"):
            _options.SetShowRandom(int(self._config.Read("slideshow/ssrandom")))
        if self._config.Exists("slideshow/ssfile"):
            _options.SetShowFile(int(self._config.Read("slideshow/ssfile")))
        if self._config.Exists("slideshow/ssdirn"):
            _options.SetShowDirection(int(self._config.Read("slideshow/ssdirn")))

        if self._config.Exists("graphic/font"):
            fontstr = self._config.Read("graphic/font")
            #extract font data
            data = string.split(fontstr, ';')
            if len(data) == 6:
                font=wxFont(int(data[0]), int(data[1]), int(data[2]), int(data[3]), int(data[4]), data[5])
            else:
                #default font
                font=wxFont(9, wxDEFAULT, wxNORMAL, wxNORMAL, false, 'Arial')
            _options.SetFont(font)

        if self._config.Exists("graphic/fontcolor"):
            colstr = self._config.Read("graphic/fontcolor")
            rgb=string.split(colstr,',')
            r=int(rgb[0])
            g=int(rgb[1])
            b=int(rgb[2])
            _options.SetFontColor(wxColour(r,g,b))

        #Read internet config
        status=0
        if self._config.Exists('internet/connection'):
            status=int(self._config.Read('internet/connection'))
        _options.SetInternetStatus(status)
        #Proxies
        proxies=""
        if self._config.Exists('internet/proxies'):
            proxiesobf=self._config.Read('internet/proxies')
            #deobfuscate the proxy string
            proxies=pyWiewCrypt.xorunobfuscate(proxiesobf)
            #parse the proxy string
            #the string is of the format, "protocol#server;..."
            proxylist=string.split(proxies,';')
            for x in range(0, len(proxylist)):
                proxy=proxylist[x]
                protocol, server=string.split(proxy, "#")

                if protocol=="http":
                    _options.SetHttpProxy(server)
                elif protocol=="https":
                    _options.SetHttpsProxy(server)
                elif protocol=="ftp":
                    _options.SetFtpProxy(server)
                else:
                    pass

        #proxyauth
        proxyauth=0
        if self._config.Exists('internet/proxyauth'):
            proxyauth=int(self._config.Read('internet/proxyauth'))
        _options.SetProxyNeedsAuth(proxyauth)
        #proxusername
        proxyusr=""
        if self._config.Exists('internet/proxyuser'):
            try:
                proxyusr=pyWiewCrypt.xorunobfuscate(self._config.Read('internet/proxyuser'))
            except:
                pass
        _options.SetProxyUser(proxyusr)
        proxypwd=""
        if self._config.Exists('internet/proxypasswd'):
            try:
                proxypwd=pyWiewCrypt.xorunobfuscate(self._config.Read('internet/proxypasswd'))
            except:
                pass
        _options.SetProxyPasswd(proxypwd)

    def WriteConfigFile(self):
        """ Write local config variables into config file. This should
        be called only at the time of exiting application """

        #Most recently used directories
        pathlist = ""
        #write only first 5 items
        mrud=self._frame.GetMrud()
        sz = len(mrud)
        if sz>5:
            sz=5
        for x in range(0, sz):
            dir = mrud[x]
            if os.path.isdir(dir):
                pathlist = pathlist +  dir
                pathlist += ';'

        self._config.Write("paths/mruds", pathlist)

        #Most recently visited urls
        urllist = ""
        #write only first 5 items
        mruu=self._frame.GetMruu()
        sz = len(mruu)
        if sz>5:
            sz=5
        for x in range(0, sz):
            url = mruu[x]
            if url != '':
                urllist = urllist +  str(url)
                urllist += ';'

        self._config.Write("paths/mruus", urllist)

        _options = self._frame.GetOptionsObject()
        #version
        self._config.Write("version", "1.0")
        #release name
        self._config.Write("productcode", "Sithara")
        #windowontop
        self._config.Write("graphic/windowontop", str(_options.FrameOnTop()))
        #startup bitmap
        self._config.Write("graphic/startupbitmap", str(_options.ShowstartupBitmap()))
        #show toolbar
        self._config.Write("graphic/showtoolbar", str(_options.ShowtoolBar()))
        #show statusbar
        self._config.Write("graphic/showstatusbar", str(_options.ShowStatusBar()))

        #panel color
        col=_options.GetPanelBgColor()
        r,g,b=col.Red(),col.Green(),col.Blue()
        colstr=str(r)+','+str(g)+','+str(b)
        self._config.Write("graphic/panelbgcolor", colstr)

        #display time
        self._config.Write("graphic/showtime", str(_options.ShowTime()))
        #resize
        self._config.Write("graphic/resize", str(not _options.ResizeAllowed()))
        #Imagefit
        self._config.Write("image/imagefit", str(_options.GetFit()))
        #small img fit
        self._config.Write("image/smallimgfit", str(_options.GetSmallImgFit()))
        #large img fit
        self._config.Write("image/largeimgfit", str(_options.GetLargeImgFit()))
        #last pil status
        self._config.Write("image/lastpilstatus", str(_options.GetPilStatus()))
        #thumbnail width
        self._config.Write("image/thumbwidth", str(_options.GetThumbnailWidth()))
        #slideshow gap
        self._config.Write("slideshow/ssgap", str(_options.ShowDuration()))
        #slideshow loop
        self._config.Write("slideshow/ssloop", str(_options.ShowLoop()))
        #slideshow random
        self._config.Write("slideshow/ssrandom", str(_options.ShowRandom()))
        #slideshow file
        self._config.Write("slideshow/ssfile", str(_options.ShowFile()))
        #slideshow direction
        self._config.Write("slideshow/ssdirn", str(_options.ShowDirection()))

        #font data
        font=_options.Font()
        fontstr= str(font.GetPointSize()) + ';' + str(font.GetFamily()) + ';' + str(font.GetStyle()) + ';' +     \
                 str(font.GetWeight()) + ';' + str(font.GetUnderlined()) + ';' + str(font.GetFaceName()) 
        self._config.Write("graphic/font", fontstr)

        #font color
        col=_options.FontColor()
        r,g,b=col.Red(),col.Green(),col.Blue()
        colstr=str(r)+','+str(g)+','+str(b)
        self._config.Write("graphic/fontcolor", colstr)

        #Internet options
        #internet status
        self._config.Write("internet/connection", str(_options.GetInternetStatus()))
        #proxy list
        #the proxy server string contains the port also
        httpproxy=_options.GetHttpProxy()
        sslproxy=_options.GetHttpsProxy()
        ftpproxy=_options.GetFtpProxy()
        #create the string
        proxystring="http#"+httpproxy+";"+"https#"+sslproxy+";"+"ftp#"+ftpproxy
        #obfuscate the proxy string
        proxystringobf=pyWiewCrypt.xorobfuscate(proxystring)
        self._config.Write("internet/proxies",proxystringobf)
        #proxy auth
        self._config.Write("internet/proxyauth",str(_options.ProxyNeedsAuth()))
        #proxy username
        proxyuser=_options.GetProxyUser()
        if proxyuser != "":
            #cyclic xor/base64 encryption
            usernamecomp=pyWiewCrypt.xorobfuscate(proxyuser)
        else:
            usernamecomp=""
        self._config.Write("internet/proxyuser", usernamecomp)
        #proxy passwd
        proxypwd=_options.GetProxyPwd()
        if proxypwd != "":
            #cyclic xor/base64 encryption
            passwdcomp=pyWiewCrypt.xorobfuscate(proxypwd)
        else:
            passwdcomp=""
        self._config.Write("internet/proxypasswd", passwdcomp)

