"""pyWiewEvtHandlers.py --- class used to handle mouse/keyboard events by pyWiew#

 Copyright 2002-2003 Anand B Pillai <abpillai@Lycos.com>

 Wed Mar 26 19:42:08 2003: Removed redundant code in Multiimagefilter.
                           

 """


import random
from wxPython.wx import *
import pyWiewGlobals
from pyWiewSlideshow import *
from pyWiewRawFilter import *
from pyWiewWidgets import *
import pyWiewMiscWidgets
import pyWiewPILFilter


class pyWiewEvtHandler:

    def __init__(self, viewFrame):

        self._frame = viewFrame
        #mouse selection starting point
        self._stpt = wxPoint(0,0)
        #mouse selection end point
        self._etpt  = wxPoint(0,0)
        #to cache mouse selection start point between
        #selections
        self._svpt  = wxPoint(0,0)
        #boolean for mouse selection
        self._selected = false
        #boolean for mouse left clicking
        self._lclicked = false
        #slideshow on or off
        self._slideshowon = 0
        #slideshow thread
        self._slidethread = None

    def SlideshowRunning(self):
        return self._slideshowon

    def OnMouseEvent(self, event):
        "Handles mouse events for main window frame"

        if event is not None:
            self._frame.SetCursor(wxStockCursor(wxCURSOR_ARROW))
            canvas = self._frame.GetCanvas()
            wdc = wxWindowDC(canvas) 
            wdc.SetLogicalFunction(wxXOR)
            wbrush = wxBrush(wxColour(255,255,255), wxTRANSPARENT)
            wpen = wxPen(wxColour(200, 200, 200), 1, wxSOLID)
            wdc.SetBrush(wbrush)
            wdc.SetPen(wpen)

            if event.LeftDown():
                canvas.Refresh()
                self._stpt = event.GetPosition()
                cur = wxStockCursor(wxCURSOR_CROSS)  # left button clicked, change cursor to cross
                self._frame.SetCursor(cur)
                self._svpt = self._stpt
                self._selected = false
                self._lclicked = true

            elif event.Dragging():  # draw rubberbanded rectangle

                if self._lclicked:
                    wdc.ResetBoundingBox()
                    wdc.BeginDrawing()
                    w = (self._svpt.x - self._stpt.x)
                    h = (self._svpt.y - self._stpt.y)
                    wdc.DrawRectangle(self._stpt.x, self._stpt.y, w,h)  # this erases previous rectangle
                    self._etpt =  event.GetPosition()

                    w = (self._etpt.x - self._stpt.x)
                    h = (self._etpt.y - self._stpt.y)
                    wdc.SetClippingRegion(self._stpt.x, self._stpt.y, w,h)
                    wdc.DrawRectangle(self._stpt.x, self._stpt.y, w,h) # this draws current rectangle
                    wdc.EndDrawing()
                    self._svpt = self._etpt # save current endpoint

            elif event.LeftUp():
                self._frame.SetCursor(wxStockCursor(wxCURSOR_ARROW)) # left button up, change cursor back
                self._selected = true
                self._lclicked = false # end of clicking

            elif event.MiddleDown():
                self._frame.doFitWindow(event)

    def OnKeyEvent(self, event):
        """ Handle keyboard events """

        kc=event.GetKeyCode()
        if kc == ord('/'):
            self._frame.OnZoomSelect(event)
        elif kc == ord('s') or kc == ord('S'):
            self.doSlideShow(event)
        elif kc == ord('i') or kc == ord('I'):
            self._frame.ToggleFitWindowScreen(event)
        elif kc == ord('f') or kc == ord('F'):
            self._frame.ToggleFitScreen(event)
        elif kc == ord('l') or kc == ord('L'):
            self._frame.doFitLargeImage(event)
        elif kc == ord('t') or kc == ord('T'):
            self._frame.doFitSmallImage(event)
        elif kc == ord('w') or kc == ord('W'):
            self._frame.doFitImage(event)
        elif kc == ord('n') or kc == ord('N'):
            self._frame.doFitWindow(event)
        elif kc == 392:
            wait=wxBusyCursor()
            self._frame.OnZoomIn(event)
        elif kc == 394:
            wait=wxBusyCursor()
            self._frame.OnZoomOut(event)
        elif kc == WXK_RIGHT:
            self._frame.OnRotateRt(event)
        elif kc == WXK_LEFT:
            self._frame.OnRotateLt(event)
        elif kc == ord('H') or kc == ord('h'):
            self._frame.OnHorFlip(event)
        elif kc == ord('V') or kc == ord('v'):
            self._frame.OnVerFlip(event)                
        elif kc == WXK_PRIOR:
            self._frame.DirWalkUp(event)
        elif kc == WXK_NEXT:
            self._frame.DirWalkDown(event)
        elif kc == WXK_ESCAPE:
            self._frame.doExit(event)
        elif kc == WXK_HOME:
            self._frame.ShowFirstFile(event)
        elif kc == WXK_END:
            self._frame.ShowLastFile(event)
        else:
            print "Got an event not handled ! ", kc

    def OnFilterEvent(self, event):
        "Event handler for filter menu of main frame"

        # think i should depend more on self._set flag!
        if not self._frame.DispImgOk():
            self._frame.SetStatusText(NO_IMAGE_ERROR)
            return

        if event is not None:
            wait=wxBusyCursor()

            id = event.GetId()
            imgsrc = pyWiewGlobals.wxImgToPILImg(self._frame._dispimg)

            img = wxEmptyImage()
            self._frame.SaveUndoInfo()

            if id is 21: # blur
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doBlurEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            elif id is 22:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doSharpenEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            elif id is 23:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doSmoothEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 24:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doContourEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 25:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doFindEdges(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            elif id is 26:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doEdgeEnhance(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            elif id is 27:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doEmbossEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 28:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doDetailEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 29:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doNegEffect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 32:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.GrayScale(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 31:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Color Reduction Factor')
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return

                imgfilter=pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.ColorEnhance(event,val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 33:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Sharpness Reduction Factor')
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.SharpnessEnhance(event,val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()

            elif id is 34:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Contrast Factor',(0, 500), 100)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.ContrastEnhance(event,val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()       
                except:
                    self._frame.HandleException()

            elif id is 35:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Brightness Reduction Factor', (0, 100), 100)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return

                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.Brighten(event,val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()    
                except:
                    self._frame.HandleException()

            elif id is 36:
                w,h=imgsrc.size
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Pixel Offset', (0, w), float(w)/2, false)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    offset=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return

                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.HorizontalWrap(event, offset)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()    
                except:
                    self._frame.HandleException()

            elif id is 37:

                w,h=imgsrc.size
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Pixel Offset', (0, h), float(h)/2, false)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    offset=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return

                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.VerticalWrap(event, offset)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()    
                except:
                    self._frame.HandleException()

            elif id is 38:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'redchn',())
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 39:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'greenchn',())
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 41:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'bluechn',())
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 42:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'switch',(1,))
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 43:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'switch',(2,))
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 44:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'switch',(-1,))
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 45:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'switch',(23,))
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 46:
                filter=pyWiewRawFilter("Filter", self._frame, self._frame._dispimg,'switch',(12,))
                filter.start()
                filter.join()
                img=filter.GetImage()
                self._frame.SetDisplayImage(img)
                self._frame.DrawImage()
            elif id is 47:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(),'Select Cutoff Percent', (0, 100), 10, false)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.AutoContrast(val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()    
                except:
                    self._frame.HandleException()

            elif id is 48:
                dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Threshold Pixel', (0, 255), 128, false)
                res=dlg.ShowModal()
                if res == wxID_OK:
                    val=dlg.GetValue()
                    dlg.Destroy()
                else:
                    self._frame.SetStatusText('Cancelled !')
                    return
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.Solarize(val)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()    
                except:
                    self._frame.HandleException()
            elif id is 49:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.doGraniteEtch(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            elif id is 50:
                imgfilter = pyWiewPILFilter.pyWiewImageFilter(imgsrc)
                try:
                    img = imgfilter.do3DFeatDetect(event)
                    self._frame.SetDisplayImage(img)
                    self._frame.DrawImage()
                except:
                    self._frame.HandleException()
            else:
                self._frame.SetStatusText('Error: Cannot find filter for selection')
                return          

    def AddText(self, event):
        "This function annotates images"

        posn = self._frame.GetPosition()
        sz = self._frame.GetSize()
        w, h = sz.GetWidth(), sz.GetHeight()

        editbox = pyWiewEditTextFrame(self._frame)
        result = editbox.ShowModal()

        if result == wxID_OK:
            _options = self._frame.GetOptionsObject()
            data = editbox.GetAllData()
            editbox.Destroy()
            text =  data[0] #editbox.GetTextData()
            trans = data[1] #editbox.GetTransparency()
            #editbox.Destroy()

            box = self.GetCurrentSelection()
            self.ClearSelection()

            # list of strings 
            usrtext = string.split(text, "\n")
            font = _options.Font()
            col = _options.FontColor()
            canvas = self._frame.GetCanvas()
            dc=wxClientDC(canvas)
            dc.DrawBitmap(canvas.GetBitmap(),0,0)
            dc.SetClippingRegion(box[0], box[1], box[2] - box[0], box[3] - box[1])
            dc.SetFont(font)
            dc.SetTextForeground(col)
            dc.SetLogicalFunction(wxXOR)
            if trans is true:
                wbrush = wxBrush(wxColour(255,255,255), wxTRANSPARENT)
            else:
                wbrush = wxBrush(wxColour(255,255,255), wxSOLID)
                dc.SetBackground(wbrush)

            wpen = wxPen(wxColour(200, 200, 200), 1, wxSOLID)
            dc.SetBrush(wbrush)
            dc.SetPen(wpen)

            # To draw multiline text
            #dc.BeginDrawing()
            x,y = box[0], box[1]
            if trans is false:
                dc.SetLogicalFunction(wxCOPY)
                dc.DrawRectangle(x,y,box[2]-box[0], box[3]-box[1])

            for t in range(0, len(usrtext)):
                subtext = usrtext[t]
                dc.DrawText(subtext, x,y)
                # get text extent, to position next text
                pos = dc.GetTextExtent(subtext)
                # push next text by y-extent of current text
                y = y + pos[1]

            #dc.EndDrawing()
            memory = wxMemoryDC()
            #Added this code to update only region
            #that need to be painted
            reg=canvas.GetUpdateRegion()
            rect=reg.GetBox()

            x,y=canvas.GetClientSizeTuple()
            bmap = canvas.GetBitmap()
            memory.SelectObject(bmap)
            dc.SetLogicalFunction(wxCOPY)
            memory.Blit( 0,0, rect.width, rect.height, dc, 0,0) 
            memory.SelectObject( wxNullBitmap)

            #update bitmap in a new thread
            self._frame.UpdateBitmap(bmap)
            # update current image
            self._frame.SetDisplayImage(wxImageFromBitmap(bmap))

        else:
            print "Frame killed!"

    def doSlideShow(self,event):
        "Starts/stops a slideshow"

        print 'Starting slideshow ...'
        _options = self._frame.GetOptionsObject()
        if not self._frame.ImageSet():
            self._frame.SetStatusText(NO_IMAGE_ERROR)
            self._frame.ToggleSlideMenu()
            return

        if not _options.ShowFile():
            _options.SetShowFilenameCaption(false)

        if self._slidethread and self._slidethread.isAlive():
            #To untoggle the slideshow menuitem of caller
            self._frame.ToggleSlideMenu()
            self._frame.SetStatusText('Stopping Slideshow...')
            self._slidethread.Terminate()
            self._frame.SetStatusText('Slidshow Ended.')
            self._frame.SetRandom(false)
            self._slideshowon=false

        else:
            #no slideshow, so start one
            self._frame.SetStatusText('Starting Slideshow...')
            self._slideshowon = true
            #gap is in milliseconds
            gap = _options.ShowDuration()
            gapsec = float(float(gap)/1000)
            print 'Slideshow gap is ', gapsec
            loop = _options.ShowLoop()
            print 'Loop value is ', loop
            randomness = _options.ShowRandom()
            print 'Randomness = ', randomness
            direction = _options.ShowDirection()
            print 'Direction = ',direction

            if randomness:
                self._frame.SetRandom(true)
            else:
                self._frame.SetRandom(false)
            if direction == 0:
                self._slidethread=pyWiewSlideThread("Slideshow", self._frame.DirWalkDown, self._frame, (gapsec,loop))
                self._slidethread.start()
            elif direction == 1:
                self._slidethread=pyWiewSlideThread("Slideshow", self._frame.DirWalkUp, self._frame, (gapsec,loop))
                self._slidethread.start()

    def OnDropFiles(self,event):
        "Open dropped files"

        file = event.GetFiles()
        self._frame.OpenImage(file[0])

    def OnToolClick(self, event):
        id=event.GetId()

        if id == 1000:
            self._frame.doOpenFile(event)
        elif id == 1001:
            self._frame.MakeNewImage(event)
        elif id == 1002:
            self._frame.doSaveImage(event)
        elif id == 1003:
            self._frame.doCloseImage(event)
        elif id == 1004:
            self._frame.doCopyImage(event)
        elif id == 1005:
            self._frame.doCutImageSel(event)
        elif id == 1006:
            self._frame.doPasteImage(event)
        elif id == 1007:
            self._frame.doUndo(event)
        elif id == 1008:
            self._frame.doRedo(event)
        elif id == 1009:
            self._frame.DirWalkDown(event)
        elif id == 1010:
            self._frame.DirWalkUp(event)
        elif id == 1011:
            self._frame.OnZoomIn(event)
        elif id == 1012:
            self._frame.OnZoomOut(event)
        elif id == 1013:
            self._frame.ScreenShot(event)
        else:
            pass

    def OnToolRClick(self, event):
        pass

    def GetCurrentSelection(self,mode=0):
        "Return a box of the current mouse selection"

        #if there is no selection, selection defaults to
        #current viewport
        if mode == 0:
            "Return a 4 tuple "
            left = wxPoint(0,0)
            right = wxPoint(0,0)
            if self._etpt.y > self._stpt.y:
                right = self._etpt
                left = self._stpt
            elif self._etpt.y < self._stpt.y:
                right = self._stpt
                left = self._etpt

            box = (left.x, left.y, right.x, right.y)
        else:
            "Return a 2 tuple of left hand upper corner"
            left = wxPoint(0,0)

            if self._etpt.y > self._stpt.y:
                left = self._stpt
            elif self._etpt.y < self._stpt.y:
                left = self._etpt
                box = (left.x, left.y)

        return box

    def ResetSelection(self):
        "Resets the mouse selection"

        self._stpt = wxPoint(0,0)
        canvas = self._frame.GetCanvas()
        sz= canvas.GetSize()
        w,h=sz.GetWidth(), sz.GetHeight()
        self._etpt = wxPoint(w,h)

    def ClearSelection(self):
        "Clears the current mouse selection"

        box = self.GetCurrentSelection()
        canvas = self._frame.GetCanvas()

        dc=wxClientDC(canvas)

        w = box[2] - box[0]
        h = box[3] - box[1]
        dc.SetClippingRegion(box[0], box[1], w, h)
        dc.SetLogicalFunction(wxXOR)

        wbrush = wxBrush(wxColour(255,255,255), wxTRANSPARENT)
        wpen = wxPen(wxColour(200, 200, 200), 1, wxSOLID)
        dc.SetBrush(wbrush)
        dc.SetPen(wpen)
        dc.DrawRectangle(box[0], box[1], w,h)
        self._selected = false

        # reset selection to viewport size
        self.ResetSelection()

    def GetSlideThread(self):
        return self._slidethread

    def MultiImageFilter(self, event):
        """ Perform filter event for multiple images """

        id = event.GetId()
        self._frame.SaveUndoInfo()

        img1, img2, filter = None, None, None

        if id in range(800, 806):
            file1=self._frame.SelectFileDialog(self._frame._dir, 0)
            if file1 is not None:
                file2=self._frame.SelectFileDialog(self._frame._dir, 0)
                if file2 is None:
                    return
            else:
                return
            
            self._frame.SetImage(file1)
            img1=pyWiewGlobals.wxImgToPILImg(self._frame.GetImage())
            self._frame.SetImage(file2)
            img2=pyWiewGlobals.wxImgToPILImg(self._frame.GetImage())
            filter=pyWiewPILFilter.pyWiewMultiFilter(img1,img2)

        if id == 800:
            "Blend operation"
            
            w1,h1=img1.size
            w2,h2=img2.size
            
            if w1 != w2 and h1 != h2:
                self._frame.MessageDialog("Error: Images have different sizes", "Error")
                return
                
            dlg=pyWiewMiscWidgets.pyWiewSlider(self._frame, wxNewId(), 'Select Blend Coefficient')
            res=dlg.ShowModal()

            if res == wxID_OK:
                val=dlg.GetValue()
                dlg.Destroy()

                try:
                    imgout=filter.Blend(val)
                except ValueError, e:
                    msg='Error: ' + str(e)
                    self._frame.MessageDialog(msg, "Error!")
                    self._frame.SetStatusText(msg)
                    return

                self._frame.SetDisplayImage(imgout)
                str="(Blended Image)"
                self._frame.SetFileName(str)
                self._frame.SetCaption(str)
                self._frame.DrawImage()

        elif id == 801:
            "Subtract two images"

            imgout=filter.Subtract()
            self._frame.SetDisplayImage(imgout)
            str="(Difference Image)"
            self._frame.SetFileName(str)
            self._frame.SetCaption(str)
            self._frame.DrawImage()

        elif id == 802:
            "Add two images"

            imgout=filter.Add()
            self._frame.SetDisplayImage(imgout)
            str="(Sum Image)"
            self._frame.SetFileName(str)
            self._frame.SetCaption(str)
            self._frame.DrawImage()

        elif id == 803:
            "Superimpose two images"

            imgout=filter.Superimpose()
            self._frame.SetDisplayImage(imgout)
            str="(Superimposed Image)"
            self._frame.SetFileName(str)
            self._frame.SetCaption(str)
            self._frame.DrawImage()

        elif id == 804:
            "Lighter image"

            imgout=filter.Lighter()
            self._frame.SetDisplayImage(imgout)
            str="(Lighter Image)"
            self._frame.SetFileName(str)
            self._frame.SetCaption(str)         
            self._frame.DrawImage()

        elif id == 805:
            "Darker image"

            imgout=filter.Darker()
            self._frame.SetDisplayImage(imgout)
            str="(Darker Image)"
            self._frame.SetFileName(str)
            self._frame.SetCaption(str)         
            self._frame.DrawImage()




