""" pyWiewMiscWidgets.py --- class used by pyWiew to perform miscallaneous operations#
Copyright 2002-2003 Anand B Pillai <abpillai@lycos.com> """


from wxPython.wx import *

#File confirmation dialog#
class pyWiewConfirmDialog(wxDialog):
	""" Confirmation dialog """

	def __init___(self, parent, id, title):
		wxDialog.__init__(self, parent, id, title, wxDefaultPosition,
						  wxSize(300,100), wxDEFAULT_DIALOG_STYLE)
		self.SetTitle(title)
		
	def SetMsg(self,msg):
		q=msg + ' ? '
		self.panel=wxPanel(self,wxNewId())
		self.txt=wxStaticText(self.panel, wxNewId(), q, wxPoint(20,10))
		self.panel.Fit()
		w, h = self.panel.GetSizeTuple()
		wxButton(self.panel, wxID_OK,'OK', wxPoint(20 , 40))
		wxButton(self.panel, wxID_CANCEL,'Cancel', wxPoint(w-40, 40))
		self.panel.Fit()
		self.Fit()
		

#---------------------- The following widgets are not used right now ----------------------------------------#
#Show thumbnails as buttons#
class pyWiewThumbnailButton(wxBitmapButton):
	
	def __init__(self,parent,id,bmap, img, posn, size):
		wxBitmapButton.__init__(self,parent,id,bmap, img, posn, size)
		self._img=img
		EVT_BUTTON(self, self.GetImage)
		
	def GetImage(self,event):
		return self._img
		 
	
# Preview pane class
class pyWiewPreviewPane(wxFrame):
	"Frame based widget to preview images"
	
	def __init__(self, parent, name="Preview", pos=wxDefaultPosition, size=wxDefaultSize,style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE):
		wxFrame.__init__(self, parent, wxNewId(), name, pos, size, style)
		self.AddPreviewPane()

	def AddPreviewPane(self):
		self.ppane = wxPanel(self, wxNewId())
		self.pbmap = wxStaticBitmap(self.ppane, wxNewId(), wxEmptyBitmap(30,30), wxPoint(0,0), wxSize(128,128))
		
	# pass a wxImage to this class
	def PreviewImage(self, img):
		pimg = wxImgToPILImg(img)
		thumb = pimg.thumbnail((128,128))
		newimg = PilImgTowxImg(thumb)
		self.pbmap.SetBitmap(newimg.ConvertToBitmap())
		imgw = newimg.GetWidth()
		h = (128 * newimg.GetHeight()/imgw)
		self.pbmap.SetSize(wxSize(128, 128))
		self.ppane.Fit()
		self.Fit()
		self.Show(true)
		
# Not using right now
class pyWiewImgFileDialog(wxFileDialog):
	"Dialog based widget to browse for files"
	
	def __init__(self, parent, wildcard="*.*", msg="Choose a File", style = wxOPEN|wxMULTIPLE|wxOVERWRITE_PROMPT, pos=wxDefaultPosition):
		wxFileDialog.__init__(self, parent, msg, "", "", wildcard, style, pos)


class pyWiewUrlOpener(wxDialog):
	""" Interactive url opener widget """

	def __init__(self, parent, id, title):
		wxDialog.__init__(self, parent, id, title, wxDefaultPosition, wxSize(380, 190),
						  wxDEFAULT_DIALOG_STYLE)

		self._panel=wxPanel(self, wxNewId(), wxDefaultPosition, wxDefaultSize,
							wxTAB_TRAVERSAL | wxSIMPLE_BORDER)
		#add a text control
		wxStaticText(self._panel, wxNewId(), 'Enter Image URL:', wxPoint(15,5))
		self._urlentry=wxTextCtrl(self._panel, wxNewId(), "", wxPoint(15, 25), wxSize(330, 25))
		id=wxNewId()
		wxButton(self._panel, wxID_OK, ' OK ', wxPoint(20, 60), wxSize(70, 25))
		wxButton(self._panel, wxID_CANCEL, ' Cancel ', wxPoint(140, 60), wxSize(70, 25))
		self._morebtn=wxButton(self._panel, id, ' More Options... ', wxPoint(260, 60), wxSize(100, 25))
		EVT_BUTTON(self, id, self.ShowPasswdOptions)
		self._expanded=0
		self._auth=0
		self._user=""
		self._passwd=""
		self._savevalues=0
		self._panel.Fit()
		self.Fit()
		self.Show(true)

	def ShowPasswdOptions(self, event):
		""" Expand current dialog asking for username and password """

		if not self._expanded:
			self._line=wxStaticLine(self._panel, -5, wxPoint(5, 110), wxSize(370,2))
			self._text=wxStaticText(self._panel, wxNewId(), 'URL Needs Authorization:', wxPoint(15, 120))
			id=wxNewId()
			self._text2=wxStaticText(self._panel, wxNewId(), 'Username: ', wxPoint(90, 155))
			self._authuser=wxTextCtrl(self._panel, wxNewId(), "", wxPoint(150, 150), wxSize(120, 25))
			self._text3=wxStaticText(self._panel, wxNewId(), 'Password: ', wxPoint(90, 185))
			self._authpasswd=wxTextCtrl(self._panel, wxNewId(), "", wxPoint(150, 180), wxSize(120, 25), wxTE_PASSWORD)
			self._expanded=1
			self._auth=1
			self._morebtn.SetLabel(' Less Options... ')
		else:
			self._panel.RemoveChild(self._line)
			self._panel.RemoveChild(self._text)
			self._panel.RemoveChild(self._text2)
			self._panel.RemoveChild(self._text3)
			#self._panel.RemoveChild(self._savebox)
			self._panel.RemoveChild(self._authuser)
			self._panel.RemoveChild(self._authpasswd)
			self._expanded=0
			self._auth=0
			self._morebtn.SetLabel(' More Options... ')
			
		self._panel.Fit()
		self.Fit()
		pass

	def GetUserName(self):
		return str(self._authuser.GetValue())

	def GetPassword(self):
		return str(self._authpasswd.GetValue())

	def GetUrl(self):
		return str(self._urlentry.GetValue())

	def UrlNeedsAuth(self):
		return self._auth

	def SaveUserInfo(self, event):
		self._user=self.GetUserName()
		self._passwd=self.GetPassword()
	
#Interactive slider widget
class pyWiewSlider(wxDialog):
	""" Slider widget for selecting values """
	
	def __init__(self, parent, id, title, range=(0,100), initval=50, unitrange=true):
		
		wxDialog.__init__(self, parent, id, title, wxDefaultPosition,
						  wxSize(300, 80), wxDEFAULT_DIALOG_STYLE)
		self._panel=wxPanel(self, wxNewId(), wxDefaultPosition, wxDefaultSize,
							  wxTAB_TRAVERSAL | wxSIMPLE_BORDER)
		self._unitrange=unitrange

		wxStaticText(self._panel,wxNewId(),'Move Slider to set value: ', wxPoint(5, 5))
		id=wxNewId()
		self._slider=wxSlider(self._panel, id, initval, range[0], range[1],
							  wxPoint(15,25), wxSize(240, 30))
		EVT_COMMAND_SCROLL(self, id, self.ShowVal)
		if unitrange:
			initval=float(initval)/100.0
			  
		id=wxNewId()
		self._txtctrl=wxTextCtrl(self._panel, wxNewId(), str(initval),
								 wxPoint(270, 25), wxSize(40, 25))
		wxButton(self._panel,  wxID_OK,'OK', wxPoint(40, 60), wxSize(75,25))
		wxButton(self._panel, wxID_CANCEL,'Cancel', wxPoint(210, 60), wxSize(75, 25))
		self._panel.Fit()
		self.Fit()
		self.Show(true)
		
	def ShowVal(self, event):
		""" Shows the value of the slider in the text control """
		
		val=float(self._slider.GetValue())
		
		if self._unitrange:
			val = float(val)/100.0

		self._txtctrl.SetValue(str(val))
				   
	def GetValue(self):
		""" Return the value of the slider """
		
		return float(self._txtctrl.GetValue())



