""" pyWiewOptions --- Class to set/get pyWiew options for the end user#
    Copyright (C) 2002-2003 Anand B Pillai <abpillai@lycos.com>#

    Modified Sat Feb 22 18:27:53 2003 Added code for setting internet                                                                     related options.
	Modified Wed Apr 09 16:00:01 2003 Set frameontop default value to false

"""
     
import pyWiewWidgets
import pyWiewGlobals
from pyWiewGlobals import *
from wxPython.wx import *

class pyWiewImageOptions:

    def __init__(self):

        #thumbnail width
        self._version=""
        self.InitAppearanceVariables()
        self.InitFitVariables()
        self.InitSlideshowVariables()
        self.InitImgVariables()
        self.InitOtherVariables()
        self.InitNetworkVariables()

    def InitAppearanceVariables(self):

        self._windowontop = false
        self._showstatusbar = true
        self._showtoolbar  = true
        self._showstartupbitmap = true
        self._singleinstance = true
        self._noresize = false
        self._panelbgcolor = wxColour(0,0,0)
        self._displaytime = true

    def InitSlideshowVariables(self):
        """ Initialize slideshow variables """

        #slide show gap
        self._sgap=2000
        #default is do loop
        self._slideloop=true
        #show filename in caption
        self._slideshowfile = true
        # direction of show, default is forward
        #(alphebetical order of filenames)
        #0 - forward
        #1 - backward
        self._slidedirn=0
        #slideshow picks random files(default is false)
        self._slideshuffle =  false


    def InitFitVariables(self):
        """ Initialize image fit variables """
        #normal fit value
        #default is fit window to image
        self._fit = 0
        #other fit options
        #fit small image to window
        self._smallimgfit = true
        #fit large image to screen
        self._largeimgfit = true

    def InitImgVariables(self):
        """ Initialize image variables like thumbnail size etc """
        self._thumbw = 80
        #Assume that PIL is not avlbl
        self._pilstatus = false
        #show filename in window title bar
        self._showfile = true

    def InitOtherVariables(self):
        """ Initialize variables like font, font color etc """

        self._font =  wxFont(8, wxDEFAULT, wxNORMAL,wxNORMAL)
        self._fontc = wxColour(0,0,0)

    def InitNetworkVariables(self):

        self._netstatus=0
        self._authproxy=0
        self._proxies={}
        self._proxyauth=()

    def GetProxyOptions(self):
        return self._proxies

    def GetProxyAuth(self):
        return self._proxyauth

    def GetProxyUser(self):
        if self._authproxy:
            return self._proxyauth[0]
        else:
            return ""

    def GetProxyPwd(self):
        if self._authproxy:
            return self._proxyauth[1]
        else:
            return ""

    def ProxyNeedsAuth(self):
        return self._authproxy

    def GetHttpProxy(self):
        try:
            return self._proxies["http"]
        except KeyError:
            return ""

    def GetHttpsProxy(self):
        try:
            return self._proxies["https"]
        except KeyError:
            return ""

    def GetFtpProxy(self):
        try:
            return self._proxies["ftp"]
        except KeyError:
            return ""

    def SetHttpProxy(self, proxy):
        self._proxies["http"]=proxy

    def SetHttpsProxy(self, proxy):
        self._proxies["https"]=proxy

    def SetFtpProxy(self, proxy):
        self._proxies["ftp"]=proxy

    def SetProxyNeedsAuth(self, val):
        
        if val>=0:
            self._authproxy=val
        else:
            self._authproxy=0

    def SetProxyUser(self, user):

        if len(self._proxyauth)==2:
            passwd=self._proxyauth[1]
            self._proxyauth=(user, passwd)
        else:
            self._proxyauth=(user,"")

    def SetProxyPasswd(self, paswd):

        if len(self._proxyauth)==2:
            username=self._proxyauth[0]
            self._proxyauth=(username, paswd)
        else:
            self._proxyauth=("",paswd)
        
    def GetInternetStatus(self):
        #0 for no conn,
        #1 for direct conn,
        #2 for conn thru proxy
        return self._netstatus

    def SetInternetStatus(self, val):
        #0 for no conn
        #1 for direct conn,
        #2 for proxy conn.
        if val == 0 or val == 1 or val == 2:
            self._netstatus=val
        else:
            #bogus value!
            self._netstatus=0
            
    def FrameOnTop(self):
        return self._windowontop

    def SetFrameOnTop(self,val):
        self._windowontop = val

    def SetVersion(self, version):
        self._version=version

    def SingleInstance(self):
        return self._singleinstance

    def SetSingleInstance(self, val):
        self._singleinstance = val

    def ShowstartupBitmap(self):
        return self._showstartupbitmap

    def SetstartupBitmapStatus(self, val):
        self._showstartupbitmap = val

    def ShowtoolBar(self):
        return self._showtoolbar

    def SetShowtoolBar(self,val):
        self._showtoolbar = val

    def ShowStatusBar(self):
        return self._showstatusbar

    def SetShowstatusBar(self, val):
        self._showstatusbar = val

    def GetPanelBgColor(self):
        return self._panelbgcolor

    def SetPanelBgColor(self, val):
        self._panelbgcolor = val

    def ShowTime(self):
        return self._displaytime

    def SetShowTime(self, val):
        self._displaytime = val

    def GetPilStatus(self):
        return self._pilstatus

    def ResizeAllowed(self):
        return not self._noresize

    def SetresizeAllowed(self, val):
        self._noresize = val

    def SetPilStatus(self,val):
        #true means pil is working
        self._pilstatus = val


    def GetThumbnailSize(self, sz):
        """ Return the thumbnail size as a wxSize instance """

        w,h = sz.GetWidth(), sz.GetHeight()
        #new width is self._thumbw
        #calc new height based on aspect ratio
        h = h*self._thumbw/w
        w = self._thumbw
        return wxSize(w,h)

    def GetThumbnailSizeTuple(self, sz):
        """ Return the thumbnail size as a size tuple """

        w,h = sz.GetWidth(), sz.GetHeight()
        #new width is self._thumbw
        #calc new height based on aspect ratio
        h = h*self._thumbw/w
        w = self._thumbw
        return (w,h)

    def SetThumbnailWidth(self,val):
        """ Set the thumbnail width """
        self._thumbw = val

    def GetThumbnailWidth(self):
        return self._thumbw

    def GetPreviewStatus(self):
        return self._preview

    def SetFit(self,val):
        self._fit=val

    def GetFit(self):
        return self._fit

    def GetSmallImgFit(self):
        return self._smallimgfit

    def SetSmallImgFit(self, val):
        self._smallimgfit = val

    def GetLargeImgFit(self):
        return self._largeimgfit

    def SetLargeImgFit(self, val):
        self._largeimgfit=val

    def ShowDuration(self):
        "Returns the gap between successive images in a slideshow"
        return self._sgap

    def SetShowDuration(self, val):
        self._sgap = val

    def ShowFile(self):
        return self._slideshowfile

    def SetShowFile(self,val):
        self._slideshowfile = val

    def ShowDirection(self):
        return self._slidedirn

    def SetShowDirection(self, dirn):
        self._slidedirn = dirn

    def ShowLoop(self):
        return self._slideloop

    def SetShowLoop(self, val):
        self._slideloop = val

    def SetShowRandom(self,val):
        self._slideshuffle = val

    def ShowRandom(self):
        return self._slideshuffle

    def ShowOptions(self):
        """ Returns a tuple of slideshow options in the order of
        slide-show dirn,slide-show gap, slide-show filename,slide-show loop
        and slide-show random """

        return (self._slidedirn, self._sgap, self._slideshowfile, self._slideloop, self._slideshuffle)

    def Font(self):
        return self._font

    def FontColor(self):
        return self._fontc

    def SetFont(self, font):
        self._font = font

    def SetFontColor(self,fontc):
        self._fontc = fontc

    def ShowFilenameCaption(self):
        """ Whether to show filename in the window title """
        return self._showfile

    def SetShowFilenameCaption(self, val):
        self._showfile = val

    def SetImageFitOptions(self, imgfitdict):
        """ Set image fit options from the dictionary """

        if imgfitdict is None:
            return

        #fit small image to window
        self._smallimgfit = imgfitdict['FitSmallImage']
        #fit large image to screen
        self._largeimgfit = imgfitdict['FitLargeImage']
        #fit image value (index)
        # 0 - no fit
        # 1 - fit image to window
        # 2 - fit image to screen
        # 3 - fit window to screen
        self._fit = imgfitdict['FitImageIndex']          
        pass

    def SetAppearanceOptions(self, apprdict):
        """ Set appearance options from the dictionary """

        if apprdict is None:
            return

        #window on top
        windowontopst = apprdict['WindowOnTop']
        #status bar status
        statusbarst = apprdict['StatusBar']
        #toolbar status
        toolbarst = apprdict['ToolBar']
        #show startup bitmap
        sbitmapst  = apprdict['StartupBitmap']
        #single instance status
        singleinstst = apprdict['SingleInstance']
        #no resize status
        noresizest = apprdict['NoResize']
        #display time status
        timest = apprdict['DisplayTime']

        #set the option directly
        #Dont bother with calling functions since
        #these members belong to the class anyway.
        self._windowontop = windowontopst
        self._showstatusbar = statusbarst
        self._showtoolbar = toolbarst
        self._showstartupbitmap = sbitmapst
        self._singleinstance = singleinstst
        self._noresize = noresizest
        self._displaytime = timest

    def SetSlideshowOptions(self, slideshowdict):
        """ Set slideshow options from the dictionary """

        if slideshowdict is None:
            return

        #Slideshow values
        self._slideloop = slideshowdict['WrapSlideStatus']
        self._slideshuffle = slideshowdict['ShuffleSlideStatus']
        self._slidedirn = slideshowdict['DirectionSlideStatus']
        self._sgap = slideshowdict['DelaySlideValue']

    def SetInternetOptions(self, internetdict):
        """ Set the options for internetworking """

        if internetdict is None:
            return

        #Internet options
        self._netstatus=internetdict['InternetStatus']
        httpproxy=internetdict['HTTPProxy']
        httpport=internetdict['HTTPProxyPort']
        sslproxy=internetdict['SSLProxy']
        sslport=internetdict['SSLProxyPort']
        ftpproxy=internetdict['FTPProxy']
        ftpport=internetdict['FTPProxyPort']
        self._proxies["http"]=httpproxy + ":" + httpport
        self._proxies["https"]=sslproxy + ":" + sslport
        self._proxies["ftp"]=ftpproxy + ":" + ftpport
        self._authproxy=internetdict['ProxyNeedsAuth']
        proxyusername=internetdict['ProxyUserName']
        proxypwd=internetdict['ProxyPassword']

        self._proxyauth=(proxyusername, proxypwd)

# Dialog widget for user to set options for pyWiew interactively #
class pyWiewOptionsDlg(wxDialog):
    "A dialog class to set options for the user."

    def __init__(self,  parent, id):
        wxDialog.__init__(self, parent, wxNewId(), "Properties ",
                          wxDefaultPosition, wxSize(500,500), wxDEFAULT_DIALOG_STYLE)
        self._options = parent.GetOptionsObject()
        #self.panel = wxPanel(self, -1)
        #wxStaticBox(self.panel, wxNewId(), '',  wxPoint(10, 10), wxSize(370, 490))
        self.AddNoteBook()
        self.FillPages()
        self.AddButtons()
        self.RegisterEventHandlers()
        self.book.Show(true)
        #self.panel.Fit()
        self.Show(true)

    def RegisterEventHandlers(self):
        pass

    def AddNoteBook(self):
        self.book = wxNotebook(self, wxNewId(), wxPoint(10,10), wxSize(475, 430) )
        col = wxColour(200,200,200)
        pg1 = self.MakePanel(col, 'Viewing Options')
        self.book.AddPage(pg1, " Viewer ")
        pg2 = self.MakePanel(col,' Browsing Options ')
        self.book.AddPage(pg2, ' Browser ')
        pg3 = self.MakePanel(col, ' Image Options ')
        self.book.AddPage(pg3, ' Image Options ')
        pg4 = self.MakePanel(col, ' Extensions ')
        self.book.AddPage(pg4, ' Extensions ')
        pg5 = self.MakePanel(col, ' Toolbar ')
        self.book.AddPage(pg5, ' Toolbar ')
        pg6 = self.MakePanel(col, ' Internet ')
        self.book.AddPage(pg6, ' Internet Options ')

    def MakePanel(self, color, title):

        p = wxPanel(self.book, wxNewId(), wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL)
        p.SetBackgroundColour(color)
        #wxStaticBox(p, wxNewId(), title,  wxPoint(8, 10), wxSize(450, 385))
        p.Show(true)
        return p

    def AddButtons(self):
        ok = wxButton(self,  wxID_OK, " OK ", wxPoint(300, 452), wxSize(75, 22))
        cancel = wxButton(self, wxID_CANCEL, " Cancel ", wxPoint(400, 452), wxSize(75, 22))

    def FillPages(self):
        self.SetExtensionsPage()
        self.SetViewPage()
        self.SetInternetPage()
        #sbox.CenterOnParent()
        pass

    def OnOk(self,event):
        self.Destroy()

    def OnCancel(self, event):
        self.Destroy()

    def SetPanelColor(self, event):
        cdlg=wxColourDialog(self)
        result = cdlg.ShowModal()
        if result == wxID_CANCEL:
            return

        #get data
        coldat = cdlg.GetColourData()
        col=coldat.GetColour()
        #panel color is set directly
        self._options.SetPanelBgColor(col)
        img=ImgOfSize((80,20), col)
        self._panelbmap.SetBitmap(img.ConvertToBitmap())
        cdlg.Destroy()

    def SetViewPage(self):

        page=self.book.GetPage(0)
        #Frame appearance settings     
        wxStaticBox(page, wxNewId(), 'Appearance', wxPoint(10,10), wxSize(445, 110))
        self._topbox = wxCheckBox(page, wxNewId(), '&Always on Top', wxPoint(20, 40))
        self._topbox.SetValue(self._options.FrameOnTop())
        self._statbox = wxCheckBox(page, wxNewId(), 'Show Status&bar', wxPoint(20, 60))
        self._statbox.SetValue(self._options.ShowStatusBar())
        self._toolbox = wxCheckBox(page, wxNewId(), 'Show &Toolbar', wxPoint(180, 40))
        self._toolbox.SetValue(self._options.ShowtoolBar())
        self._bitmapbox = wxCheckBox(page, wxNewId(), 'Load &Startup Bitmap', wxPoint(180, 60))
        self._bitmapbox.SetValue(self._options.ShowstartupBitmap())
        self._instbox = wxCheckBox(page, wxNewId(), '&Only one Instance', wxPoint(330, 40))
        self._instbox.SetValue(self._options.SingleInstance())
        self._resizebox = wxCheckBox(page, wxNewId(), '&No Resize', wxPoint(330, 60))
        self._resizebox.SetValue(not self._options.ResizeAllowed())

        #Image fit options
        wxStaticBox(page, wxNewId(), 'Image Fit', wxPoint(10, 130), wxSize(445,110))
        #combo of fit values
        wxStaticText(page, wxNewId(), '&Fit Style: ', wxPoint(20, 160))
        fitchoices=['Fit window to image(No Fit)', 'Fit image to window', 'Fit image to screen', 'Fit window to screen', 'Random Fit']
        self._fitcombo = wxComboBox(page, wxNewId(), "", wxPoint(70, 160), wxSize(160, 20), fitchoices, wxCB_DROPDOWN|wxCB_READONLY)
        self._fitcombo.SetSelection(self._options.GetFit())
        self._fitbox1 = wxCheckBox(page, wxNewId(), 'Fit &small images to Window', wxPoint(280, 160))
        self._fitbox1.SetValue(self._options.GetSmallImgFit())
        self._fitbox2 = wxCheckBox(page, wxNewId(), 'Fit &large images to Screen', wxPoint(280, 200))
        self._fitbox2.SetValue(self._options.GetLargeImgFit())

        #Misc options
        wxStaticBox(page, wxNewId(), 'Miscellaneous ', wxPoint(10, 260), wxSize(445,110))
        #mouse cursor visibility

        #hide mouse cursor ?
        self._cursvis1 = wxCheckBox(page, wxNewId(), '&Hide mouse cursor during slideshow', wxPoint(20, 290))
        col = self._options.GetPanelBgColor()
        wxStaticBox(page, wxNewId(), 'Panel Color ', wxPoint(260, 275), wxSize(180, 80))
        bmap=ImgOfSize(wxSize(80,20), col).ConvertToBitmap()

        id=wxNewId()
        title=' Set &Color '
        wxStaticText(page, wxNewId(), 'Click: ', wxPoint(280, 295))
        self._panelbmapbtn = wxButton(page, id, title, wxPoint(320, 290))
        EVT_BUTTON(self, id, self.SetPanelColor)
        wxStaticText(page, wxNewId(), 'Color: ', wxPoint(280, 325))
        self._panelbmap = wxStaticBitmap(page, wxNewId(), bmap, wxPoint(320, 320), wxSize(80,20))
        self._timebox = wxCheckBox(page, wxNewId(), 'Display &time', wxPoint(20, 320))
        self._timebox.SetValue(self._options.ShowTime())

    def ShowSlideDelay(self, event):
        """ Show the delay set by the slider in the text control """

        val = self._slideshowbar.GetValue()
        self._slidedelaytxt.SetValue(str(val))

    def SetExtensionsPage(self):

        page=self.book.GetPage(3)
        wxStaticBox(page, wxNewId(), 'Associated File Extensions ', wxPoint(10,10), wxSize(445, 370))
        #build an extensions multilist
        self._extbox = pyWiewWidgets.pyWiewFileTypesBox(page,
                                                        pyWiewGlobals.AllFilesTuples(),
                                                        wxPoint(20, 30),
                                                        wxSize(330, 150))

        #select all button
        id=wxNewId()
        self._selallbtn=wxButton(page, id, ' Select all ', wxPoint(360, 30), wxSize(80, 25))
        EVT_BUTTON(self, id, self._extbox.CheckAll)
        #images only button
        id=wxNewId()
        self._imgonlybtn=wxButton(page, id, ' Images only ', wxPoint(360, 60), wxSize(80, 25))
        EVT_BUTTON(self, id, self._extbox.CheckImages)
        id=wxNewId()
        self._clrbtn=wxButton(page, id, ' Clear all ', wxPoint(360, 120), wxSize(80, 25))
        EVT_BUTTON(self, id, self._extbox.ClearAll)
        #Clear all button

        #dont load non-assoc types
        self._loadasscbox = wxCheckBox(page, wxNewId(), 'Load only associated types when moving through directory', wxPoint(20, 200))
        
        #dont load animation types
        self._dontloadanimbox = wxCheckBox(page,wxNewId(), 'Dont load animations while moving through directory', wxPoint(20, 230))
        
        
        
        
    def SetNetwork(self,event):
        if self._radio1.GetValue():
            self.EnableProxySettings(false)
        elif self._radio2.GetValue():
            self.EnableProxySettings(false)         
        elif self._radio3.GetValue():
            self.EnableProxySettings()

    def EnableProxyAuth(self, event):

        if self._radio3.GetValue() and self._proxyauthbox.GetValue():
            self._proxyuserbox.Enable(true)
            self._proxypwdbox.Enable(true)
        else:
            self._proxyuserbox.Enable(false)
            self._proxypwdbox.Enable(false)
        pass

    def EnableProxySettings(self, val=true):

        if val:
            self._httptext.Enable(true)
            self._httpport.Enable(true)
            self._httpstext.Enable(true)
            self._httpsport.Enable(true)
            self._ftptext.Enable(true)
            self._ftpport.Enable(true)
            self._proxyauthbox.Enable(true)
            self.EnableProxyAuth(None)
        else:
            self._httptext.Enable(false)
            self._httpport.Enable(false)
            self._httpstext.Enable(false)
            self._httpsport.Enable(false)
            self._ftptext.Enable(false)
            self._ftpport.Enable(false)
            self._proxyauthbox.Enable(false)
            self._proxyuserbox.Enable(false)
            self._proxypwdbox.Enable(false)

        pass

    def SetInternetPage(self):

        page=self.book.GetPage(5)
        netstat=self._options.GetInternetStatus()
        wxStaticBox(page, wxNewId(), 'Configure Network Connection', wxPoint(10,10), wxSize(445, 370))
        msg='No Internet Connection'
        id=wxNewId()
        self._radio1=wxRadioButton(page, id, msg, wxPoint(30,40))
        EVT_RADIOBUTTON(self, id, self.SetNetwork)
        msg='Direct connection to the Internet'
        id=wxNewId()
        self._radio2=wxRadioButton(page, id, msg, wxPoint(30, 70))
        EVT_RADIOBUTTON(self, id, self.SetNetwork)
        msg='Manual Proxy configuration'
        id=wxNewId()
        self._radio3=wxRadioButton(page, id, msg, wxPoint(30, 100))
        EVT_RADIOBUTTON(self, id, self.SetNetwork)

        if netstat==0:
            self._radio1.SetValue(true)
        elif netstat==1:
            self._radio2.SetValue(true)
        elif netstat==2:
            self._radio3.SetValue(true)

        httpproxy=self._options.GetHttpProxy()
        if httpproxy != "":
            proxy, port=string.split(httpproxy, ':')
        else:
            proxy, port = "",""
        wxStaticText(page, wxNewId(), '&HTTP Proxy: ', wxPoint(70, 125))
        self._httptext=wxTextCtrl(page, wxNewId(), proxy,  wxPoint(140, 122), wxSize(180, 23))
        wxStaticText(page, wxNewId(), '&Port: ', wxPoint(340, 125))
        self._httpport=wxTextCtrl(page, wxNewId(), port, wxPoint(370, 122), wxSize(50,23))
        httpsproxy=self._options.GetHttpsProxy()
        if httpsproxy != "":
            proxy, port=string.split(httpsproxy, ':')
        else:
            proxy, port = "",""

        wxStaticText(page, wxNewId(), '&SSL Proxy: ', wxPoint(70, 155))
        self._httpstext=wxTextCtrl(page, wxNewId(), proxy,  wxPoint(140, 152), wxSize(180, 23))
        wxStaticText(page, wxNewId(), '&Port: ', wxPoint(340, 155))
        self._httpsport=wxTextCtrl(page, wxNewId(), port, wxPoint(370, 152), wxSize(50,23))
        ftpproxy=self._options.GetFtpProxy()
        if ftpproxy != "":
            proxy, port=string.split(ftpproxy, ':')
        else:
            proxy, port = "",""
        wxStaticText(page, wxNewId(), '&FTP Proxy: ', wxPoint(70, 185))
        self._ftptext=wxTextCtrl(page, wxNewId(), proxy,  wxPoint(140, 182), wxSize(180, 23))
        wxStaticText(page, wxNewId(), '&Port: ', wxPoint(340, 185))
        self._ftpport=wxTextCtrl(page, wxNewId(), port, wxPoint(370, 182), wxSize(50,23))
        wxStaticBox(page, wxNewId(), 'Proxy Settings', wxPoint(140, 220), wxSize(280,140))
        id=wxNewId()
        self._proxyauthbox=wxCheckBox(page, id, 'Proxy Needs Authorization', wxPoint(170, 250))
        if self._options.ProxyNeedsAuth():
            self._proxyauthbox.SetValue(true)
        else:
            self._proxyauthbox.SetValue(false)

        EVT_CHECKBOX(self, id, self.EnableProxyAuth)
        proxyuser=self._options.GetProxyUser()
        proxypwd=self._options.GetProxyPwd()
        wxStaticText(page, wxNewId(), 'Username: ', wxPoint(180, 280))
        self._proxyuserbox=wxTextCtrl(page, wxNewId(), proxyuser, wxPoint(240, 275), wxSize(100,25))
        wxStaticText(page, wxNewId(), 'Password: ', wxPoint(180, 315))      
        self._proxypwdbox=wxTextCtrl(page, wxNewId(), proxypwd, wxPoint(240, 310), wxSize(100,25),wxTE_PASSWORD)
        self.SetNetwork(None)

    def GetViewOptions(self):
        #return all view options as a tuple of dictionaries

        #Appearance dictionary
        apprdict = {}
        apprdict['WindowOnTop'] = self._topbox.GetValue()
        apprdict['StatusBar']= self._statbox.GetValue()
        apprdict['ToolBar'] = self._toolbox.GetValue()
        apprdict['StartupBitmap'] = self._bitmapbox.GetValue()
        apprdict['SingleInstance'] = self._instbox.GetValue()
        apprdict['NoResize'] = self._resizebox.GetValue()
        apprdict['DisplayTime'] = self._timebox.GetValue()
        apprdict['HideCursor'] = self._cursvis1.GetValue()

        #Image fit dictionary
        imagefitdict = {}
        imagefitdict['FitSmallImage'] = self._fitbox1.GetValue()
        imagefitdict['FitLargeImage'] = self._fitbox2.GetValue()
        imagefitdict['FitImageIndex'] = self._fitcombo.GetSelection()
        #slidedict['WrapSlideStatus'] = self._slidewrap.GetValue()
        #slidedict['ShuffleSlideStatus'] = self._sliderandom.GetValue()
        #slidedict['DirectionSlideStatus'] = self._slidecombo.GetSelection()
        #slidedict['DelaySlideValue'] = self._slideshowbar.GetValue()

        return (apprdict, imagefitdict)

    def GetInternetOptions(self):

        netdict={}
        netstatus=0
        if self._radio1.GetValue():
            netstatus=0
        elif self._radio2.GetValue():
            netstatus=1
        elif self._radio3.GetValue():
            netstatus=2

        netdict['InternetStatus']= netstatus
        netdict['HTTPProxy']=self._httptext.GetValue()
        netdict['HTTPProxyPort']=self._httpport.GetValue()
        netdict['SSLProxy']=self._httpstext.GetValue()
        netdict['SSLProxyPort']=self._httpsport.GetValue()
        netdict['FTPProxy']=self._ftptext.GetValue()
        netdict['FTPProxyPort']=self._ftpport.GetValue()
        if self._proxyauthbox.GetValue():
            netdict['ProxyNeedsAuth']=1
        else:
            netdict['ProxyNeedsAuth']=0

        netdict['ProxyUserName']=self._proxyuserbox.GetValue()
        passwd=self._proxypwdbox.GetLineText(0)
        netdict['ProxyPassword']=passwd

        return netdict

    def GetImageOptions(self):
        pass

    def GetExtensionOptions(self):
        pass

    def GetMiscOptions(self):
        pass

    def GetToolbarOptions(self):
        pass

    def GetBrowserOptions(self):
        pass

    def SetSlideshowPage(self):

        #Slideshow options
        wxStaticBox(page,wxNewId(), 'Slideshow', wxPoint(10, 260), wxSize(445,110))
        self._slidewrap = wxCheckBox(page, wxNewId(), '&Wrap Around', wxPoint(20, 290))
        self._slidewrap.SetValue(self._options.ShowLoop())
        self._sliderandom = wxCheckBox(page, wxNewId(), '&Shuffle Images', wxPoint(180, 290))
        self._sliderandom.SetValue(self._options.ShowRandom())
        slideorder = ['Forward', 'Backward']
        wxStaticText(page, wxNewId(), '&Direction: ', wxPoint(290, 290))
        self._slidecombo = wxComboBox(page, wxNewId(), '', wxPoint(345, 285), wxSize(80, 20), slideorder, wxCB_DROPDOWN | wxCB_READONLY)
        self._slidecombo.SetSelection(self._options.ShowDirection())
        #delay
        wxStaticText(page, wxNewId(), 'De&lay: ', wxPoint(20, 320))
        #slide bar for delay
        id=wxNewId()
        duration = self._options.ShowDuration()
        self._slideshowbar = wxSlider(page, id, duration, 0, 10000, wxPoint(80, 320), wxSize(250, 40))
        EVT_COMMAND_SCROLL(self, id, self.ShowSlideDelay)
        #edit box showing slideshow delay value
        self._slidedelaytxt = wxTextCtrl(page, wxNewId(), str(duration), wxPoint(340, 320), wxSize(40, 20))
        wxStaticText(page, wxNewId(), ' millseconds', wxPoint(380, 320))







