""" pyWiewPILFilter.py --- class used by pyWiew to filter/enhance image using PIL(Python Imaging Library)
Copyright 2002-2003 Anand B Pillai <abpillai@lycos.com> """

from pyWiewGlobals import *
from wxPython.wx import *
import Image
import ImageFilter
import ImageChops
import ImageEnhance
import ImageOps

class pyWiewImageFilter:
	""" Image filtering class. Defines filtering operations 
	Currently this class works only with an instance of a PIL 
	image and returns a wxImage instance/PIL image instance """

	def __init__(self, imgdata):
		self._imgdata = imgdata
		self.enh_count = 0
		self.sm_count = 0

	def	 doNegEffect(self, event):
		"Negative of image"

		imgout = ImageChops.invert(self._imgdata)
		self._imgdata = imgout
		return self.GetImage()					  

	def	 doEmbossEffect(self, event):
		"Emboss image"

		imout = self._imgdata.filter(ImageFilter.EMBOSS)
		self._imgdata = imout
		return self.GetImage()

	def doEdgeEnhance(self, event):
		"Enhance edges"

		if self.enh_count is 0:
			imout = self._imgdata.filter(ImageFilter.EDGE_ENHANCE)
		else:
			imout = self._imgdata.filter(ImageFilter.EDGE_ENHANCE_MORE)

		self.enh_count += 1
		self._imgdata = imout
		return self.GetImage()			

	def doDetailEffect(self, event):
		"Detail of image"

		imout = self._imgdata.filter(ImageFilter.DETAIL)
		self._imgdata = imout
		return self.GetImage()			

	def doContourEffect(self, event):
		"Get contours"

		imout = self._imgdata.filter(ImageFilter.CONTOUR)
		self._imgdata = imout
		return self.GetImage()			

	def doBlurEffect(self, event):
		"Blur the image"

		imout = self._imgdata.filter(ImageFilter.BLUR)
		self._imgdata = imout
		return self.GetImage()			

	def doSharpenEffect(self, event):
		"Sharpen the image"

		imout = self._imgdata.filter(ImageFilter.SHARPEN)
		self._imgdata = imout
		return self.GetImage()

	def doSmoothEffect(self, event):
		"Smooth the image"

		if self.sm_count is 0:
			imout = self._imgdata.filter(ImageFilter.SMOOTH)
		else:
			imout = self._imgdata.filter(ImageFilter.SMOOTH_MORE)

		self.sm_count += 1
		self._imgdata = imout
		return self.GetImage()

	def doFindEdges(self, event):
		"Find edges of image"
		imout = self._imgdata.filter(ImageFilter.FIND_EDGES)
		self._imgdata = imout
		return self.GetImage()

	def doGraniteEtch(self, event):
		"Sketch of the image"

		imout = self._imgdata.filter(ImageFilter.GRANITE_ETCH)
		#imout = self._imgdata.filter(ImageFilter.RANDOM)
		self._imgdata = imout
		return self.GetImage()
		
	def do3DFeatDetect(self,event):

		imout = self._imgdata.filter(ImageFilter.FEATDETECT)
		self._imgdata = imout
		return self.GetImage()

	def RedChannel(self, event):

		imout = self._imgdata.convert("L")
		self._imgdata = imout
		return self.GetImage()

	def GreenChannel(self,event):

		imout = self._imgdata.filter(ImageFilter.GREENFILTER)
		self._imgdata = imout
		return self.GetImage()

	def BlueChannel(self, event):

		imout = self._imgdata.filter(ImageFilter.BLUEFILTER)
		self._imgdata = imout
		return self.GetImage()
		
	def FineRotate(self, event):
		"Rotate image by 45 degrees"

		self._imgdata = self._imgdata.rotate(45)
		return self.GetImage()

	def GrayScale(self, event):
		"Return grayscaled image"

		imout = ImageOps.grayscale(self._imgdata)
		self._imgdata = imout
		return self.GetImage()
		
	def ColorEnhance(self, event, factor):
		"Enhance colors"

		enh = ImageEnhance.Color(self._imgdata)
		imout = enh.enhance(factor)
		self._imgdata = imout
		return self.GetImage()

	def ContrastEnhance(self, event, factor):
		"Enhance contrast"

		enh = ImageEnhance.Contrast(self._imgdata)
		imout = enh.enhance(factor)
		self._imgdata = imout
		return self.GetImage()

	def Brighten(self, event, factor):
		"Brighten image"

		enh = ImageEnhance.Brightness(self._imgdata)
		imout = enh.enhance(factor)
		self._imgdata = imout
		return self.GetImage()

	def SharpnessEnhance(self, event, factor):
		"Enhance sharpness"

		enh = ImageEnhance.Sharpness(self._imgdata)
		imout = enh.enhance(factor)
		self._imgdata = imout
		return self.GetImage()

	def HorizontalWrap(self, event, factor):
		"Wrap horizontally"

		self._imgdata = ImageChops.offset(self._imgdata, factor, 0)
		return self.GetImage()

	def VerticalWrap(self, event, factor):
		"Wrap vertically"

		self._imgdata = ImageChops.offset(self._imgdata, 0, factor)
		return self.GetImage()

	def AutoContrast(self, cutoff=10):
		""" Autocontrast of an image """

		self._imgdata = ImageOps.autocontrast(self._imgdata, cutoff)
		return self.GetImage()

	def Solarize(self, threshold=128):
		""" Threshold negative of image """

		self._imgdata = ImageOps.solarize(self._imgdata, threshold)
		return self.GetImage()

	def GetImage(self):
		"Returns a wxImage instance of the filtered image"

		outimg = apply(wxEmptyImage, self._imgdata.size )
		outimg.SetData(self._imgdata.convert( "RGB").tostring() )
		return outimg

	def GetPILImage(self):
		"Returns PIL image"

		return self._imgdata

class pyWiewMultiFilter:
	""" Multiple image filtering class """
	
	def __init__(self, img1, img2):
		self._img1=img1.copy()
		self._img2=img2.copy()
		self._imgout=None
		
	def Blend(self, alpha):
		self._imgout=ImageChops.blend(self._img1, self._img2, alpha)
		return self.GetImage()

	def Subtract(self):
		self._imgout=ImageChops.subtract(self._img1, self._img2)
		return self.GetImage()
	
	def Add(self):
		self._imgout=ImageChops.add(self._img1, self._img2)
		return self.GetImage()

	def Superimpose(self):
		self._imgout=ImageChops.multiply(self._img1, self._img2)
		return self.GetImage()
	
	def Lighter(self):
		
		self._imgout=ImageChops.lighter(self._img1, self._img2)
		return self.GetImage()

	def Darker(self):
		
		self._imgout=ImageChops.darker(self._img1, self._img2)
		return self.GetImage()

	def GetImage(self):
		
		outimg = apply(wxEmptyImage, self._imgout.size )
		outimg.SetData(self._imgout.convert( "RGB").tostring() )
		return outimg
