#pyWiewWidgets.py --- helper classes used along with pyWiew for annotation/browsing etc#
#Copyright 2002-2003 Anand B Pillai <abpillai@Lycos.com>#

from pyWiewGlobals import *
from wxPython.wx import *

class pyWiewFileTypesBox(wxCheckListBox):

    def __init__(self, parent, tuples, posn=wxDefaultPosition, size=wxDefaultSize):

        #create the string of choices
        self._strings=[]
        self._types=[]

        for x in tuples:
            type=x[0]
            str = x[1]
            self._strings.append(str)
            self._types.append(type)

        #initialize
        wxCheckListBox.__init__(self, parent, wxNewId(), posn, size,
                                self._strings)
        
    def CheckAll(self, event):
        """ Check all the items """

        for x in range(0, len(self._strings)):
            self.Check(x, TRUE)

    def CheckImages(self, event):
        """ Check only image items """

        for x in range(0, len(self._strings)):
            str = string.lower(self._strings[x])
            if string.find(str, "anim") == -1:
                self.Check(x, TRUE)
            else:
                self.Check(x, FALSE)

    def CheckAnimations(self, event):
        """ Check only animation items """

        for x in range(0, len(self._strings)):
            str = string.lower(self._strings[x])
            if string.find(str, "anim") == -1:
                self.Check(x, FALSE)
            else:
                self.Check(x, TRUE)

    def ClearAll(self, event):

        for x in range(0, len(self._strings)):
            self.Check(x, FALSE)
                                
class pyWiewEditTextFrame(wxDialog):
    """Dialog widget to add text to the current image"""

    def __init__(self, parent, posn=wxDefaultPosition):
		
        wxDialog.__init__(self, parent, wxNewId(), "Add Text to image", posn, wxSize(350,200), wxDEFAULT_DIALOG_STYLE)
        self.panel = wxPanel(self, wxNewId())
        sbox = wxStaticBox(self.panel, wxNewId(), "", wxPoint(5, 2), wxSize(390, 180))
        wxStaticText(self.panel, wxNewId(), "Text:", wxPoint(13, 13))
        self.txtctrl = wxTextCtrl(sbox, wxNewId(), "", wxPoint(10, 26), wxSize(240, 115), wxTE_MULTILINE
                                  , wxDefaultValidator, "Enter Text")
        self._trans = wxCheckBox(self.panel, wxNewId(), "Text is transparent", wxPoint(265, 28))
        self._trans.SetValue(true)
        # button
        id = wxNewId()
        wxButton(self.panel, id, "Choose Font", wxPoint(270, 145))
        EVT_BUTTON(self, id, self.SetFont)
        wxButton(self.panel, wxID_OK,'OK', wxPoint(120, 190))
        wxButton(self.panel, wxID_CANCEL,'Cancel', wxPoint(240, 190))
        self.panel.Fit()
        self.Fit()
        self.Show(true)

    def GetAllData(self):
        """ Return all the data as a two sized tuple """

        txt=self.txtctrl.GetValue()
        transval=self._trans.GetValue()
        self.Show(false)
        return (txt, transval)

    def GetTextData(self):
        """ Return text entered by user """
        txt = self.txtctrl.GetValue()
        self.Show(false)
        return txt

    def GetTransparency(self):
        """ Return the transparency value """

        return self._trans.GetValue()

    def SetFont(self, event):
        "Sets the text's font"

        fdat = wxFontData()
        fdg = wxFontDialog(self, fdat)
        result = fdg.ShowModal()
        if result == wxID_CANCEL:
            fdg.Destroy()
            return
        fdat = fdg.GetFontData()
        options = self.GetParent().GetOptionsObject()
        options.SetFont(fdat.GetChosenFont())
        options.SetFontColor(fdat.GetColour())
        fdg.Show(false)
        fdg.Destroy()


class pyWiewBrowseWindow(wxFrame):
    """ Frame based widget to browse directories and create thumbnails """

    def __init__(self, parent, directory=".", name="Browse", pos=wxDefaultPosition, size=wxDefaultSize,
                 style=wxDEFAULT_FRAME_STYLE):

        wxFrame.__init__(self, parent, wxNewId(), name, pos, size, style)

        #Main splitter for the window
        #Split the window into left and right frames
        self._splitter1=wxSplitterWindow(self, wxNewId())
        self.sizer = wxBoxSizer(wxVERTICAL)

        #Left panel for displaying images
        self._imgpanel = wxPanel(self._splitter1, wxNewId())
        self._imgpanel.SetSize(wxSize(465, 700))
        self._imgpanel.SetBackgroundColour(wxColor(150,150,150))
        self._splitter1.Initialize(self._imgpanel)

        #Right panel for browsing directories and showing
        #thumbnails
        self._rightpanel=wxPanel(self._splitter1, wxNewId())
        self._splitter1.SplitVertically(self._imgpanel, self._rightpanel,300)
        
        #Create another splitter
        #self._splitter2=wxSplitterWindow(self._rightpanel, wxNewId())
        #Split right panel into browsepanel and thumbpanel

        #self._thumbpanel=wxPanel(self._splitter2, wxNewId())
        #self._thumbpanel.SetSize(wxSize(300,500))
        #self._thumbpanel.SetBackgroundColour(wxColor(160,160,160))
        #self._splitter2.Initialize(self._thumbpanel)

        #self._browsepanel=wxPanel(self._splitter2, wxNewId())
        #self._splitter2.SplitHorizontally(self._browsepanel, self._thumbpanel)

        #self.subbox = wxBoxSizer(wxHORIZONTAL)

        #self.subbox.Add(self._imgpanel, 1, wxGROW)
        #self.subbox.Add(self._rightpanel, 1, wxGROW)


        #self.subbox = wxBoxSizer(wxHORIZONTAL)

        #self.subbox.Add(self._imgpanel, 1, wxGROW)
        #self.subbox.Add(self._rightpanel, 1, wxGROW)

        
        self._dir = str(directory)
        #self._imgpanel.SetAutoLayout(true)
        #self._browsepanel.SetAutoLayout(true)
        #self._thumbpanel.SetAutoLayout(true)
        
        EVT_LEFT_DOWN(self._imgpanel, self.PaintOnParent)
        
        self.lastx = 20
        self.lasty = 20
        self.maxy = 20
        self.row = 0
        self._imglist = []
        self.AddMenus()

        self.CreateStatusBar()
        self.Show(true)

    def AddMenus(self):
        self.MainMenu = wxMenuBar()
        self.AddFileMenu(self.MainMenu)
        self.SetMenuBar(self.MainMenu)
        pass

    def AddFileMenu(self, menu):
        fileMenu = wxMenu()
        self.AddMenuItem(fileMenu, 'Change &Dir \tCtrl-D', 'Change Directory', self.ChangeDir)
        self.AddMenuItem(fileMenu, 'Save Page As &Image \t(Alt-I)', 'Save page as an image', self.PageToImage)
        self.AddMenuItem(fileMenu, 'Save Thumbnails', 'Saves thumbnails of all images', self.SaveThumbs)
        self.AddMenuItem(fileMenu, 'Reload', 'Reloads current directory', self.Update)
        self.AddMenuItem(fileMenu, '&Close \t(Esc)', 'Close the Window', self.OnExit)
        menu.Append(fileMenu, '&File')

    def AddMenuItem(self, menu, itemText, itemDescription, itemHandler):
        menuId = wxNewId()
        menu.Append(menuId, itemText, itemDescription)
        EVT_MENU(self, menuId, itemHandler)
        return menuId

    def SaveThumbs(self, event):
        "Saves each image as a thumbnail"

        count = 0
        for x in range(0, len(self._imgdata)):
            data = self._imgdata[x]
            # image is first element, access it
            img = data[0]
            # filename is last element, acccess it
            f = data[3]
            list = os.path.splitext(f)
            base = list[0]  # test.jpeg -> test, .jpg
            newfile = list[0] + '_t' + list[1]  # newfile = test_t.jpg
            f = newfile
            size = _options.GetThumbnailSizeTuple(wxSize(img.GetWidth(),img.GetHeight()))
            pimg = wxImgToPILImg(img)
            pimg.thumbnail(size)
            msg = 'Saving file ' + f + ' ...'
            self.SetStatusText(msg)
            try:
                pimg.save(f)
                count = count + 1
            except:
                msg = 'Error saving file ' + f + ' !'
                self.SetStatusText(msg)
                msg = 'Saved ' + str(count) + ' thumbnails.'

            self.SetStatusText(msg)

    def Update(self,event):
        "Updates and reloads images in current directory"

        dir2 = self._dir
        files = os.listdir(dir2)
        self._imglist = []
        for x in range(0, len(files)):
            imgfile  = os.path.join(dir2, files[x])
            if self.GetParent().FileExtnOk(imgfile):
                img = self.GetParent().GetImage(imgfile)
                data = (img,imgfile)
                self._imglist.append(data)

        self.SetStatusText("Working...")
        self.AddImagesToPage()
        self._imgpanel.Fit()
        self.Fit()
        self.DoRefresh()

    def ChangeDir(self,event):
        "Browse a directory"

        dlg = wxDirDialog(self, "Choose a dir", str(self._dir))
        result = dlg.ShowModal()
        if result == wxID_OK:
            self._dir = str(dlg.GetPath())
        else:
            return

        self.Update(event)

    def PageToImage(self,event):
        "Saves the thumbnails as a single image"

        context = wxClientDC( self ) 
        memory = wxMemoryDC( ) 
        x,y = self.GetClientSizeTuple() 
        bitmap = wxEmptyBitmap( x,y, -1 ) 
        memory.SelectObject( bitmap ) 
        memory.Blit( 0,0, x,y, context, 0,0) 
        memory.SelectObject( wxNullBitmap)
        f = self._dir + "/panel.bmp"
        try:
            bitmap.SaveFile( f, wxBITMAP_TYPE_BMP )
        except:
            self.GetParent().GetStatusBar().SetStatusText('Error in saving panel as image')

        self.GetParent().GetStatusBar().SetStatusText('Saved file ' + os.path.abspath(f))


    def SaveHtmlThumbs(self,event):
        "Saves the thumbnails as an HTML page(not implemented)"
        pass

    def OnExit(self, event):
        self.GetParent().Display(true)
        self.Destroy()

    def PaintOnParent(self, event):
        "Send the current thumbnail to the main window"

        if event is not None:
            pt = event.GetPosition()
            tup = self.FindImage(pt)
            if tup is not None and tup[0] is not None:
                #locate the image
                parent = self.GetParent()
                # tup[1] is the filename
                parent.SetImage(tup[1])
                parent.DrawImage()
                parent.SetFocus()

    def FindImage(self,posn):
        "Locate an image based on position"

        x = posn.x
        y = posn.y

        for index in range(0, len(self._imgdata)):
            tup = self._imgdata[index]
            init = tup[1]
            size = tup[2]
            img = tup[0]
            file = tup[3]
            # get extent
            w = size.GetWidth()
            h = size.GetHeight()
            # check whether point falls within extent
            X = init.x + w
            Y = init.y + h + 20 # to account for 20 pixels gap between images in y directopn
            if x > init.x and x < X and y > init.y and y < Y:
                return (img,file)

        return None

    def AddImageToPage(self, img, filename):
        "Adds an image to the thumbnail page"

        # we have a wxImage
        pimg = wxImgToPILImg(img)
        # aspect ratio
        h = img.GetHeight()
        w = img.GetWidth()
        # we fix 128 as the width and change the height
        tw = 80
        th = tw*h/w
        pimg.thumbnail((tw,th))
        newimg = PilImgTowxImg(pimg)
        # calculate position
        bmp = wxStaticBitmap(self._imgpanel, wxNewId(), newimg.ConvertToBitmap(), wxPoint(self.lastx, self.lasty),wxDefaultSize, wxDOUBLE_BORDER)
        tup = (img, wxPoint(self.lastx,self.lasty),wxSize(tw,th), filename)
        self._imgdata.append(tup)

        # set position for next image
        self.lastx = self.lastx + tw + 20
        self.row += 1
        if th > self.maxy:
            self.maxy = th
            # exceeding row size, create new row
        if self.row == MAXROWS: 
            self.lastx = 20
            self.lasty = self.lasty + self.maxy + 20
            self.row = 0

    def AddImagesToPage(self):
        "Adds a list of images to page"
        # add a list of images
        # reset posns
        self.lastx = 20
        self.lasty = 20
        self.maxy = 0
        self.row = 0
        self._imgdata = []
        # clear window
        self._imgpanel.DestroyChildren()
        self._imgpanel.Clear()
        wxBusyInfo("Please wait.Creating thumbnails...")
        for img in range(0, len(self._imglist)):
            data = self._imglist[img]
            # first elem is image
            # second elem is filename
            self.AddImageToPage(data[0], data[1])

        self.SetStatusText('Done.')

    def DoRefresh(self):
        self.Refresh()
        self.SetStatusText('Click below any thumbnail to bring it into the main window')


class pyWiewCreateImageDialog(wxDialog):
    """Dialog widget for creating new image"""

    def __init__(self,parent):
        wxDialog.__init__(self, parent, wxNewId(), "Create New Image", wxDefaultPosition,
                          wxSize(365,300), wxDEFAULT_DIALOG_STYLE)

        self.panel = wxPanel(self, wxNewId())
        sbox1 = wxStaticBox(self.panel, wxNewId(), "Define new image:", wxPoint(12, 10), wxSize(164, 220))
        sbox2 = wxStaticBox(self.panel, wxNewId(), "Colors:", wxPoint(172, 10), wxSize(172, 220))
        wxStaticText(self.panel, wxNewId(), "Image width: ", wxPoint(22, 38))
        self.txt1 = wxTextCtrl(sbox1, wxNewId(), "200", wxPoint(80, 25), wxSize(50, 25))
        wxStaticText(self.panel, wxNewId(), " pixel ", wxPoint(142, 38))
        wxStaticText(self.panel, wxNewId(), "Image height: ", wxPoint(22, 68))
        self.txt2 = wxTextCtrl(sbox1, wxNewId(), "200", wxPoint(80, 55), wxSize(50, 25))
        wxStaticText(self.panel, wxNewId(), "pixel ", wxPoint(144, 68))
        wxStaticText(self.panel, wxNewId(), "X-DPI: ", wxPoint(50, 100))
        self.txt3 = wxTextCtrl(sbox1, wxNewId(), "96", wxPoint(80, 85), wxSize(50, 25))
        wxStaticText(self.panel, wxNewId(), "Y-DPI: ", wxPoint(50, 135))
        self.txt4 = wxTextCtrl(sbox1, wxNewId(), "96", wxPoint(80, 120), wxSize(50, 25))
        #color bitmap
        self._bgcol = wxNamedColour("white")
        img=ImgOfSize((130, 20), self._bgcol)
        self.bmap = wxStaticBitmap(self.panel, wxNewId(), img.ConvertToBitmap(),wxPoint(30, 165), wxSize(130,20), wxSUNKEN_BORDER)

        #color chooser button
        id=wxNewId()
        self.btn=wxButton(self.panel, id, "Background Color", wxPoint(30, 190), wxSize(130, 30))
        EVT_BUTTON(self, id, self.SetBackColor)
        #Other half of panel
        msg='&Black/White' + '            ' + '[1BPP]'
        self.bt1=wxRadioButton(self.panel, wxNewId(), msg, wxPoint(190, 40))
        msg='&16 Colors'   + '                 ' + '[4BPP]'
        self.bt2=wxRadioButton(self.panel, wxNewId(), msg, wxPoint(190, 68))
        msg='&256 Colors' +  '               ' + '[8BPP]'
        self.bt3=wxRadioButton(self.panel, wxNewId(), msg, wxPoint(190, 96))
        msg='&1&6.7 Million colors' + '  ' +     '[24BPP]'
        self.bt4=wxRadioButton(self.panel, wxNewId(), msg, wxPoint(190, 124))
        self.bt5 = wxCheckBox(self.panel, wxNewId(), "Grayscale", wxPoint(190, 180))
        self.bt4.SetValue(true)

        #ok/cancel buttons
        wxButton(self.panel, wxID_OK,'OK', wxPoint(80, 245))
        wxButton(self.panel, wxID_CANCEL,'Cancel', wxPoint(230, 245))
        self.panel.Fit()
        self.Fit()

    def SetBackColor(self,event):
        """ Dialog box for setting bgrnd color of image """

        cdlg=wxColourDialog(self)
        result = cdlg.ShowModal()
        if result == wxID_CANCEL:
            return

        #get data
        coldat = cdlg.GetColourData()
        self._bgcol=coldat.GetColour()
        img=ImgOfSize((130,20), self._bgcol)
        self.bmap.SetBitmap(img.ConvertToBitmap())#img.ConvertToBitmap())
        cdlg.Destroy()

    def GetXdpi(self):
        return self.txt3.GetValue()

    def GetImgWidth(self):
        return self.txt1.GetValue()

    def GetImgHeight(self):
        return self.txt2.GetValue()

    def GetYdpi(self):
        return self.txt4.GetValue()

    def GetBitdepth(self):

        if self.bt1.GetValue():
            return 1
        elif self.bt2.GetValue():
            return 4
        elif self.bt3.GetValue():
            return 8
        else:
            return 24

    def GetGrayScale(self):
        return self.bt5.GetValue()

    def GetBackgroundColor(self):
        return self._bgcol


